@@ -11,9 +11,9 @@
 #include "rb_gsl_config.h"
 
 #include <gsl/gsl_blas.h>
-#include "rb_gsl_common.h"
 #include "rb_gsl_complex.h"
 #include "rb_gsl_array.h"
+#include "rb_gsl_common.h"
 
 static int get_vector1(int argc, VALUE *argv, VALUE obj, gsl_vector **x);
 static int get_vector_complex1(int argc, VALUE *argv, VALUE obj, gsl_vector_complex **x);
@@ -31,10 +31,10 @@ static int get_vector1(int argc, VALUE *argv, VALUE obj, gsl_vector **x)
   case T_OBJECT:
     if (argc != 1) rb_raise(rb_eArgError, "wrong number of arguments (%d for 1)",
 			    argc);
-    Data_Get_Vector(argv[0], *x);
+    Data_Get_Vector(argv[0], (*x));
     break;
   default:
-    Data_Get_Struct(obj, gsl_vector, *x);
+    Data_Get_Vector(obj, (*x));
     flag = 1;
     break;
   }
@@ -51,10 +51,10 @@ static int get_vector_complex1(int argc, VALUE *argv, VALUE obj, gsl_vector_comp
     if (argc != 1) rb_raise(rb_eArgError, "wrong number of arguments (%d for 1)",
 			    argc);
     CHECK_VECTOR_COMPLEX(argv[0]);
-    Data_Get_Struct(argv[0], gsl_vector_complex, *x);
+    Data_Get_Struct(argv[0], gsl_vector_complex, (*x));
     break;
   default:
-    Data_Get_Struct(obj, gsl_vector_complex, *x);
+    Data_Get_Struct(obj, gsl_vector_complex, (*x));
     flag = 1;
     break;
   }
@@ -71,14 +71,14 @@ static int get_vector2(int argc, VALUE *argv, VALUE obj,
   case T_OBJECT:
     if (argc != 2) rb_raise(rb_eArgError, "wrong number of arguments (%d for 2)",
 			    argc);
-    Data_Get_Vector(argv[0], *x);
-    Data_Get_Vector(argv[1], *y);
+    Data_Get_Vector(argv[0], (*x));
+    Data_Get_Vector(argv[1], (*y));
     break;
   default:
     if (argc != 1) rb_raise(rb_eArgError, "wrong number of arguments (%d for 1)",
 			    argc);
-    Data_Get_Struct(obj, gsl_vector, *x);
-    Data_Get_Vector(argv[0], *y);
+    Data_Get_Vector(obj, (*x));
+    Data_Get_Vector(argv[0], (*y));
     flag = 1;
     break;
   }
@@ -98,15 +98,15 @@ static int get_vector_complex2(int argc, VALUE *argv, VALUE obj,
 			    argc);
     CHECK_VECTOR_COMPLEX(argv[0]);
     CHECK_VECTOR_COMPLEX(argv[1]);
-    Data_Get_Struct(argv[0], gsl_vector_complex, *x);
-    Data_Get_Struct(argv[1], gsl_vector_complex, *y);
+    Data_Get_Struct(argv[0], gsl_vector_complex, (*x));
+    Data_Get_Struct(argv[1], gsl_vector_complex, (*y));
     break;
   default:
     if (argc != 1) rb_raise(rb_eArgError, "wrong number of arguments (%d for 1)",
 			    argc);
     CHECK_VECTOR_COMPLEX(argv[0]);
-    Data_Get_Struct(obj, gsl_vector_complex, *x);
-    Data_Get_Struct(argv[0], gsl_vector_complex, *y);
+    Data_Get_Struct(obj, gsl_vector_complex, (*x));
+    Data_Get_Struct(argv[0], gsl_vector_complex, (*y));
     flag = 1;
     break;
   }
@@ -598,7 +598,7 @@ void Init_gsl_blas1(VALUE module)
   rb_define_module_function(module, "ddot", rb_gsl_blas_ddot, -1);
   rb_define_method(cgsl_vector, "blas_ddot", rb_gsl_blas_ddot, -1);
   rb_define_alias(cgsl_vector, "ddot", "blas_ddot");
-  rb_define_alias(cgsl_vector, "dot", "blas_ddot");
+  /*  rb_define_alias(cgsl_vector, "dot", "blas_ddot");*/
 
   rb_define_module_function(module, "zdotu", rb_gsl_blas_zdotu, -1);
   rb_define_method(cgsl_vector_complex, "blas_zdotu", rb_gsl_blas_zdotu, -1);
@@ -708,12 +708,11 @@ void Init_gsl_blas1(VALUE module)
   rb_define_module_function(module, "zscal!", rb_gsl_blas_zscal, -1);
   rb_define_method(cgsl_vector_complex, "blas_zscal!", rb_gsl_blas_zscal, -1);
   rb_define_alias(cgsl_vector_complex, "zscal!", "blas_zscal!");
-  rb_define_alias(cgsl_vector_complex, "scal!", "blas_zscal!");
 
-  rb_define_module_function(module, "zscal", rb_gsl_blas_zscal2, -1);
-  rb_define_method(cgsl_vector_complex, "blas_zscal", rb_gsl_blas_zscal2, -1);
-  rb_define_alias(cgsl_vector_complex, "zscal", "blas_zscal");
-  rb_define_alias(cgsl_vector_complex, "scal", "blas_zscal");
+  rb_define_module_function(module, "zscal2", rb_gsl_blas_zscal2, -1);
+  rb_define_method(cgsl_vector_complex, "blas_zscal2", rb_gsl_blas_zscal2, -1);
+  rb_define_alias(cgsl_vector_complex, "zscal2", "blas_zscal2");
+  rb_define_alias(cgsl_vector_complex, "scal2", "blas_zscal2");
 
   rb_define_module_function(module, "drot!", rb_gsl_blas_drot, 4);
   rb_define_module_function(module, "drot", rb_gsl_blas_drot2, 4);
