@@ -39,6 +39,7 @@ static void rb_gsl_const_mks(VALUE module)
 		  rb_float_new(GSL_CONST_MKS_ELECTRON_VOLT));
   rb_define_const(module, "MASS_ELECTRON", 
 		  rb_float_new(GSL_CONST_MKS_MASS_ELECTRON));
+  rb_define_const(module, "MASS_MUON", rb_float_new(GSL_CONST_MKS_MASS_MUON));
   rb_define_const(module, "MASS_PROTON", rb_float_new(GSL_CONST_MKS_MASS_PROTON));
   rb_define_const(module, "MASS_NEUTRON", rb_float_new(GSL_CONST_MKS_MASS_NEUTRON));
   rb_define_const(module, "RYDBERG", rb_float_new(GSL_CONST_MKS_RYDBERG));
@@ -175,6 +176,7 @@ static void rb_gsl_const_cgs(VALUE module)
 		  rb_float_new(GSL_CONST_CGS_ELECTRON_VOLT));
   rb_define_const(module, "MASS_ELECTRON", 
 		  rb_float_new(GSL_CONST_CGS_MASS_ELECTRON));
+  rb_define_const(module, "MASS_MUON", rb_float_new(GSL_CONST_CGS_MASS_MUON));
   rb_define_const(module, "MASS_PROTON", 
 		  rb_float_new(GSL_CONST_CGS_MASS_PROTON));
   rb_define_const(module, "MASS_NEUTRON", rb_float_new(GSL_CONST_CGS_MASS_NEUTRON));
@@ -359,6 +361,7 @@ static void rb_gsl_const_mks(VALUE module)
 		  rb_float_new(GSL_CONST_MKSA_ELECTRON_VOLT));
   rb_define_const(module, "MASS_ELECTRON", 
 		  rb_float_new(GSL_CONST_MKSA_MASS_ELECTRON));
+  rb_define_const(module, "MASS_MUON", rb_float_new(GSL_CONST_MKSA_MASS_MUON));
   rb_define_const(module, "MASS_PROTON", rb_float_new(GSL_CONST_MKSA_MASS_PROTON));
   rb_define_const(module, "MASS_NEUTRON", rb_float_new(GSL_CONST_MKSA_MASS_NEUTRON));
   rb_define_const(module, "RYDBERG", rb_float_new(GSL_CONST_MKSA_RYDBERG));
@@ -496,6 +499,7 @@ static void rb_gsl_const_cgs(VALUE module)
 		  rb_float_new(GSL_CONST_CGSM_ELECTRON_VOLT));
   rb_define_const(module, "MASS_ELECTRON", 
 		  rb_float_new(GSL_CONST_CGSM_MASS_ELECTRON));
+  rb_define_const(module, "MASS_MUON", rb_float_new(GSL_CONST_CGSM_MASS_MUON));
   rb_define_const(module, "MASS_PROTON", rb_float_new(GSL_CONST_CGSM_MASS_PROTON));
   rb_define_const(module, "MASS_NEUTRON", rb_float_new(GSL_CONST_CGSM_MASS_NEUTRON));
   rb_define_const(module, "RYDBERG", rb_float_new(GSL_CONST_CGSM_RYDBERG));
