@@ -1253,11 +1253,11 @@ static VALUE rb_gsl_graph_graph(int argc, VALUE *argv, VALUE obj)
   if (fp == NULL) rb_raise(rb_eIOError, "GNU graph not found.");
   for (i = 0; i < size; i++) {
     if (h) 
-      fprintf(fp, "%e %e\n%e %e\n", h->range[i], h->bin[i], h->range[i+1], h->bin[i]);
+      fprintf(fp, "%g %g\n%g %g\n", h->range[i], h->bin[i], h->range[i+1], h->bin[i]);
     else if (y == NULL) 
-      fprintf(fp, "%d %e\n", (int) i, gsl_vector_get(x, i));
+      fprintf(fp, "%d %g\n", (int) i, gsl_vector_get(x, i));
     else
-      fprintf(fp, "%e %e\n", gsl_vector_get(x, i), gsl_vector_get(y, i));
+      fprintf(fp, "%g %g\n", gsl_vector_get(x, i), gsl_vector_get(y, i));
   }
   fflush(fp);
   fclose(fp);
@@ -1321,14 +1321,14 @@ static VALUE rb_gsl_graph_step(int argc, VALUE *argv, VALUE obj)
   if (fp == NULL) rb_raise(rb_eIOError, "GNU graph not found.");
   for (i = 0; i < size; i++) {
     if (y == NULL) {
-      fprintf(fp, "%d %e\n%d %e\n", (int) i, gsl_vector_get(x, i),
+      fprintf(fp, "%d %g\n%d %g\n", (int) i, gsl_vector_get(x, i),
 	      (int) (i+1), gsl_vector_get(x, i));
     } else {
       if (i != size-1)
-	fprintf(fp, "%e %e\n%e %e\n", gsl_vector_get(x, i), gsl_vector_get(y, i),
+	fprintf(fp, "%g %g\n%g %g\n", gsl_vector_get(x, i), gsl_vector_get(y, i),
 		gsl_vector_get(x, i+1), gsl_vector_get(y, i));
       else 	
-	fprintf(fp, "%e %e\n%e %e", gsl_vector_get(x, i), gsl_vector_get(y, i),
+	fprintf(fp, "%g %g\n%g %g", gsl_vector_get(x, i), gsl_vector_get(y, i),
 		2.0*gsl_vector_get(x, i)-gsl_vector_get(x, i-1), gsl_vector_get(y, i));
     }
   }
@@ -1346,7 +1346,7 @@ void Init_gsl_graph(VALUE module)
 {
   VALUE cgsl_graph;
 
-  cgsl_graph = rb_define_class_under(module, "Graph", rb_cObject);
+  cgsl_graph = rb_define_class_under(module, "Graph", cGSL_Object);
 
   rb_define_singleton_method(cgsl_graph, "new", rb_gsl_graph_new, -1);
   rb_define_singleton_method(cgsl_graph, "alloc", rb_gsl_graph_new, -1);
