@@ -16,6 +16,7 @@
 
 VALUE cgsl_histogram2d;
 VALUE cgsl_histogram2d_view;
+static VALUE cgsl_histogram2d_integ;
 
 #ifdef GSL_0_9_4_LATER
 static VALUE rb_gsl_histogram2d_alloc_uniform(int argc, VALUE *argv, VALUE klass);
@@ -135,14 +136,28 @@ static VALUE rb_gsl_histogram2d_set_ranges(int argc, VALUE *argv, VALUE obj)
   return obj;
 }
 
-static VALUE rb_gsl_histogram2d_set_ranges_uniform(VALUE obj, VALUE xmin, VALUE xmax, VALUE ymin, VALUE ymax)
+static VALUE rb_gsl_histogram2d_set_ranges_uniform(int argc, VALUE *argv, VALUE obj)
 {
   gsl_histogram2d *h = NULL;
-  Need_Float(xmin); Need_Float(xmax);
-  Need_Float(ymin); Need_Float(ymax);
+  double xmin, xmax, ymin, ymax;
+  switch (argc) {
+  case 2:
+    Check_Type(argv[0], T_ARRAY); Check_Type(argv[1], T_ARRAY);
+    xmin = NUM2DBL(rb_ary_entry(argv[0], 0));
+    xmax = NUM2DBL(rb_ary_entry(argv[0], 1));
+    ymin = NUM2DBL(rb_ary_entry(argv[1], 0));
+    ymax = NUM2DBL(rb_ary_entry(argv[1], 1));
+    break;
+  case 4:
+    xmin = NUM2DBL(argv[0]); xmax = NUM2DBL(argv[1]);
+    ymin = NUM2DBL(argv[2]); ymax = NUM2DBL(argv[3]);
+    break;
+  default:
+    rb_raise(rb_eArgError, "wrong number of arguments (%d for 2 or 4)", argc);
+    break;
+  }
   Data_Get_Struct(obj, gsl_histogram2d, h);
-  gsl_histogram2d_set_ranges_uniform(h, NUM2DBL(xmin), NUM2DBL(xmax),
-				     NUM2DBL(ymin), NUM2DBL(ymax));
+  gsl_histogram2d_set_ranges_uniform(h, xmin, xmax, ymin, ymax);
   return obj;
 }
 
@@ -168,22 +183,32 @@ static VALUE rb_gsl_histogram2d_memcpy(VALUE obj, VALUE vhdest, VALUE vhsrc)
 static VALUE rb_gsl_histogram2d_accumulate(int argc, VALUE *argv, VALUE obj)
 {
   gsl_histogram2d *h = NULL;
-  double x, y, weight = 1;
+  gsl_vector *vx, *vy;
+  size_t n, i;
+  double weight = 1;
   switch (argc) {
   case 3:
     Need_Float(argv[2]);
     weight = NUM2DBL(argv[2]);
-    /* no break */
+    break;
   case 2:
-    Need_Float(argv[0]); Need_Float(argv[1]); 
-    x = NUM2DBL(argv[0]); y = NUM2DBL(argv[1]);
+    weight = 1;
     break;
   default:
     rb_raise(rb_eArgError, "wrong number of arguments (%d for 2 or 3)", argc);
     break;
   }
   Data_Get_Struct(obj, gsl_histogram2d, h);
-  gsl_histogram2d_accumulate(h, x, y, weight);
+  if (VECTOR_P(argv[0]) && VECTOR_P(argv[1])) {
+    Data_Get_Struct(argv[0], gsl_vector, vx);
+    Data_Get_Struct(argv[1], gsl_vector, vy);
+    n = (size_t) GSL_MIN_INT((int) vx->size, (int) vy->size);
+    for (i = 0; i < n; i++)
+      gsl_histogram2d_accumulate(h, gsl_vector_get(vx, i), gsl_vector_get(vy, i),
+				 weight);
+  } else {
+    gsl_histogram2d_accumulate(h, NUM2DBL(argv[0]), NUM2DBL(argv[1]), weight);
+  }
   return obj;
 }
 
@@ -226,18 +251,27 @@ static VALUE rb_gsl_histogram2d_get(int argc, VALUE *argv, VALUE obj)
     break;
   case 1:
     Data_Get_Struct(obj, gsl_histogram2d, h2);
-    if (TYPE(argv[0]) == T_ARRAY)
+    switch (TYPE(argv[0])) {
+    case T_ARRAY:
       return rb_float_new(gsl_histogram2d_get(h2, FIX2INT(rb_ary_entry(argv[0], 0)),
 					      FIX2INT(rb_ary_entry(argv[0], 1))));
-    CHECK_FIXNUM(argv[0]); 
-    i = (size_t) FIX2INT(argv[0]);
-    if (i >= h2->ny)
-      rb_raise(rb_eIndexError, "wrong index");
-    h1 = ALLOC(mygsl_histogram2d_view);
-    h1->h.n = h2->ny;
-    h1->h.range = h2->yrange;
-    h1->h.bin = h2->bin + i*h2->ny;
-    return Data_Wrap_Struct(cgsl_histogram2d_view, 0, free, h1);
+      break;
+    case T_FIXNUM:
+      CHECK_FIXNUM(argv[0]); 
+      i = (size_t) FIX2INT(argv[0]);
+      if (i >= h2->ny)
+	rb_raise(rb_eIndexError, "wrong index");
+      h1 = ALLOC(mygsl_histogram2d_view);
+      h1->h.n = h2->ny;
+      h1->h.range = h2->yrange;
+      h1->h.bin = h2->bin + i*h2->ny;
+      return Data_Wrap_Struct(cgsl_histogram2d_view, 0, free, h1);
+      break;
+    default:
+      rb_raise(rb_eTypeError, "wrong argument type %s (Array or Fixnum expected)",
+	       rb_class2name(CLASS_OF(argv[0])));
+      break;
+    }
     break;
   default:
     rb_raise(rb_eArgError, "wrong number of arguments (%d for 2 or 1)",argc);
@@ -754,14 +788,162 @@ static VALUE rb_gsl_histogram2d_yproject(int argc, VALUE *argv, VALUE obj)
   return Data_Wrap_Struct(cgsl_histogram, 0, gsl_histogram_free, h);
 }
 
+static int mygsl_histogram2d_fread2(FILE * stream, gsl_histogram2d * h)
+{  
+  double xmin, xmax, ymin, ymax;
+  int status;
+  status = gsl_block_raw_fread(stream, &xmin, 1, 1);
+  if (status)    return status;  
+  status = gsl_block_raw_fread(stream, &xmax, 1, 1);
+  if (status)    return status;  
+  status = gsl_block_raw_fread(stream, &ymin, 1, 1);
+  if (status)    return status;  
+  status = gsl_block_raw_fread(stream, &ymax, 1, 1);
+  if (status)    return status;  
+  gsl_histogram2d_set_ranges_uniform(h, xmin, xmax, ymin, ymax);
+  status = gsl_block_raw_fread (stream, h->bin, h->nx*h->ny, 1);  
+  if (status)    return status;  
+  return status;
+}
+
+static int mygsl_histogram2d_fwrite2(FILE * stream, const gsl_histogram2d * h)
+{  
+  int status;
+  status = gsl_block_raw_fwrite (stream, h->xrange, 1, 1);
+  if (status)    return status;  
+  status = gsl_block_raw_fwrite (stream, h->xrange+h->nx, 1, 1);
+  if (status)    return status;  
+  status = gsl_block_raw_fwrite (stream, h->yrange, 1, 1);
+  if (status)    return status;  
+  status = gsl_block_raw_fwrite (stream, h->yrange+h->ny, 1, 1);
+  if (status)    return status;  
+  status = gsl_block_raw_fwrite (stream, h->bin, h->nx*h->ny, 1);  
+  return status;
+}
+
+static VALUE rb_gsl_histogram2d_fwrite2(VALUE obj, VALUE io)
+{
+  gsl_histogram2d *h = NULL;
+  FILE *f;
+  int status, flag = 0;
+  Data_Get_Struct(obj, gsl_histogram2d, h);
+  f = rb_gsl_open_writefile(io, &flag);
+  status = mygsl_histogram2d_fwrite2(f, h);
+  if (flag == 1) fclose(f);
+  return INT2FIX(status);
+}
+
+static VALUE rb_gsl_histogram2d_fread2(VALUE obj, VALUE io)
+{
+  gsl_histogram2d *h = NULL;
+  FILE *f;
+  int status, flag = 0;
+  Data_Get_Struct(obj, gsl_histogram2d, h);
+  f = rb_gsl_open_readfile(io, &flag);
+  status = mygsl_histogram2d_fread2(f, h);
+  if (flag == 1) fclose(f);
+  return INT2FIX(status);
+}
+
+static gsl_histogram2d* mygsl_histogram2d_calloc_integrate(const gsl_histogram2d *h,
+							   int flag)
+{
+  gsl_histogram2d *hi;
+  size_t i, j, k;
+  size_t nx = h->nx, ny = h->ny, n = h->nx*h->ny;
+  hi = gsl_histogram2d_calloc(nx, ny);
+  gsl_histogram2d_set_ranges(hi, h->xrange, nx+1, h->yrange, ny+1);
+  if (flag == -1) {
+    hi->bin[n-1] = h->bin[n-1];
+    i = nx - 1;
+    for (j = ny-2, k = 0; j >= 0; j--, k++) {
+      hi->bin[n-1-k] = gsl_histogram2d_get(hi, i, j+1) + gsl_histogram2d_get(h, i, j);
+      if (j == 0) break;
+    }
+    j = ny - 1;
+    for (i = nx-2; i >= 0; i--) {
+      hi->bin[i*ny + j] = gsl_histogram2d_get(hi, i+1, j) + gsl_histogram2d_get(h, i, j);
+      if (i == 0) break;
+    }
+    for (i = nx-2; i >= 0; i--) {
+      for (j = ny-2; j >= 0; j--) {
+	hi->bin[i*ny+j] = ((gsl_histogram2d_get(hi, i+1, j) 
+			    + gsl_histogram2d_get(hi, i, j+1))
+			   - gsl_histogram2d_get(hi, i+1, j+1))
+	  + gsl_histogram2d_get(h, i, j);
+	if (j == 0) break;
+      }
+      if (i == 0) break;
+    }
+  } else {
+    hi->bin[0] = h->bin[0];
+    for (j = 1; j < ny; j++) hi->bin[j] = gsl_histogram2d_get(hi, 0, j-1) 
+			       + gsl_histogram2d_get(h, 0, j);
+    for (i = 1; i < nx; i++) hi->bin[i*ny] = gsl_histogram2d_get(hi, i-1, 0)
+			       + gsl_histogram2d_get(h, i, 0);
+    for (i = 1; i < nx; i++) {
+      for (j = 1; j < ny; j++) {
+	hi->bin[i*ny+j] = ((gsl_histogram2d_get(hi, i-1, j) 
+			    + gsl_histogram2d_get(hi, i, j-1))
+			   - gsl_histogram2d_get(hi, i-1, j-1))
+	  + gsl_histogram2d_get(h, i, j);
+      }
+    }
+  }
+  return hi;
+}
+
+static VALUE rb_gsl_histogram2d_integrate(int argc, VALUE *argv, VALUE obj)
+{
+  gsl_histogram2d *h = NULL, *hi = NULL;
+  int flag;
+  Data_Get_Struct(obj, gsl_histogram2d, h);
+  switch (argc) {
+  case 0:
+    flag = 1;
+    break;
+  case 1:
+    flag = FIX2INT(argv[0]);
+    break;
+  default:
+    rb_raise(rb_eArgError, "wrong number of arguments (%d for 0 or 1)", argc);
+    break;
+  }
+  hi = mygsl_histogram2d_calloc_integrate(h, flag);
+  return Data_Wrap_Struct(cgsl_histogram2d_integ, 0, gsl_histogram2d_free, hi);
+}
+
+static VALUE rb_gsl_histogram2d_normalize_bang(VALUE obj)
+{
+  gsl_histogram2d *h = NULL;
+  double scale;
+  Data_Get_Struct(obj, gsl_histogram2d, h);
+  if (CLASS_OF(obj) == cgsl_histogram2d_integ)
+    scale = 1.0/h->bin[h->nx*h->ny-1];
+  else
+    scale = 1.0/gsl_histogram2d_sum(h);
+  gsl_histogram2d_scale(h, scale);
+  return obj;
+}
+
+static VALUE rb_gsl_histogram2d_normalize(VALUE obj)
+{
+  gsl_histogram2d *h = NULL, *hnew = NULL;
+  Data_Get_Struct(obj, gsl_histogram2d, h);
+  hnew = gsl_histogram2d_clone(h);
+  return rb_gsl_histogram2d_normalize_bang(Data_Wrap_Struct(CLASS_OF(obj), 0, gsl_histogram2d_free, hnew));
+}
+
 void Init_gsl_histogram2d(VALUE module)
 {
   VALUE cgsl_histogram2d_pdf;
 
-  cgsl_histogram2d = rb_define_class_under(module, "Histogram2d", rb_cObject);
+  cgsl_histogram2d = rb_define_class_under(module, "Histogram2d", cGSL_Object);
   cgsl_histogram2d_view = rb_define_class_under(cgsl_histogram2d, "View",
 						cgsl_histogram);
 
+  cgsl_histogram2d_integ = rb_define_class_under(cgsl_histogram2d, "Integral", 
+					       cgsl_histogram2d);
 #ifdef GSL_0_9_4_LATER
   rb_define_singleton_method(cgsl_histogram2d, "new", rb_gsl_histogram2d_alloc, -1);
   rb_define_singleton_method(cgsl_histogram2d, "alloc", rb_gsl_histogram2d_alloc, -1);
@@ -779,7 +961,8 @@ void Init_gsl_histogram2d(VALUE module)
   rb_define_method(cgsl_histogram2d, "set_ranges",  
 		   rb_gsl_histogram2d_set_ranges, -1);
   rb_define_method(cgsl_histogram2d, "set_ranges_uniform", 
-		   rb_gsl_histogram2d_set_ranges_uniform, 4);
+		   rb_gsl_histogram2d_set_ranges_uniform, -1);
+
   rb_define_singleton_method(cgsl_histogram2d, "memcpy",  
 			     rb_gsl_histogram2d_memcpy, 2);
   rb_define_method(cgsl_histogram2d, "clone",  rb_gsl_histogram2d_clone, 0);
@@ -839,10 +1022,12 @@ void Init_gsl_histogram2d(VALUE module)
 
   rb_define_method(cgsl_histogram2d, "fwrite",  rb_gsl_histogram2d_fwrite, 1);
   rb_define_method(cgsl_histogram2d, "fread",  rb_gsl_histogram2d_fread, 1);
+  rb_define_method(cgsl_histogram2d, "fwrite2",  rb_gsl_histogram2d_fwrite2, 1);
+  rb_define_method(cgsl_histogram2d, "fread2",  rb_gsl_histogram2d_fread2, 1);
   rb_define_method(cgsl_histogram2d, "fprintf",  rb_gsl_histogram2d_fprintf, -1);
   rb_define_method(cgsl_histogram2d, "fscanf",  rb_gsl_histogram2d_fscanf, 3);
 
-  cgsl_histogram2d_pdf = rb_define_class_under(cgsl_histogram2d, "Pdf", rb_cObject);
+  cgsl_histogram2d_pdf = rb_define_class_under(cgsl_histogram2d, "Pdf", cGSL_Object);
 #ifdef GSL_0_9_4_LATER
   rb_define_singleton_method(cgsl_histogram2d_pdf, "new", 
 			     rb_gsl_histogram2d_pdf_alloc, 2);
@@ -865,6 +1050,12 @@ void Init_gsl_histogram2d(VALUE module)
   rb_define_method(cgsl_histogram2d, "reset",  rb_gsl_histogram2d_reset, 0);
   rb_define_method(cgsl_histogram2d, "xproject",  rb_gsl_histogram2d_xproject, -1);
   rb_define_method(cgsl_histogram2d, "yproject",  rb_gsl_histogram2d_yproject, -1);
+
+  rb_define_method(cgsl_histogram2d, "integrate",  rb_gsl_histogram2d_integrate, -1);
+  rb_undef_method(cgsl_histogram2d_integ, "integrate");
+
+  rb_define_method(cgsl_histogram2d, "normalize",  rb_gsl_histogram2d_normalize, 0);
+  rb_define_method(cgsl_histogram2d, "normalize!",  rb_gsl_histogram2d_normalize_bang, 0);
 }
 
 #ifdef HISTOGRAM2D_P
