@@ -18,7 +18,7 @@ static VALUE cgsl_histogram3d_view;
 
 static VALUE rb_gsl_histogram3d_new(int argc, VALUE *argv, VALUE klass)
 {
-  mygsl_histogram3d *h;
+  mygsl_histogram3d *h = NULL;
   gsl_vector *xrange, *yrange, *zrange;
   double xmin, xmax, ymin, ymax, zmin, zmax;
   size_t nx, ny, nz;
@@ -779,7 +779,7 @@ static VALUE rb_gsl_histogram3d_div_scale(VALUE obj, VALUE hh)
 
 void Init_gsl_histogram3d(VALUE module)
 {
-  cgsl_histogram3d = rb_define_class_under(module, "Histogram3d", rb_cObject);
+  cgsl_histogram3d = rb_define_class_under(module, "Histogram3d", cGSL_Object);
   cgsl_histogram3d_view = rb_define_class_under(cgsl_histogram3d, "View",
 						cgsl_histogram2d);
 
