@@ -1048,7 +1048,7 @@ void Init_gsl_integration(VALUE module)
   rb_define_alias(cgsl_function, "qawc", "integration_qawc");
 
   cgsl_integration_qaws_table = rb_define_class_under(mgsl_integ, "QAWS_Table", 
-						      rb_cObject);
+						      cGSL_Object);
   rb_define_singleton_method(cgsl_integration_qaws_table, "alloc",
 			     rb_gsl_integration_qaws_table_alloc, -1);
   rb_define_singleton_method(cgsl_integration_qaws_table, "new",
@@ -1060,12 +1060,11 @@ void Init_gsl_integration(VALUE module)
   rb_define_method(rb_cArray, "to_gsl_integration_qaws_table", 
 		   rb_gsl_ary_to_integration_qaws_table, 0);
   rb_define_alias(rb_cArray, "to_qaws_table", "to_gsl_integration_qaws_table");
-  rb_define_alias(rb_cArray, "to_qaws_table", "to_gsl_integration_qaws_table");
   rb_define_method(cgsl_function, "integration_qaws", rb_gsl_integration_qaws, -1);
   rb_define_alias(cgsl_function, "qaws", "integration_qaws");
 
   cgsl_integration_qawo_table = rb_define_class_under(mgsl_integ, "QAWO_Table", 
-						      rb_cObject);
+						      cGSL_Object);
   rb_define_singleton_method(cgsl_integration_qawo_table, "alloc",
 			     rb_gsl_integration_qawo_table_alloc, -1);
   rb_define_singleton_method(cgsl_integration_qawo_table, "new",
@@ -1084,7 +1083,7 @@ void Init_gsl_integration(VALUE module)
   rb_define_alias(cgsl_function, "qawf", "integration_qawf");
 
   cgsl_integration_workspace = rb_define_class_under(mgsl_integ, 
-						     "Workspace", rb_cObject);
+						     "Workspace", cGSL_Object);
 
   rb_define_singleton_method(cgsl_integration_workspace, "new",
 			     rb_gsl_integration_workspace_alloc, -1);
