@@ -12,6 +12,10 @@
 
 #include "rb_gsl_config.h"
 #include "rb_gsl_math.h"
+#include "rb_gsl_complex.h"
+#ifdef HAVE_NARRAY_H
+#include "narray.h"
+#endif
 
 static void rb_gsl_define_const(VALUE module);
 
@@ -97,12 +101,50 @@ static VALUE rb_gsl_finite2(VALUE obj, VALUE x)
 
 static VALUE rb_gsl_math_eval(double (*func)(const double), VALUE xx);
 
+VALUE rb_gsl_math_complex_eval(gsl_complex (*func)(gsl_complex), VALUE obj)
+{
+  gsl_complex *z, *znew;
+  gsl_vector_complex *v, *vnew;
+  gsl_matrix_complex *m, *mnew;
+  size_t i, j;
+  if (COMPLEX_P(obj)) {
+    Data_Get_Struct(obj, gsl_complex, z);
+    znew = xmalloc(sizeof(gsl_complex));
+    *znew = (*func)(*z);
+    return Data_Wrap_Struct(cgsl_complex, 0, free, znew);
+  } else if (VECTOR_COMPLEX_P(obj)) {
+    Data_Get_Struct(obj, gsl_vector_complex, v);
+    vnew = gsl_vector_complex_alloc(v->size);
+    for (i = 0; i < v->size; i++) {
+      z = GSL_COMPLEX_AT(v, i);
+      gsl_vector_complex_set(vnew, i, (*func)(*z));
+    }
+    return Data_Wrap_Struct(cgsl_vector_complex, 0, gsl_vector_complex_free, vnew);
+  } else if (MATRIX_COMPLEX_P(obj)) {
+    Data_Get_Struct(obj, gsl_matrix_complex, m);
+    mnew = gsl_matrix_complex_alloc(m->size1, m->size2);
+    for (i = 0; i < m->size1; i++) {
+      for (j = 0; j < m->size2; j++) {
+	gsl_matrix_complex_set(mnew, i, j, (*func)(gsl_matrix_complex_get(m, i, j)));
+      }
+    }
+    return Data_Wrap_Struct(cgsl_matrix_complex, 0, gsl_matrix_complex_free, mnew);
+  } else {
+    rb_raise(rb_eTypeError, 
+	     "wrong argument type %s "
+	     " (GSL::Complex or GSL::Vector::Complex expected)", 
+	     rb_class2name(CLASS_OF(obj)));
+  }
+}
+
 static VALUE rb_gsl_math_eval(double (*func)(const double), VALUE xx)
 {
   VALUE x, ary;
-  size_t i, j, size;
-  gsl_vector *v = NULL, *vnew = NULL;
-  gsl_matrix *m = NULL, *mnew = NULL;
+  size_t i, size;
+#ifdef HAVE_NARRAY_H
+  struct NARRAY *na;
+  double *ptr1, *ptr2;
+#endif
   if (CLASS_OF(xx) == rb_cRange) xx = rb_gsl_range2ary(xx);
   switch (TYPE(xx)) {
   case T_FIXNUM:
@@ -121,22 +163,21 @@ static VALUE rb_gsl_math_eval(double (*func)(const double), VALUE xx)
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(xx)) {
+      GetNArray(xx, na);
+      ptr1 = (double*) na->ptr;
+      size = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(xx));
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < size; i++) ptr2[i] = (*func)(ptr1[i]);
+      return ary;
+    }
+#endif
     if (VECTOR_P(xx)) {
-      Data_Get_Struct(xx, gsl_vector, v);
-      vnew = gsl_vector_alloc(v->size);
-      for (i = 0; i < v->size; i++) {
-	gsl_vector_set(vnew, i, (*func)(gsl_vector_get(v, i)));
-      }
-      return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, vnew);
+      return vector_eval_create(xx, func);
     } else if (MATRIX_P(xx)) {
-      Data_Get_Struct(xx, gsl_matrix, m);
-      mnew = gsl_matrix_alloc(m->size1, m->size2);
-      for (i = 0; i < m->size1; i++) {
-	for (j = 0; j < m->size2; j++) {
-	  gsl_matrix_set(mnew, i, j, (*func)(gsl_matrix_get(m, i, j)));
-	}
-      }
-      return Data_Wrap_Struct(cgsl_matrix, 0, gsl_matrix_free, mnew);
+      return matrix_eval_create(xx, func);
     } else {
       rb_raise(rb_eTypeError, "wrong argument type %s (Array or Vector or Matrix expected)", rb_class2name(CLASS_OF(xx)));
     }
@@ -155,6 +196,10 @@ static VALUE rb_gsl_math_eval2(double (*func)(const double, const double), VALUE
   size_t i, j, size;
   gsl_vector *v = NULL, *v2 = NULL, *vnew = NULL;
   gsl_matrix *m = NULL, *m2 = NULL, *mnew = NULL;
+#ifdef HAVE_NARRAY_H
+  struct NARRAY *nax, *nay;
+  double *ptr1, *ptr2, *ptr3;
+#endif
   if (CLASS_OF(xx) == rb_cRange) xx = rb_gsl_range2ary(xx);
   switch (TYPE(xx)) {
   case T_FIXNUM:
@@ -177,6 +222,19 @@ static VALUE rb_gsl_math_eval2(double (*func)(const double, const double), VALUE
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(xx)) {
+      GetNArray(xx, nax);
+      GetNArray(yy, nay);
+      ptr1 = (double*) nax->ptr;
+      ptr2 = (double*) nay->ptr;
+      size = nax->total;
+      ary = na_make_object(NA_DFLOAT, nax->rank, nax->shape, CLASS_OF(xx));
+      ptr3 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < size; i++) ptr3[i] = (*func)(ptr1[i], ptr2[i]);
+      return ary;
+    }
+#endif
     if (VECTOR_P(xx)) {
       CHECK_VECTOR(yy);
       Data_Get_Struct(xx, gsl_vector, v);
@@ -198,7 +256,9 @@ static VALUE rb_gsl_math_eval2(double (*func)(const double, const double), VALUE
       }
       return Data_Wrap_Struct(cgsl_matrix, 0, gsl_matrix_free, mnew);
     } else {
-      rb_raise(rb_eTypeError, "wrong argument type %s (Array or Vector or Matrix expected)", rb_class2name(CLASS_OF(xx)));
+      rb_raise(rb_eTypeError, 
+	       "wrong argument type %s "
+	       "(Array or Vector or Matrix expected)", rb_class2name(CLASS_OF(xx)));
     }
     break;
   }
@@ -223,36 +283,49 @@ static VALUE rb_gsl_hypot(VALUE obj, VALUE x, VALUE y)
 
 static VALUE rb_gsl_acosh(VALUE obj, VALUE x)
 {
+  if (COMPLEX_P(x) || VECTOR_COMPLEX_P(x) || MATRIX_COMPLEX_P(x)) 
+    return rb_gsl_math_complex_eval(gsl_complex_arccosh, x);
   return rb_gsl_math_eval(gsl_acosh, x);
 }
 
 static VALUE rb_gsl_asinh(VALUE obj, VALUE x)
 {
+  if (COMPLEX_P(x) || VECTOR_COMPLEX_P(x) || MATRIX_COMPLEX_P(x)) 
+    return rb_gsl_math_complex_eval(gsl_complex_arcsinh, x);
   return rb_gsl_math_eval(gsl_asinh, x);
 }
 
 static VALUE rb_gsl_atanh(VALUE obj, VALUE x)
 {
+  if (COMPLEX_P(x) || VECTOR_COMPLEX_P(x) || MATRIX_COMPLEX_P(x)) 
+    return rb_gsl_math_complex_eval(gsl_complex_arctanh, x);
   return rb_gsl_math_eval(gsl_atanh, x);
 }
 
 #include <math.h>
+/* xx: Numeric, Complex, Vector, Matrix
+   nn: Numeric, Complex
+*/
 static VALUE rb_gsl_pow(VALUE obj, VALUE xx, VALUE nn)
 {
-  VALUE x, ary;
+  VALUE x, ary, argv[2];
   size_t i, j, size;
   double n;
   gsl_vector *v = NULL, *vnew = NULL;
   gsl_matrix *m = NULL, *mnew = NULL;
-  n = NUM2DBL(nn);
+#ifdef HAVE_NARRAY_H
+  struct NARRAY *na;
+  double *ptr1, *ptr2;
+#endif
   if (CLASS_OF(xx) == rb_cRange) xx = rb_gsl_range2ary(xx);
   switch (TYPE(xx)) {
   case T_FIXNUM:
   case T_BIGNUM:
   case T_FLOAT:
-    return rb_float_new(pow(NUM2DBL(xx), n));
+    return rb_float_new(pow(NUM2DBL(xx), NUM2DBL(nn)));
     break;
   case T_ARRAY:
+    n = NUM2DBL(nn);
     size = RARRAY(xx)->len;
     ary = rb_ary_new2(size);
     for (i = 0; i < size; i++) {
@@ -263,14 +336,29 @@ static VALUE rb_gsl_pow(VALUE obj, VALUE xx, VALUE nn)
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(xx)) {
+      n = NUM2DBL(nn);
+      GetNArray(xx, na);
+      ptr1 = (double*) na->ptr;
+      size = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(xx));
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < size; i++) ptr2[i] = pow(ptr1[i], n);
+      return ary;
+    }
+#endif
     if (VECTOR_P(xx)) {
+      n = NUM2DBL(nn);
       Data_Get_Struct(xx, gsl_vector, v);
       vnew = gsl_vector_alloc(v->size);
       for (i = 0; i < v->size; i++) {
 	gsl_vector_set(vnew, i, pow(gsl_vector_get(v, i), n));
       }
       return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, vnew);
-    } else if (MATRIX_P(xx)) {
+    } 
+    if (MATRIX_P(xx)) {
+      n = NUM2DBL(nn);
       Data_Get_Struct(xx, gsl_matrix, m);
       mnew = gsl_matrix_alloc(m->size1, m->size2);
       for (i = 0; i < m->size1; i++) {
@@ -279,9 +367,13 @@ static VALUE rb_gsl_pow(VALUE obj, VALUE xx, VALUE nn)
 	}
       }
       return Data_Wrap_Struct(cgsl_matrix, 0, gsl_matrix_free, mnew);
-    } else {
-      rb_raise(rb_eTypeError, "wrong argument type %s (Array or Vector or Matrix expected)", rb_class2name(CLASS_OF(xx)));
+    } 
+    if (COMPLEX_P(xx) || VECTOR_COMPLEX_P(xx) || MATRIX_COMPLEX_P(xx)) {
+      argv[0] = xx;
+      argv[1] = nn;
+      return rb_gsl_complex_pow(2, argv, obj);
     }
+    rb_raise(rb_eTypeError, "wrong argument type %s (Array or Vector or Matrix expected)", rb_class2name(CLASS_OF(xx)));
     break;
   }
   /* never reach here */
@@ -290,21 +382,26 @@ static VALUE rb_gsl_pow(VALUE obj, VALUE xx, VALUE nn)
 
 static VALUE rb_gsl_pow_int(VALUE obj, VALUE xx, VALUE nn)
 {
-  VALUE x, ary;
+  VALUE x, ary, argv[2];
   size_t i, j, size;
-  double n;
+  int n;
   gsl_vector *v = NULL, *vnew = NULL;
   gsl_matrix *m = NULL, *mnew = NULL;
-  CHECK_FIXNUM(nn);
-  n = FIX2INT(nn);
+#ifdef HAVE_NARRAY_H
+  struct NARRAY *na;
+  double *ptr1, *ptr2;
+#endif
+
   if (CLASS_OF(xx) == rb_cRange) xx = rb_gsl_range2ary(xx);
   switch (TYPE(xx)) {
   case T_FIXNUM:
   case T_BIGNUM:
   case T_FLOAT:
-    return rb_float_new(gsl_pow_int(NUM2DBL(xx), n));
+    return rb_float_new(gsl_pow_int(NUM2DBL(xx), FIX2INT(nn)));
     break;
   case T_ARRAY:
+    CHECK_FIXNUM(nn);
+    n = FIX2INT(nn);
     size = RARRAY(xx)->len;
     ary = rb_ary_new2(size);
     for (i = 0; i < size; i++) {
@@ -315,7 +412,22 @@ static VALUE rb_gsl_pow_int(VALUE obj, VALUE xx, VALUE nn)
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(xx)) {
+      CHECK_FIXNUM(nn);
+      n = FIX2INT(nn);
+      GetNArray(xx, na);
+      ptr1 = (double*) na->ptr;
+      size = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(xx));
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < size; i++) ptr2[i] = gsl_pow_int(ptr1[i], n);
+      return ary;
+    }
+#endif
     if (VECTOR_P(xx)) {
+      CHECK_FIXNUM(nn);
+      n = FIX2INT(nn);
       Data_Get_Struct(xx, gsl_vector, v);
       vnew = gsl_vector_alloc(v->size);
       for (i = 0; i < v->size; i++) {
@@ -323,6 +435,8 @@ static VALUE rb_gsl_pow_int(VALUE obj, VALUE xx, VALUE nn)
       }
       return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, vnew);
     } else if (MATRIX_P(xx)) {
+      CHECK_FIXNUM(nn);
+      n = FIX2INT(nn);
       Data_Get_Struct(xx, gsl_matrix, m);
       mnew = gsl_matrix_alloc(m->size1, m->size2);
       for (i = 0; i < m->size1; i++) {
@@ -331,6 +445,10 @@ static VALUE rb_gsl_pow_int(VALUE obj, VALUE xx, VALUE nn)
 	}
       }
       return Data_Wrap_Struct(cgsl_matrix, 0, gsl_matrix_free, mnew);
+    } else if (COMPLEX_P(xx) || VECTOR_COMPLEX_P(xx) || MATRIX_COMPLEX_P(xx)) {
+      argv[0] = xx;
+      argv[1] = nn;
+      return rb_gsl_complex_pow_real(2, argv, obj);
     } else {
       rb_raise(rb_eTypeError, "wrong argument type %s (Array or Vector or Matrix expected)", rb_class2name(CLASS_OF(xx)));
     }
