@@ -66,7 +66,8 @@ static VALUE FUNCTION(rb_gsl_matrix,new)(int argc, VALUE *argv, VALUE klass)
   VALUE ary;
   struct NARRAY *na;
 #endif
-  if (argc < 1) rb_raise(rb_eArgError, "wrong number of arguments (%d for >= 1)", argc);
+  if (argc < 1) rb_raise(rb_eArgError, 
+			 "wrong number of arguments (%d for >= 1)", argc);
 
 #ifdef HAVE_NARRAY_H
   if (NA_IsNArray(argv[0])) {
@@ -211,7 +212,7 @@ GSL_TYPE(gsl_matrix)* FUNCTION(gsl_matrix,alloc_from_vectors)(int argc, VALUE *a
 
 /* create a matrix from two sizes and an array */
 GSL_TYPE(gsl_matrix)* FUNCTION(gsl_matrix,alloc_from_array_sizes)(VALUE ary,
-						     VALUE nn1, VALUE nn2)
+								  VALUE nn1, VALUE nn2)
 {
   size_t n1, n2, len;
   GSL_TYPE(gsl_matrix) *m = NULL;
@@ -281,8 +282,10 @@ static VALUE FUNCTION(rb_gsl_matrix,diagonal_singleton)(int argc, VALUE *argv, V
     switch (TYPE(argv[0])) {
     case T_FIXNUM:
     case T_FLOAT:
-      m = FUNCTION(gsl_matrix,alloc)(1,1);
-      FUNCTION(gsl_matrix,set)(m, 0, 0, NUMCONV2(argv[0]));
+      len = FIX2INT(argv[0]);
+      m = FUNCTION(gsl_matrix,alloc)(len, len);
+      for (i = 0; i < len; i++)
+	FUNCTION(gsl_matrix,set)(m, i, i, 1);
       return Data_Wrap_Struct(klass, 0, FUNCTION(gsl_matrix,free), m);
       break;
     default:
@@ -325,27 +328,26 @@ static VALUE FUNCTION(rb_gsl_matrix,diagonal_singleton)(int argc, VALUE *argv, V
 static VALUE FUNCTION(rb_gsl_matrix,eye)(int argc, VALUE *argv, VALUE klass)
 {
   GSL_TYPE(gsl_matrix) *m = NULL;
-  BASE val;
-  size_t n, i;
+  size_t n1, n2, n, i;
   switch (argc) {
   case 1:
     CHECK_FIXNUM(argv[0]);
     n = FIX2INT(argv[0]);
-    val = 1;
+    n1 = n2 = n;
     break;
   case 2:
-    CHECK_FIXNUM(argv[0]);
-    n = FIX2INT(argv[0]);
-    val = NUMCONV2(argv[1]);
+    CHECK_FIXNUM(argv[0]); CHECK_FIXNUM(argv[1]);
+    n1 = FIX2INT(argv[0]); n2 = FIX2INT(argv[1]);
+    n = GSL_MIN_INT(n1, n2);
     break;
   default:
     rb_raise(rb_eArgError, "wrong number of arguments (%d for 1 or 2)", argc);
     break;
   }
-  m = FUNCTION(gsl_matrix,calloc)(n, n);
+  m = FUNCTION(gsl_matrix,calloc)(n1, n2);
   if (m == NULL) rb_raise(rb_eNoMemError, "gsl_matrix_calloc failed");
   for (i = 0; i < n; i++) {
-    FUNCTION(gsl_matrix,set)(m, i, i, val);
+    FUNCTION(gsl_matrix,set)(m, i, i, 1);
   }
   return Data_Wrap_Struct(klass, 0, FUNCTION(gsl_matrix,free), m);
 }
@@ -573,7 +575,7 @@ static VALUE FUNCTION(rb_gsl_matrix,inspect)(VALUE obj)
 {
   VALUE str;
   char buf[64];
-  sprintf(buf, "%s: \n", rb_class2name(CLASS_OF(obj)));
+  sprintf(buf, "%s\n", rb_class2name(CLASS_OF(obj)));
   str = rb_str_new2(buf);
   return rb_str_concat(str, FUNCTION(rb_gsl_matrix,to_s)(obj));
 }
@@ -1207,7 +1209,7 @@ static VALUE FUNCTION(rb_gsl_matrix,equal_singleton)(int argc, VALUE *argv, VALU
 {
   GSL_TYPE(gsl_matrix) *a, *b;
   VALUE aa, bb;
-  double eps;
+  double eps = 1e-10;
   BASE x, y;
   size_t i, j;
   switch (argc) {
@@ -1641,6 +1643,149 @@ static VALUE FUNCTION(rb_gsl_matrix,block)(VALUE obj)
   return Data_Wrap_Struct(GSL_TYPE(cgsl_block), 0, NULL, m->block);
 }
 
+static VALUE FUNCTION(rb_gsl_matrix,info)(VALUE obj)
+{
+  GSL_TYPE(gsl_matrix) *m;
+  char buf[256];
+  Data_Get_Struct(obj, GSL_TYPE(gsl_matrix), m);
+  sprintf(buf, "Class:      %s\n", rb_class2name(CLASS_OF(obj)));
+  sprintf(buf, "%sSuperClass: %s\n", buf, rb_class2name(RCLASS(CLASS_OF(obj))->super));
+  sprintf(buf, "%sDimension:  %dx%d\n", buf, (int) m->size1, (int) m->size2);
+  sprintf(buf, "%sSize:       %d\n", buf, (int) (m->size1*m->size2));
+  return rb_str_new2(buf);
+}
+
+static VALUE FUNCTION(rb_gsl_matrix,any)(VALUE obj)
+{
+  GSL_TYPE(gsl_matrix) *m;
+  QUALIFIED_VIEW(gsl_vector,view) vv;
+  GSL_TYPE(gsl_vector) *v;
+  gsl_vector_int *vnew;
+  size_t j;
+  Data_Get_Struct(obj, GSL_TYPE(gsl_matrix), m);
+  vnew = gsl_vector_int_alloc(m->size2);
+  for (j = 0; j < m->size2; j++) {
+    vv = FUNCTION(gsl_matrix,column)(m, j);
+    v = &vv.vector;
+    if (FUNCTION(gsl_vector,isnull(v))) gsl_vector_int_set(vnew, j, 0);
+    else gsl_vector_int_set(vnew, j, 1);
+  }
+  return Data_Wrap_Struct(cgsl_vector_int, 0, gsl_vector_int_free, vnew);
+}
+
+static VALUE FUNCTION(rb_gsl_matrix,rot90)(int argc, VALUE *argv, VALUE obj)
+{
+  GSL_TYPE(gsl_matrix) *m, *mnew, *mtmp;
+  int p;
+  switch (argc) {
+  case 0:
+    p = 1;
+    break;
+  case 1:
+    p = FIX2INT(argv[0])%4;
+    break;
+  default:
+    rb_raise(rb_eArgError, "wrong number of arguments (%d for 0 or 1)", argc);
+    break;
+  }
+  Data_Get_Struct(obj, GSL_TYPE(gsl_matrix), m);
+  switch (p) {
+  case 0:
+    mnew = FUNCTION(gsl_matrix,alloc)(m->size1, m->size2);
+    FUNCTION(gsl_matrix,memcpy(mnew, m));
+    break;
+  case 1:
+  case -3:
+    mtmp = FUNCTION(gsl_matrix,alloc)(m->size2, m->size1);
+    FUNCTION(gsl_matrix,transpose_memcpy(mtmp, m));
+    mnew = FUNCTION(gsl_matrix,alloc)(m->size2, m->size1);
+    FUNCTION(mygsl_matrix,reverse_rows)(mnew, mtmp);
+    FUNCTION(gsl_matrix,free)(mtmp);
+    break;
+  case 2:
+  case -2:
+    mtmp = FUNCTION(gsl_matrix,alloc)(m->size1, m->size2);
+    FUNCTION(mygsl_matrix,reverse_rows)(mtmp,m);
+    mnew = FUNCTION(gsl_matrix,alloc)(m->size1, m->size2);
+    FUNCTION(mygsl_matrix,reverse_columns)(mnew, mtmp);
+    FUNCTION(gsl_matrix,free)(mtmp);
+    break;
+  case 3:
+  case -1:
+    mtmp = FUNCTION(gsl_matrix,alloc)(m->size2, m->size1);
+    FUNCTION(gsl_matrix,transpose_memcpy(mtmp, m));
+    mnew = FUNCTION(gsl_matrix,alloc)(m->size2, m->size1);
+    FUNCTION(mygsl_matrix,reverse_columns)(mnew, mtmp);
+    FUNCTION(gsl_matrix,free)(mtmp);
+    break;
+  default:
+    return Qnil;
+    break;
+  }
+  return Data_Wrap_Struct(GSL_TYPE(cgsl_matrix), 0, FUNCTION(gsl_matrix,free), mnew);
+}
+
+void FUNCTION(mygsl_vector,diff)(GSL_TYPE(gsl_vector) *vdst,
+				 GSL_TYPE(gsl_vector) *vsrc, size_t n);
+
+static VALUE FUNCTION(rb_gsl_matrix,diff)(int argc, VALUE *argv, VALUE obj)
+{
+  GSL_TYPE(gsl_matrix) *m, *mnew;
+  QUALIFIED_VIEW(gsl_vector,view) v1, v2;
+  size_t n, j;
+  switch (argc) {
+  case 0:
+    n = 1;
+    break;
+  case 1:
+    n = FIX2INT(argv[0]);
+    break;
+  default:
+    rb_raise(rb_eArgError, "wrong number of arguments (%d for 0 or 1)", argc);
+    break;
+  }
+  if (n <= 0) return obj;
+  Data_Get_Struct(obj, GSL_TYPE(gsl_matrix), m);
+  mnew = FUNCTION(gsl_matrix,alloc)(m->size1-n, m->size2);
+  if (m->size1 <= n) return obj;
+  for (j = 0; j < m->size2; j++) {
+    v1 = FUNCTION(gsl_matrix,column)(m, j);
+    v2 = FUNCTION(gsl_matrix,column)(mnew, j);
+    FUNCTION(mygsl_vector,diff)(&v2.vector, &v1.vector, n);
+  }
+  return Data_Wrap_Struct(GSL_TYPE(cgsl_matrix), 0, FUNCTION(gsl_matrix,free), mnew);
+}
+
+static VALUE FUNCTION(rb_gsl_matrix,test)(VALUE obj, int (*f)(const double))
+{
+  GSL_TYPE(gsl_matrix) *m;
+  gsl_matrix_int *mi;
+  size_t i, j;
+  Data_Get_Struct(obj, GSL_TYPE(gsl_matrix), m);
+  mi = gsl_matrix_int_alloc(m->size1, m->size2);
+  for (i = 0; i < m->size1; i++) {
+    for (j = 0; j < m->size2; j++) {
+      gsl_matrix_int_set(mi, i, j, (*f)(FUNCTION(gsl_matrix,get)(m, i, j)));
+    }
+  }
+  return Data_Wrap_Struct(cgsl_matrix_int, 0, gsl_matrix_int_free, mi);
+}
+
+static VALUE FUNCTION(rb_gsl_matrix,isnan)(VALUE obj)
+{
+  return FUNCTION(rb_gsl_matrix,test)(obj, gsl_isnan);
+}
+
+static VALUE FUNCTION(rb_gsl_matrix,isinf)(VALUE obj)
+{
+  return FUNCTION(rb_gsl_matrix,test)(obj, gsl_isinf);
+}
+
+static VALUE FUNCTION(rb_gsl_matrix,finite)(VALUE obj)
+{
+  return FUNCTION(rb_gsl_matrix,test)(obj, gsl_finite);
+}
+
 void FUNCTION(Init_gsl_matrix,init)(VALUE module)
 {
   rb_define_singleton_method(GSL_TYPE(cgsl_matrix), "new", FUNCTION(rb_gsl_matrix,new), -1);
@@ -1648,12 +1793,12 @@ void FUNCTION(Init_gsl_matrix,init)(VALUE module)
   rb_define_singleton_method(GSL_TYPE(cgsl_matrix), "alloc", FUNCTION(rb_gsl_matrix,new), -1);
   rb_define_singleton_method(GSL_TYPE(cgsl_matrix), "calloc", FUNCTION(rb_gsl_matrix,calloc), 2);
   rb_define_singleton_method(GSL_TYPE(cgsl_matrix), "eye", FUNCTION(rb_gsl_matrix,eye), -1);
-  rb_define_singleton_method(GSL_TYPE(cgsl_matrix), "scalar", 
-			     FUNCTION(rb_gsl_matrix,eye), -1);
   rb_define_singleton_method(GSL_TYPE(cgsl_matrix), "diagonal", 
 			     FUNCTION(rb_gsl_matrix,diagonal_singleton), -1);
   rb_define_singleton_method(GSL_TYPE(cgsl_matrix), "identity", 
 			     FUNCTION(rb_gsl_matrix,identity), 1);
+  rb_define_singleton_method(GSL_TYPE(cgsl_matrix), "scalar", 
+			     FUNCTION(rb_gsl_matrix,identity), 1);
   rb_define_singleton_method(GSL_TYPE(cgsl_matrix), "unit", 
 			     FUNCTION(rb_gsl_matrix,identity), 1);
   rb_define_singleton_method(GSL_TYPE(cgsl_matrix), "I", 
@@ -1735,10 +1880,12 @@ void FUNCTION(Init_gsl_matrix,init)(VALUE module)
   rb_define_alias(GSL_TYPE(cgsl_matrix), "trans!", "transpose!");
   rb_define_method(GSL_TYPE(cgsl_matrix), "reverse_columns", 
 		   FUNCTION(rb_gsl_matrix,reverse_columns), 0);
+  rb_define_alias(GSL_TYPE(cgsl_matrix), "fliplr", "reverse_columns");
   rb_define_method(GSL_TYPE(cgsl_matrix), "reverse_columns!", 
 		   FUNCTION(rb_gsl_matrix,reverse_columns_bang), 0);
   rb_define_method(GSL_TYPE(cgsl_matrix), "reverse_rows", 
 		   FUNCTION(rb_gsl_matrix,reverse_rows), 0);
+  rb_define_alias(GSL_TYPE(cgsl_matrix), "flipud", "reverse_rows");
   rb_define_method(GSL_TYPE(cgsl_matrix), "reverse_rows!", 
 		   FUNCTION(rb_gsl_matrix,reverse_rows_bang), 0);
   /*****/
@@ -1866,6 +2013,9 @@ void FUNCTION(Init_gsl_matrix,init)(VALUE module)
 		   FUNCTION(rb_gsl_matrix,indgen), -1);
   rb_define_method(GSL_TYPE(cgsl_matrix), "indgen!", 
 		   FUNCTION(rb_gsl_matrix,indgen_bang), -1);
+
+  rb_define_method(GSL_TYPE(cgsl_matrix), "info", 
+		   FUNCTION(rb_gsl_matrix,info), 0);
 #ifdef BASE_DOUBLE
   rb_define_singleton_method(GSL_TYPE(cgsl_matrix), "hilbert", 
 			     FUNCTION(rb_gsl_matrix,hilbert), 1);
@@ -1877,6 +2027,17 @@ void FUNCTION(Init_gsl_matrix,init)(VALUE module)
 			     FUNCTION(rb_gsl_matrix,invhilbert), 1);
 #endif
 
+  rb_define_method(GSL_TYPE(cgsl_matrix), "any", 
+		   FUNCTION(rb_gsl_matrix,any), 0);
+  rb_define_method(GSL_TYPE(cgsl_matrix), "rot90", 
+		   FUNCTION(rb_gsl_matrix,rot90), -1);
+
+  rb_define_method(GSL_TYPE(cgsl_matrix), "diff", 
+		   FUNCTION(rb_gsl_matrix,diff), -1);
+  rb_define_method(GSL_TYPE(cgsl_matrix), "isnan", FUNCTION(rb_gsl_matrix,isnan), 0);
+  rb_define_method(GSL_TYPE(cgsl_matrix), "isinf", FUNCTION(rb_gsl_matrix,isinf), 0);
+  rb_define_method(GSL_TYPE(cgsl_matrix), "finite", FUNCTION(rb_gsl_matrix,finite), 0);
+
 }
 
 #undef NUMCONV
