@@ -17,14 +17,14 @@
 #ifndef CHECK_MULTIMIN_FUNCTION
 #define CHECK_MULTIMIN_FUNCTION(x) if(CLASS_OF(x)!=cgsl_multimin_function)\
       rb_raise(rb_eTypeError,\
-      "wrong argument type %s (MultiMin::Function expected)",\
+      "wrong argument type %s (GSL::MultiMin::Function expected)",\
       rb_class2name(CLASS_OF(x)));
 #endif
 
-#ifndef CHECK_FUNCTION_FDF
-#define CHECK_FUNCTION_FDF(x) if(CLASS_OF(x)!=cgsl_multimin_function_fdf)\
+#ifndef CHECK_MULTIMIN_FUNCTION_FDF
+#define CHECK_MULTIMIN_FUNCTION_FDF(x) if(CLASS_OF(x)!=cgsl_multimin_function_fdf)\
       rb_raise(rb_eTypeError,\
-      "wrong argument type %s (MultiMin::Function_fdf expected)",\
+      "wrong argument type %s (GSL::MultiMin::Function_fdf expected)",\
       rb_class2name(CLASS_OF(x)));
 #endif
 
@@ -79,7 +79,8 @@ static VALUE rb_gsl_multimin_function_new(int argc, VALUE *argv, VALUE klass)
   F = ALLOC(gsl_multimin_function);
   F->f = &rb_gsl_multimin_function_f;
   ary = rb_ary_new2(2);
-  (VALUE) F->params = ary;
+  /*  (VALUE) F->params = ary;*/
+  F->params = (void *) ary;
   if (rb_block_given_p()) rb_ary_store(ary, 0, RB_GSL_MAKE_PROC);
   else rb_ary_store(ary, 0, Qnil);
   rb_ary_store(ary, 1, Qnil);
@@ -181,7 +182,8 @@ static VALUE rb_gsl_multimin_function_set_params(int argc, VALUE *argv, VALUE ob
   Data_Get_Struct(obj, gsl_multimin_function, F);
   if (F->params == NULL) {
     ary = rb_ary_new2(4);
-    (VALUE) F->params = ary;
+    /*    (VALUE) F->params = ary;*/
+    F->params = (void *) ary;
   } else {
     ary = (VALUE) F->params;
   }
@@ -212,7 +214,8 @@ static VALUE rb_gsl_multimin_function_fdf_new(int argc, VALUE *argv, VALUE klass
   F->df = &rb_gsl_multimin_function_fdf_df;
   F->fdf = &rb_gsl_multimin_function_fdf_fdf;
   ary = rb_ary_new2(4);
-  (VALUE) F->params = ary;
+  /*  (VALUE) F->params = ary;*/
+  F->params = (void *) ary;
   rb_ary_store(ary, 2, Qnil);
   rb_ary_store(ary, 3, Qnil);
   set_function_fdf(argc, argv, F);
@@ -236,7 +239,8 @@ static void set_function_fdf(int argc, VALUE *argv, gsl_multimin_function_fdf *F
   VALUE ary;
   if (F->params == NULL) {
     ary = rb_ary_new2(4);
-    (VALUE) F->params = ary;
+    /*    (VALUE) F->params = ary;*/
+    F->params = (void *) ary;
   } else {
     ary = (VALUE) F->params;
   }
@@ -285,7 +289,8 @@ static VALUE rb_gsl_multimin_function_fdf_set_procs(int argc, VALUE *argv, VALUE
   Data_Get_Struct(obj, gsl_multimin_function_fdf, F);
   if (F->params == NULL) {
     ary = rb_ary_new2(4);
-    (VALUE) F->params = ary;
+    /*    (VALUE) F->params = ary;*/
+    F->params = (void *) ary;
   } else {
     ary = (VALUE) F->params;
   }
@@ -333,7 +338,8 @@ static VALUE rb_gsl_multimin_function_fdf_set_params(int argc, VALUE *argv, VALU
   Data_Get_Struct(obj, gsl_multimin_function_fdf, F);
   if (F->params == NULL) {
     ary = rb_ary_new2(4);
-    (VALUE) F->params = ary;
+    /*    (VALUE) F->params = ary;*/
+    F->params = (void *) ary;
   } else {
     ary = (VALUE) F->params;
   }
@@ -484,7 +490,7 @@ static VALUE rb_gsl_fdfminimizer_set(VALUE obj, VALUE ff, VALUE xx, VALUE ss,
   gsl_vector *x;
   double stepsize, tol;
   int status;
-  CHECK_FUNCTION_FDF(ff);
+  CHECK_MULTIMIN_FUNCTION_FDF(ff);
   Need_Float(ss); Need_Float(tt);
   Data_Get_Struct(obj, gsl_multimin_fdfminimizer, gmf);
   Data_Get_Struct(ff, gsl_multimin_function_fdf, F);
@@ -696,8 +702,8 @@ void Init_gsl_multimin(VALUE module)
   rb_define_singleton_method(mgsl_multimin, "test_size", rb_gsl_multimin_test_size, 2);
 #endif
 
-  cgsl_multimin_fdfminimizer = rb_define_class_under(mgsl_multimin, "FdfMinimizer", rb_cObject);
-  cgsl_multimin_fminimizer = rb_define_class_under(mgsl_multimin, "FMinimizer", rb_cObject);
+  cgsl_multimin_fdfminimizer = rb_define_class_under(mgsl_multimin, "FdfMinimizer", cGSL_Object);
+  cgsl_multimin_fminimizer = rb_define_class_under(mgsl_multimin, "FMinimizer", cGSL_Object);
   define_const(cgsl_multimin_fdfminimizer, cgsl_multimin_fminimizer);
 
   cgsl_multimin_function = rb_define_class_under(mgsl_multimin, "Function",
@@ -713,7 +719,7 @@ void Init_gsl_multimin(VALUE module)
   rb_define_method(cgsl_multimin_function, "n", rb_gsl_multimin_function_n, 0);
 
   cgsl_multimin_function_fdf = rb_define_class_under(mgsl_multimin, "Function_fdf",
-						     rb_cObject);
+						     cGSL_Object);
   rb_define_singleton_method(cgsl_multimin_function_fdf, "new",
 			     rb_gsl_multimin_function_fdf_new, -1);
 
@@ -752,7 +758,7 @@ void Init_gsl_multimin(VALUE module)
 #ifdef CHECK_MULTIMIN_FUNCTION
 #undef CHECK_MULTIMIN_FUNCTION
 #endif
-#ifdef CHECK_FUNCTION_FDF
-#undef CHECK_FUNCTION_FDF
+#ifdef CHECK_MULTIMIN_FUNCTION_FDF
+#undef CHECK_MULTIMIN_FUNCTION_FDF
 #endif
 
