@@ -17,14 +17,14 @@
 #ifndef CHECK_MULTIROOT_FUNCTION
 #define CHECK_MULTIROOT_FUNCTION(x) if(CLASS_OF(x)!=cgsl_multiroot_function)\
       rb_raise(rb_eTypeError,\
-      "wrong argument type %s (MultiRoot::Function expected)",\
+      "wrong argument type %s (GSL::MultiRoot::Function expected)",\
       rb_class2name(CLASS_OF(x)));
 #endif
 
 #ifndef CHECK_MULTIROOT_FUNCTION_FDF
 #define CHECK_MULTIROOT_FUNCTION_FDF(x) if(CLASS_OF(x)!=cgsl_multiroot_function_fdf)\
       rb_raise(rb_eTypeError,\
-      "wrong argument type %s (MultiRoot::Function_fdf expected)",\
+      "wrong argument type %s (GSL::MultiRoot::Function_fdf expected)",\
       rb_class2name(CLASS_OF(x)));
 #endif
 
@@ -67,7 +67,8 @@ static VALUE rb_gsl_multiroot_function_new(int argc, VALUE *argv, VALUE klass)
   F = ALLOC(gsl_multiroot_function);
   F->f = &rb_gsl_multiroot_function_f;
   ary = rb_ary_new2(2);
-  (VALUE) F->params = ary;
+  /*  (VALUE) F->params = ary;*/
+  F->params = (void *) ary;
   if (rb_block_given_p()) rb_ary_store(ary, 0, RB_GSL_MAKE_PROC);
   else rb_ary_store(ary, 0, Qnil);
   rb_ary_store(ary, 1, Qnil);
@@ -177,7 +178,8 @@ static VALUE rb_gsl_multiroot_function_set_params(int argc, VALUE *argv, VALUE o
   Data_Get_Struct(obj, gsl_multiroot_function, F);
   if (F->params == NULL) {
     ary = rb_ary_new2(4);
-    (VALUE) F->params = ary;
+    /*    (VALUE) F->params = ary;*/
+    F->params = (void *) ary;
   } else {
     ary = (VALUE) F->params;
   }
@@ -215,7 +217,8 @@ static VALUE rb_gsl_multiroot_function_fdf_new(int argc, VALUE *argv, VALUE klas
   F->df = &rb_gsl_multiroot_function_fdf_df;
   F->fdf = &rb_gsl_multiroot_function_fdf_fdf;
   ary = rb_ary_new2(4);
-  (VALUE) F->params = ary;
+  /*  (VALUE) F->params = ary;*/
+  F->params = (void *) ary;
   rb_ary_store(ary, 2, Qnil);
   rb_ary_store(ary, 3, Qnil);
   set_function_fdf(argc, argv, F);
@@ -240,7 +243,8 @@ static void set_function_fdf(int argc, VALUE *argv, gsl_multiroot_function_fdf *
   VALUE ary;
   if (F->params == NULL) {
     ary = rb_ary_new2(4);
-    (VALUE) F->params = ary;
+    /*    (VALUE) F->params = ary;*/
+    F->params = (void *) ary;
   } else {
     ary = (VALUE) F->params;
   }
@@ -308,7 +312,8 @@ static VALUE rb_gsl_multiroot_function_fdf_set_params(int argc, VALUE *argv, VAL
   Data_Get_Struct(obj, gsl_multiroot_function_fdf, F);
   if (F->params == NULL) {
     ary = rb_ary_new2(4);
-    (VALUE) F->params = ary;
+    /*    (VALUE) F->params = ary;*/
+    F->params = (void *) ary;
   } else {
     ary = (VALUE) F->params;
   }
@@ -581,7 +586,7 @@ static VALUE rb_gsl_multiroot_fdfsolver_set(VALUE obj, VALUE vf, VALUE vx)
 {
   gsl_multiroot_fdfsolver *s = NULL;
   gsl_multiroot_function_fdf *f = NULL;
-  gsl_vector *x;
+  gsl_vector *x = NULL;
   int flag = 0, status;
   CHECK_MULTIROOT_FUNCTION_FDF(vf);
   Data_Get_Struct(obj, gsl_multiroot_fdfsolver, s);
@@ -894,7 +899,7 @@ void Init_gsl_multiroot(VALUE module)
   rb_define_method(cgsl_multiroot_function_fdf, "df", rb_gsl_multiroot_function_fdf_get_df, 0);
 
   /* solver */
-  cgsl_multiroot_fsolver = rb_define_class_under(mgsl_multiroot, "FSolver", rb_cObject);
+  cgsl_multiroot_fsolver = rb_define_class_under(mgsl_multiroot, "FSolver", cGSL_Object);
   cgsl_multiroot_fdfsolver = rb_define_class_under(mgsl_multiroot, "FdfSolver", cgsl_multiroot_fsolver);
 
   rb_define_singleton_method(cgsl_multiroot_fsolver, "new", 
