@@ -16,35 +16,35 @@
 #ifndef CHECK_SYSTEM
 #define CHECK_SYSTEM(x) if(CLASS_OF(x)!=cgsl_odeiv_system)\
       rb_raise(rb_eTypeError,\
-      "wrong argument type %s (Odeiv::System expected)",\
+      "wrong argument type %s (GSL::Odeiv::System expected)",\
       rb_class2name(CLASS_OF(x)));
 #endif
 
 #ifndef CHECK_STEP
 #define CHECK_STEP(x) if(CLASS_OF(x)!=cgsl_odeiv_step)\
       rb_raise(rb_eTypeError,\
-      "wrong argument type %s (Odeiv::Step expected)",\
+      "wrong argument type %s (GSL::Odeiv::Step expected)",\
       rb_class2name(CLASS_OF(x)));
 #endif
 
 #ifndef CHECK_CONTROL
 #define CHECK_CONTROL(x) if(CLASS_OF(x)!=cgsl_odeiv_control)\
       rb_raise(rb_eTypeError,\
-      "wrong argument type %s (Odeiv::Control expected)",\
+      "wrong argument type %s (GSL::Odeiv::Control expected)",\
       rb_class2name(CLASS_OF(x)));
 #endif
 
 #ifndef CHECK_EVOLVE
 #define CHECK_EVOLVE(x) if(CLASS_OF(x)!=cgsl_odeiv_evolve)\
       rb_raise(rb_eTypeError,\
-      "wrong argument type %s (Odeiv::Evolve expected)",\
+      "wrong argument type %s (GSL::Odeiv::Evolve expected)",\
       rb_class2name(CLASS_OF(x)));
 #endif
 
 #ifndef CHECK_SOLVER
 #define CHECK_SOLVER(x) if(CLASS_OF(x)!=cgsl_odeiv_solver)\
       rb_raise(rb_eTypeError,\
-      "wrong argument type %s (Odeiv::Solver expected)",\
+      "wrong argument type %s (GSL::Odeiv::Solver expected)",\
       rb_class2name(CLASS_OF(x)));
 #endif
 
@@ -65,6 +65,7 @@ enum {
   GSL_ODEIV_STEP_BSIMP,
   GSL_ODEIV_STEP_GEAR1,
   GSL_ODEIV_STEP_GEAR2,
+  GSL_ODEIV_STEP_RK2SIMP,
 };
 
 typedef struct __rb_gsl_odeiv_evolve {
@@ -194,7 +195,8 @@ static void set_sys(int argc, VALUE *argv, gsl_odeiv_system *sys)
 
   if (sys->params == NULL) {
     ary = rb_ary_new2(4);
-    (VALUE) sys->params = ary;
+    /*    (VALUE) sys->params = ary;*/
+    sys->params = (void *) ary;
   } else {
     ary = (VALUE) sys->params;
   }
@@ -335,6 +337,9 @@ static const gsl_odeiv_step_type* rb_gsl_odeiv_step_type_get(VALUE tt)
     case GSL_ODEIV_STEP_BSIMP: T = gsl_odeiv_step_bsimp; break;
     case GSL_ODEIV_STEP_GEAR1: T = gsl_odeiv_step_gear1; break;
     case GSL_ODEIV_STEP_GEAR2: T = gsl_odeiv_step_gear2; break;
+#ifdef GSL_1_6_LATER
+    case GSL_ODEIV_STEP_RK2SIMP: T = gsl_odeiv_step_rk2simp; break;
+#endif
     default:
       rb_raise(rb_eArgError, "wrong argument type (Fixnum expected)");
       break;
@@ -352,6 +357,9 @@ static const gsl_odeiv_step_type* rb_gsl_odeiv_step_type_get(VALUE tt)
     else if (str_tail_grep(name, "bsimp") == 0) T = gsl_odeiv_step_bsimp;
     else if (str_tail_grep(name, "gear1") == 0) T = gsl_odeiv_step_gear1;
     else if (str_tail_grep(name, "gear2") == 0) T = gsl_odeiv_step_gear2;
+#ifdef GSL_1_6_LATER
+    else if (str_tail_grep(name, "rk2simp") == 0) T = gsl_odeiv_step_rk2simp;
+#endif
     else {
       rb_raise(rb_eArgError, "wrong argument type %s", name);
     }
@@ -432,6 +440,18 @@ static VALUE rb_gsl_odeiv_step_apply(int argc, VALUE *argv, VALUE obj)
 				      dydt_in, dydt_out, sys));
 }
 
+static VALUE rb_gsl_odeiv_step_info(VALUE obj)
+{
+  gsl_odeiv_step *s;
+  char buf[256];
+  Data_Get_Struct(obj, gsl_odeiv_step, s);
+  sprintf(buf, "Class:      %s\n", rb_class2name(CLASS_OF(obj)));
+  sprintf(buf, "%sSuperClass: %s\n", buf, rb_class2name(RCLASS(CLASS_OF(obj))->super));
+  sprintf(buf, "%sType:       %s\n", buf, gsl_odeiv_step_name(s));
+  sprintf(buf, "%sDimension:  %d\n", buf, (int) s->dimension);
+  return rb_str_new2(buf);
+}
+
 static gsl_odeiv_control* make_control_standard(VALUE epsabs, 
 					       VALUE epsrel,
 						VALUE ay, VALUE adydt);
@@ -826,7 +846,7 @@ void Init_gsl_odeiv(VALUE module)
   rb_define_const(mgsl_odeiv, "HADJ_INC", INT2FIX(GSL_ODEIV_HADJ_INC));
   rb_define_const(mgsl_odeiv, "HADJ_NIL", INT2FIX(GSL_ODEIV_HADJ_NIL));
   cgsl_odeiv_step = rb_define_class_under(mgsl_odeiv, "Step", 
-					    rb_cObject);
+					    cGSL_Object);
   rb_define_singleton_method(cgsl_odeiv_step, "new", rb_gsl_odeiv_step_new, -1);
   rb_define_singleton_method(cgsl_odeiv_step, "alloc", rb_gsl_odeiv_step_new, -1);
 
@@ -841,6 +861,7 @@ void Init_gsl_odeiv(VALUE module)
   rb_define_const(cgsl_odeiv_step, "BSIMP", INT2FIX(GSL_ODEIV_STEP_BSIMP));
   rb_define_const(cgsl_odeiv_step, "GEAR1", INT2FIX(GSL_ODEIV_STEP_GEAR1));
   rb_define_const(cgsl_odeiv_step, "GEAR2", INT2FIX(GSL_ODEIV_STEP_GEAR2));
+  rb_define_const(cgsl_odeiv_step, "RK2SIMP", INT2FIX(GSL_ODEIV_STEP_RK2SIMP));
   /*****/
 
   rb_define_method(cgsl_odeiv_step, "reset", rb_gsl_odeiv_step_reset, 0);
@@ -849,11 +870,12 @@ void Init_gsl_odeiv(VALUE module)
   rb_define_method(cgsl_odeiv_step, "dimension", rb_gsl_odeiv_step_dimension, 0);
   rb_define_alias(cgsl_odeiv_step, "dim", "dimension");
   rb_define_method(cgsl_odeiv_step, "apply", rb_gsl_odeiv_step_apply, -1);
+  rb_define_method(cgsl_odeiv_step, "info", rb_gsl_odeiv_step_info, 0);
 
   /****/
 
   cgsl_odeiv_control = rb_define_class_under(mgsl_odeiv, "Control", 
-					    rb_cObject);
+					    cGSL_Object);
   rb_define_singleton_method(cgsl_odeiv_control, "new", rb_gsl_odeiv_control_standard_new, 4);
   rb_define_singleton_method(cgsl_odeiv_control, "standard_new", rb_gsl_odeiv_control_standard_new, 4);
  rb_define_singleton_method(cgsl_odeiv_control, "y_new", rb_gsl_odeiv_control_y_new, 2);
@@ -868,7 +890,7 @@ void Init_gsl_odeiv(VALUE module)
 
  /****/
   cgsl_odeiv_evolve = rb_define_class_under(mgsl_odeiv, "Evolve", 
-					     rb_cObject);
+					     cGSL_Object);
   rb_define_singleton_method(cgsl_odeiv_evolve, "new", rb_gsl_odeiv_evolve_new, 1);
   rb_define_singleton_method(cgsl_odeiv_evolve, "alloc", rb_gsl_odeiv_evolve_new, 1);
 
@@ -883,7 +905,7 @@ void Init_gsl_odeiv(VALUE module)
   /*****/
 
   cgsl_odeiv_system = rb_define_class_under(mgsl_odeiv, "System", 
-					    rb_cObject);
+					    cGSL_Object);
   rb_define_singleton_method(cgsl_odeiv_system, "new", rb_gsl_odeiv_system_new, -1);
   rb_define_singleton_method(cgsl_odeiv_system, "alloc", rb_gsl_odeiv_system_new, -1);
   rb_define_method(cgsl_odeiv_system, "set", rb_gsl_odeiv_system_set, -1);
@@ -901,7 +923,7 @@ void Init_gsl_odeiv(VALUE module)
   rb_define_alias(cgsl_odeiv_system, "dim", "dimension");
 
   /*****/
-  cgsl_odeiv_solver = rb_define_class_under(mgsl_odeiv, "Solver", rb_cObject);
+  cgsl_odeiv_solver = rb_define_class_under(mgsl_odeiv, "Solver", cGSL_Object);
   rb_define_singleton_method(cgsl_odeiv_solver, "new", rb_gsl_odeiv_solver_new, -1);
   rb_define_singleton_method(cgsl_odeiv_solver, "alloc", rb_gsl_odeiv_solver_new, -1);
   rb_define_method(cgsl_odeiv_solver, "step", rb_gsl_odeiv_solver_step, 0);
