@@ -45,15 +45,64 @@ static VALUE rb_gsl_permutation_init(VALUE obj)
   return obj;
 }
 
-static VALUE rb_gsl_permutation_get(VALUE obj, VALUE ii)
+void get_range_beg_en_n(VALUE range, int *beg, int *en, size_t *n, int *step);
+static VALUE rb_gsl_permutation_get(int argc, VALUE *argv, VALUE obj)
 {
-  gsl_permutation *p = NULL;
-  size_t i;
-  CHECK_FIXNUM(ii);
-  Data_Get_Struct(obj, gsl_permutation, p);
-  i = FIX2INT(ii);
-  if (i >= p->size) rb_raise(rb_eArgError, "index out of range");
-  return INT2FIX(gsl_permutation_get(p, i));
+  gsl_permutation *b, *bnew;
+  gsl_index *p;
+  int beg, en, i, step;
+  size_t n, j, k;
+  Data_Get_Struct(obj, gsl_permutation, b);
+  switch (argc) {
+  case 0:
+    rb_raise(rb_eArgError, "too few arguments (%d for >= 1)", argc);
+    break;
+  case 1:
+    switch (TYPE(argv[0])) {
+    case T_FIXNUM:
+      i = FIX2INT(argv[0]);
+      if (i < 0) j = b->size + i; else j = (size_t) i;
+      return INT2FIX((int) b->data[j]);
+      break;
+    case T_ARRAY:
+      n = RARRAY(argv[0])->len;
+      bnew = gsl_permutation_alloc(n);
+      for (j = 0; j < n; j++) {
+	i = FIX2INT(rb_ary_entry(argv[0], j));
+	if (i < 0) k = b->size + i; else k = i;
+	bnew->data[j] = b->data[k];
+      }
+      return Data_Wrap_Struct(CLASS_OF(argv[0]), 0, gsl_permutation_free, bnew);
+      break;
+    default:
+      if (PERMUTATION_P(argv[0])) {
+	Data_Get_Struct(argv[0], gsl_index, p);
+	bnew = gsl_permutation_alloc(p->size);
+	for (j = 0; j < p->size; j++) bnew->data[j] = b->data[p->data[j]];
+	return Data_Wrap_Struct(CLASS_OF(argv[0]), 0, gsl_permutation_free, bnew);
+      } else if (CLASS_OF(argv[0]) == rb_cRange) {
+	get_range_beg_en_n(argv[0], &beg, &en, &n, &step);
+	bnew = gsl_permutation_alloc(n);
+	for (j = 0; j < n; j++) 
+	  bnew->data[j] = b->data[beg+j];
+	return Data_Wrap_Struct(CLASS_OF(obj), 0, gsl_permutation_free, bnew);
+      } else {
+	rb_raise(rb_eArgError, "wrong argument type %s (Fixnum, Array, or Range expected)", rb_class2name(CLASS_OF(argv[0])));
+	break;
+      }
+    }
+    break;
+  default:
+    bnew = gsl_permutation_alloc(argc);
+    for (j = 0; j < argc; j++) {
+      i = FIX2INT(argv[j]);
+      if (i < 0) k = b->size + i; else k = i;
+      bnew->data[j] = b->data[k];
+    }
+    return Data_Wrap_Struct(CLASS_OF(argv[0]), 0, gsl_permutation_free, bnew);
+    break;
+  }
+  return Qnil;
 }
 
 #ifdef GSL_1_1_LATER
@@ -274,7 +323,7 @@ static VALUE rb_gsl_permutation_inspect(VALUE obj)
 {
   VALUE str;
   char buf[64];
-  sprintf(buf, "%s: \n", rb_class2name(CLASS_OF(obj)));
+  sprintf(buf, "%s\n", rb_class2name(CLASS_OF(obj)));
   str = rb_str_new2(buf);
   return rb_str_concat(str, rb_gsl_permutation_to_s(obj));
 }
@@ -479,7 +528,7 @@ void Init_gsl_permutation(VALUE module)
   rb_define_method(cgsl_permutation, "print", rb_gsl_permutation_print, 0);
   rb_define_method(cgsl_permutation, "inspect", rb_gsl_permutation_inspect, 0);
   rb_define_method(cgsl_permutation, "to_s", rb_gsl_permutation_to_s, 0);
-  rb_define_method(cgsl_permutation, "get", rb_gsl_permutation_get, 1);
+  rb_define_method(cgsl_permutation, "get", rb_gsl_permutation_get, -1);
   rb_define_alias(cgsl_permutation, "[]", "get");
   rb_define_method(cgsl_permutation, "set", rb_gsl_permutation_set, 2);
   rb_define_alias(cgsl_permutation, "[]=", "set");
@@ -531,4 +580,5 @@ void Init_gsl_permutation(VALUE module)
 
   rb_define_method(cgsl_permutation, "equal?", rb_gsl_permutation_equal, 1);
   rb_define_alias(cgsl_permutation, "==", "equal?");
+
 }
