@@ -77,6 +77,13 @@ static VALUE FUNCTION(rb_gsl_poly,eval)(VALUE obj, VALUE xx)
   gsl_matrix *mnew = NULL;
   VALUE x, ary;
   size_t i, j;
+#ifdef BASE_DOUBLE
+#ifdef HAVE_NARRAY_H
+  struct NARRAY *na;
+  double *ptr1, *ptr2;
+  size_t n;
+#endif
+#endif
   Data_Get_Struct(obj, GSL_TYPE(gsl_poly), p);
   if (CLASS_OF(xx) == rb_cRange) xx = rb_gsl_range2ary(xx);
   switch (TYPE(xx)) {
@@ -95,6 +102,19 @@ static VALUE FUNCTION(rb_gsl_poly,eval)(VALUE obj, VALUE xx)
     return ary;
     break;
   default:
+#ifdef BASE_DOUBLE
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(xx)) {
+      GetNArray(xx, na);
+      ptr1 = (double*) na->ptr;
+      n = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(xx));
+      ptr2 = NA_PTR_TYPE(ary,double*);
+      for (i = 0; i < n; i++) ptr2[i] = FUNCTION(gsl_poly,eval)(p->data,p->size,ptr1[i]);
+      return ary;
+    }
+#endif
+#endif
     if (VEC_P(xx)) {
       Data_Get_Struct(xx, GSL_TYPE(gsl_vector), v);
       vnew = gsl_vector_alloc(v->size);
@@ -130,6 +150,12 @@ static VALUE FUNCTION(rb_gsl_poly,eval2)(int argc, VALUE *argv, VALUE obj)
   gsl_matrix  *mnew = NULL;
   VALUE xx, x, ary;
   size_t i, j, size;
+#ifdef BASE_DOUBLE
+#ifdef HAVE_NARRAY_H
+  struct NARRAY *na;
+  double *ptr1, *ptr2;
+#endif
+#endif
   switch (argc) {
   case 2:
     Data_Get_Struct(argv[0], GSL_TYPE(gsl_poly), p);
@@ -162,6 +188,19 @@ static VALUE FUNCTION(rb_gsl_poly,eval2)(int argc, VALUE *argv, VALUE obj)
     return ary;
     break;
   default:
+#ifdef BASE_DOUBLE
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(xx)) {
+      GetNArray(xx, na);
+      ptr1 = (double*) na->ptr;
+      size = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(xx));
+      ptr2 = NA_PTR_TYPE(ary,double*);
+      for (i = 0; i < size; i++) ptr2[i] = FUNCTION(gsl_poly,eval)(p->data,p->size,ptr1[i]);
+      return ary;
+    }
+#endif
+#endif
     if (VEC_P(xx)) {
       Data_Get_Struct(xx, GSL_TYPE(gsl_vector), v);
       vnew = gsl_vector_alloc(v->size);
@@ -351,8 +390,6 @@ static VALUE FUNCTION(rb_gsl_poly,complex_solve_cubic)(int argc, VALUE *argv, VA
   return Data_Wrap_Struct(cgsl_vector_complex, 0, gsl_vector_complex_free, r);
 }
 
-/* yoshiki */
-
 /* singleton method */
 VALUE FUNCTION(rb_gsl_poly,complex_solve)(int argc, VALUE *argv, VALUE obj)
 {
@@ -1129,6 +1166,107 @@ static VALUE FUNCTION(rb_gsl_poly,companion_matrix)(VALUE obj)
   return Data_Wrap_Struct(cgsl_matrix, 0, gsl_matrix_free, m);
 }
 
+static VALUE FUNCTION(rb_gsl_poly,info)(VALUE obj)
+{
+  GSL_TYPE(gsl_poly) *v;
+  char buf[256];
+  Data_Get_Struct(obj, GSL_TYPE(gsl_poly), v);
+  sprintf(buf, "Class:      %s\n", rb_class2name(CLASS_OF(obj)));
+  sprintf(buf, "%sSuperClass: %s\n", buf, rb_class2name(RCLASS(CLASS_OF(obj))->super));
+  sprintf(buf, "%sOrder:      %d\n", buf, (int) v->size-1);
+  return rb_str_new2(buf);
+}
+
+#ifdef BASE_DOUBLE
+#include "rb_gsl_fit.h"
+/* singleton */
+static VALUE rb_gsl_poly_fit(int argc, VALUE *argv, VALUE obj)
+{
+  gsl_multifit_linear_workspace *space = NULL;
+  gsl_matrix *X = NULL, *cov = NULL;
+  gsl_vector *x, *y = NULL, *c = NULL;
+  gsl_vector_view xx, yy;
+  size_t order, i, j;
+  double chisq, val;
+  int status, flag = 0;
+  VALUE vc, vcov;
+  if (argc != 3 && argc != 4) 
+    rb_raise(rb_eArgError, "wrong number of arguments (%d for 3 or 4)", argc);
+  x = &xx.vector;
+  y = &yy.vector;
+  Data_Get_Vector(argv[0], x);
+  Data_Get_Vector(argv[1], y);
+  order = NUM2INT(argv[2]);
+  if (argc == 4) {
+    Data_Get_Struct(argv[3], gsl_multifit_linear_workspace, space);
+  } else {
+    space = gsl_multifit_linear_alloc(x->size, order + 1);
+    flag = 1;
+  }
+  cov = gsl_matrix_alloc(order + 1, order + 1);
+  c = gsl_vector_alloc(order + 1);
+  X = gsl_matrix_alloc(x->size, order + 1);
+  for (i = 0; i < x->size; i++) {
+    val = 1.0;
+    gsl_matrix_set(X, i, 0, val);
+    for (j = 1; j <= order; j++) {
+      val *= gsl_vector_get(x, i);
+      gsl_matrix_set(X, i, j, val);
+    }
+  }
+  status = gsl_multifit_linear(X, y, c, cov, &chisq, space);
+  if (flag == 1) gsl_multifit_linear_free(space);
+  vc = Data_Wrap_Struct(cgsl_poly, 0, gsl_vector_free, c);
+  vcov = Data_Wrap_Struct(cgsl_matrix, 0, gsl_matrix_free, cov);
+  gsl_matrix_free(X);
+  return rb_ary_new3(4, vc, vcov, rb_float_new(chisq), INT2FIX(status));
+}
+
+static VALUE rb_gsl_poly_wfit(int argc, VALUE *argv, VALUE obj)
+{
+  gsl_multifit_linear_workspace *space = NULL;
+  gsl_matrix *X = NULL, *cov = NULL;
+  gsl_vector *x, *y = NULL, *w, *c = NULL;
+  gsl_vector_view xx, yy, ww;
+  size_t order, i, j;
+  double chisq, val;
+  int status, flag = 0;
+  VALUE vc, vcov;
+  if (argc != 4 && argc != 5) 
+    rb_raise(rb_eArgError, "wrong number of arguments (%d for 4 or 5)", argc);
+  x = &xx.vector;
+  w = &ww.vector;
+  y = &yy.vector;
+  Data_Get_Vector(argv[0], x);
+  Data_Get_Vector(argv[1], w);
+  Data_Get_Vector(argv[2], y);
+  order = NUM2INT(argv[3]);
+  if (argc == 5) {
+    Data_Get_Struct(argv[4], gsl_multifit_linear_workspace, space);
+  } else {
+    space = gsl_multifit_linear_alloc(x->size, order + 1);
+    flag = 1;
+  }
+  cov = gsl_matrix_alloc(order + 1, order + 1);
+  c = gsl_vector_alloc(order + 1);
+  X = gsl_matrix_alloc(x->size, order + 1);
+  for (i = 0; i < x->size; i++) {
+    val = 1.0;
+    gsl_matrix_set(X, i, 0, val);
+    for (j = 1; j <= order; j++) {
+      val *= gsl_vector_get(x, i);
+      gsl_matrix_set(X, i, j, val);
+    }
+  }
+  status = gsl_multifit_linear(X, y, c, cov, &chisq, space);
+  if (flag == 1) gsl_multifit_linear_free(space);
+  vc = Data_Wrap_Struct(cgsl_poly, 0, gsl_vector_free, c);
+  vcov = Data_Wrap_Struct(cgsl_matrix, 0, gsl_matrix_free, cov);
+  gsl_matrix_free(X);
+  return rb_ary_new3(4, vc, vcov, rb_float_new(chisq), INT2FIX(status));
+}
+#endif
+
 void FUNCTION(Init_gsl_poly,init)(VALUE module)
 {
 #ifdef BASE_DOUBLE
@@ -1136,10 +1274,10 @@ void FUNCTION(Init_gsl_poly,init)(VALUE module)
   cgsl_poly = rb_define_class_under(module, "Poly", cgsl_vector);
   cgsl_poly_int = rb_define_class_under(cgsl_poly, "Int", cgsl_vector_int);
 
-  cgsl_poly_workspace = rb_define_class_under(cgsl_poly, "Workspace", rb_cObject);
+  cgsl_poly_workspace = rb_define_class_under(cgsl_poly, "Workspace", cGSL_Object);
   mgsl_poly_complex = rb_define_module_under(cgsl_poly, "Complex");
   cgsl_poly_complex_workspace = rb_define_class_under(mgsl_poly_complex, 
-						      "Workspace", rb_cObject);
+						      "Workspace", cGSL_Object);
   rb_define_singleton_method(cgsl_poly_workspace, "new", 
 			     rb_gsl_poly_workspace_new, 1);
   rb_define_singleton_method(cgsl_poly_complex_workspace, "new",
@@ -1248,6 +1386,17 @@ void FUNCTION(Init_gsl_poly,init)(VALUE module)
   rb_define_method(GSL_TYPE(cgsl_poly), "companion_matrix", 
 		   FUNCTION(rb_gsl_poly,companion_matrix), 0);
   rb_define_alias(GSL_TYPE(cgsl_poly), "compan", "companion_matrix");
+
+  /*****/
+  rb_define_method(GSL_TYPE(cgsl_poly), "info", 
+		   FUNCTION(rb_gsl_poly,info), 0);
+
+#ifdef BASE_DOUBLE
+  rb_define_singleton_method(GSL_TYPE(cgsl_poly), "fit", 
+		   FUNCTION(rb_gsl_poly,fit), -1);
+  rb_define_singleton_method(GSL_TYPE(cgsl_poly), "wfit", 
+		   FUNCTION(rb_gsl_poly,wfit), -1);
+#endif
 }
 
 #undef NUMCONV
