@@ -55,10 +55,22 @@ static VALUE rb_gsl_ran_eval1(int argc, VALUE *argv, VALUE obj,
 			      double (*f)(const gsl_rng*, double))
 {
   gsl_rng *r;
+  gsl_vector *v;
+  size_t n, i;
   double a;
   switch (TYPE(obj)) {
   case T_MODULE: case T_CLASS: case T_OBJECT:
     switch (argc) {
+    case 3:
+      n = NUM2INT(argv[2]);
+      a = NUM2DBL(argv[1]);
+      CHECK_RNG(argv[0]);
+      Data_Get_Struct(argv[0], gsl_rng, r);    
+      v = gsl_vector_alloc(n);
+      for (i = 0; i < n; i++) 
+	gsl_vector_set(v, i, (*f)(r, a));
+      return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+      break;
     case 2:
       CHECK_RNG(argv[0]);
       Data_Get_Struct(argv[0], gsl_rng, r);  
@@ -71,6 +83,15 @@ static VALUE rb_gsl_ran_eval1(int argc, VALUE *argv, VALUE obj,
     break;
   default:
     switch (argc) {
+    case 2:
+      n = NUM2INT(argv[1]);
+      a = NUM2DBL(argv[0]);
+      Data_Get_Struct(obj, gsl_rng, r);    
+      v = gsl_vector_alloc(n);
+      for (i = 0; i < n; i++) 
+	gsl_vector_set(v, i, (*f)(r, a));
+      return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+      break;
     case 1:
       a = NUM2DBL(argv[0]);
       break;
@@ -88,10 +109,22 @@ static VALUE rb_gsl_ran_eval1_uint(int argc, VALUE *argv, VALUE obj,
 			      unsigned int (*f)(const gsl_rng*, double))
 {
   gsl_rng *r;
+  gsl_vector_int *v;
+  size_t n, i;
   double a;
   switch (TYPE(obj)) {
   case T_MODULE: case T_CLASS: case T_OBJECT:
     switch (argc) {
+    case 3:
+      n = NUM2INT(argv[2]);
+      a = NUM2DBL(argv[1]);
+      CHECK_RNG(argv[0]);
+      Data_Get_Struct(argv[0], gsl_rng, r);    
+      v = gsl_vector_int_alloc(n);
+      for (i = 0; i < n; i++) 
+	gsl_vector_int_set(v, i, (int) (*f)(r, a));
+      return Data_Wrap_Struct(cgsl_vector_int, 0, gsl_vector_int_free, v);
+      break;
     case 2:
       CHECK_RNG(argv[0]);
       Data_Get_Struct(argv[0], gsl_rng, r);  
@@ -104,6 +137,15 @@ static VALUE rb_gsl_ran_eval1_uint(int argc, VALUE *argv, VALUE obj,
     break;
   default:
     switch (argc) {
+    case 2:
+      n = NUM2INT(argv[1]);
+      a = NUM2DBL(argv[0]);
+      Data_Get_Struct(obj, gsl_rng, r);    
+      v = gsl_vector_int_alloc(n);
+      for (i = 0; i < n; i++) 
+	gsl_vector_int_set(v, i, (*f)(r, a));
+      return Data_Wrap_Struct(cgsl_vector_int, 0, gsl_vector_int_free, v);
+      break;
     case 1:
       a = NUM2DBL(argv[0]);
       break;
@@ -121,10 +163,21 @@ static VALUE rb_gsl_ran_eval2(int argc, VALUE *argv, VALUE obj,
 			      double (*f)(const gsl_rng*, double, double))
 {
   gsl_rng *r;
+  gsl_vector *v;
+  size_t n, i;
   double a, b;
   switch (TYPE(obj)) {
   case T_MODULE: case T_CLASS: case T_OBJECT:
     switch (argc) {
+    case 4:
+      CHECK_RNG(argv[0]);
+      Data_Get_Struct(argv[0], gsl_rng, r);  
+      a = NUM2DBL(argv[1]); b = NUM2DBL(argv[2]);
+      n = NUM2INT(argv[3]);
+      v = gsl_vector_alloc(n);
+      for (i = 0; i < n; i++) gsl_vector_set(v, i, (*f)(r, a, b));
+      return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+      break;
     case 3:
       CHECK_RNG(argv[0]);
       Data_Get_Struct(argv[0], gsl_rng, r);  
@@ -137,6 +190,14 @@ static VALUE rb_gsl_ran_eval2(int argc, VALUE *argv, VALUE obj,
     break;
   default:
     switch (argc) {
+    case 3:
+      Data_Get_Struct(obj, gsl_rng, r);  
+      a = NUM2DBL(argv[0]); b = NUM2DBL(argv[1]);
+      n = NUM2INT(argv[2]);
+      v = gsl_vector_alloc(n);
+      for (i = 0; i < n; i++) gsl_vector_set(v, i, (*f)(r, a, b));
+      return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+      break;
     case 2:
       a = NUM2DBL(argv[0]); b = NUM2DBL(argv[1]);
       break;
@@ -154,10 +215,21 @@ static VALUE rb_gsl_ran_eval3(int argc, VALUE *argv, VALUE obj,
 			      double (*f)(const gsl_rng*, double, double, double))
 {
   gsl_rng *r;
+  gsl_vector *v;
+  size_t n, i;
   double a, b, c;
   switch (TYPE(obj)) {
   case T_MODULE: case T_CLASS: case T_OBJECT:
     switch (argc) {
+    case 5:
+      CHECK_RNG(argv[0]);
+      Data_Get_Struct(argv[0], gsl_rng, r);  
+      a = NUM2DBL(argv[1]); b = NUM2DBL(argv[2]); c = NUM2DBL(argv[3]);
+      n = NUM2INT(argv[4]);
+      v = gsl_vector_alloc(n);
+      for (i = 0; i < n; i++) gsl_vector_set(v, i, (*f)(r, a, b, c));
+      return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+      break;
     case 4:
       CHECK_RNG(argv[0]);
       Data_Get_Struct(argv[0], gsl_rng, r);  
@@ -170,6 +242,14 @@ static VALUE rb_gsl_ran_eval3(int argc, VALUE *argv, VALUE obj,
     break;
   default:
     switch (argc) {
+    case 4:
+      Data_Get_Struct(obj, gsl_rng, r);  
+      a = NUM2DBL(argv[0]); b = NUM2DBL(argv[1]); c = NUM2DBL(argv[2]);
+      n = NUM2INT(argv[3]);
+      v = gsl_vector_alloc(n);
+      for (i = 0; i < n; i++) gsl_vector_set(v, i, (*f)(r, a, b, c));
+      return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+      break;
     case 3:
       a = NUM2DBL(argv[0]); b = NUM2DBL(argv[1]); c = NUM2DBL(argv[2]);
       break;
@@ -186,10 +266,22 @@ static VALUE rb_gsl_ran_eval3(int argc, VALUE *argv, VALUE obj,
 static VALUE rb_gsl_ran_gaussian(int argc, VALUE *argv, VALUE obj)
 {
   gsl_rng *r = NULL;	
+  gsl_vector *v;
+  size_t n, i;
   double sigma = 1.0;
   switch (TYPE(obj)) {
   case T_MODULE: case T_CLASS: case T_OBJECT:
     switch (argc) {
+    case 3:
+      n = NUM2INT(argv[2]);
+      sigma = NUM2DBL(argv[1]);
+      CHECK_RNG(argv[0]);
+      Data_Get_Struct(argv[0], gsl_rng, r);    
+      v = gsl_vector_alloc(n);
+      for (i = 0; i < n; i++) 
+	gsl_vector_set(v, i, gsl_ran_gaussian(r, sigma));
+      return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+      break;
     case 2:
       sigma = NUM2DBL(argv[1]);
       /* no break */
@@ -207,6 +299,14 @@ static VALUE rb_gsl_ran_gaussian(int argc, VALUE *argv, VALUE obj)
   default:
     Data_Get_Struct(obj, gsl_rng, r);    
     switch (argc) {
+    case 2:
+      n = NUM2INT(argv[1]);
+      sigma = NUM2DBL(argv[0]);
+      v = gsl_vector_alloc(n);
+      for (i = 0; i < n; i++) 
+	gsl_vector_set(v, i, gsl_ran_gaussian(r, sigma));
+      return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+      break;
     case 1:
       sigma = NUM2DBL(argv[0]);
       /* no break */
@@ -275,10 +375,23 @@ static VALUE rb_gsl_ran_gaussian_pdf(int argc, VALUE *argv, VALUE obj)
 static VALUE rb_gsl_ran_gaussian_tail(int argc, VALUE *argv, VALUE obj)
 {
   gsl_rng *r = NULL;	
+  gsl_vector *v;
+  size_t n, i;
   double a, sigma = 1.0;
   switch (TYPE(obj)) {
   case T_MODULE: case T_CLASS: case T_OBJECT:
     switch (argc) {
+    case 4:
+      n = NUM2INT(argv[3]);
+      sigma = NUM2DBL(argv[2]);
+      a = NUM2DBL(argv[1]);
+      CHECK_RNG(argv[0]);
+      Data_Get_Struct(argv[0], gsl_rng, r);    
+      v = gsl_vector_alloc(n);
+      for (i = 0; i < n; i++) 
+	gsl_vector_set(v, i, gsl_ran_gaussian_tail(r, a, sigma));
+      return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+      break;
     case 3:
       sigma = NUM2DBL(argv[2]);
       /* no break */
@@ -296,6 +409,15 @@ static VALUE rb_gsl_ran_gaussian_tail(int argc, VALUE *argv, VALUE obj)
   default:
     Data_Get_Struct(obj, gsl_rng, r);    
     switch (argc) {
+    case 3:
+      n = NUM2INT(argv[2]);
+      sigma = NUM2DBL(argv[1]);
+      a = NUM2DBL(argv[0]);
+      v = gsl_vector_alloc(n);
+      for (i = 0; i < n; i++) 
+	gsl_vector_set(v, i, gsl_ran_gaussian_tail(r, a, sigma));
+      return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+      break;
     case 2:
       sigma = NUM2DBL(argv[1]);
       /* no break */
@@ -313,12 +435,16 @@ static VALUE rb_gsl_ran_gaussian_tail(int argc, VALUE *argv, VALUE obj)
 
 static VALUE rb_gsl_ran_gaussian_tail_pdf(int argc, VALUE *argv, VALUE obj)
 {
-  if (argc == 2) {
+  switch (argc) {
+  case 2:
     return rb_gsl_eval_pdf_cdf2(argv[0], argv[1], gsl_ran_ugaussian_tail_pdf);
-  } else if (argc == 3) {
+    break;
+  case 3:
     return rb_gsl_eval_pdf_cdf3(argv[0], argv[1], argv[2], gsl_ran_gaussian_tail_pdf);
-  } else {
-    rb_raise(rb_eArgError, "wrong number of arguments (%d or 3)", argc);
+    break;
+  default:
+    rb_raise(rb_eArgError, "wrong number of arguments (%d for 2 or 3)", argc);
+    break;
   }
 }
 
@@ -1154,13 +1280,21 @@ static VALUE rb_gsl_ran_discrete_pdf(VALUE obj, VALUE k, VALUE gg)
   return rb_float_new(gsl_ran_discrete_pdf(FIX2INT(k), g));
 }
 
+#ifdef HAVE_NARRAY_H
+#include "narray.h"
+#endif
+
 /*****/
 VALUE rb_gsl_eval_pdf_cdf(VALUE xx, double (*f)(double))
 {
   VALUE x, ary;
   gsl_vector *v = NULL, *vnew = NULL;
   gsl_matrix *m = NULL, *mnew = NULL;
-  size_t i, j, n;
+  size_t i, j, n;  
+#ifdef HAVE_NARRAY_H
+  struct NARRAY *na;
+  double *ptr1, *ptr2;
+#endif
   if (CLASS_OF(xx) == rb_cRange) xx = rb_gsl_range2ary(xx);
   switch(TYPE(xx)) {
   case T_FIXNUM:
@@ -1179,6 +1313,18 @@ VALUE rb_gsl_eval_pdf_cdf(VALUE xx, double (*f)(double))
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(xx)) {
+      xx = na_change_type(xx, NA_DFLOAT);
+      GetNArray(xx, na);
+      ptr1 = (double *) na->ptr;
+      n = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(xx));
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < n; i++) ptr2[i] = (*f)(ptr1[i]);
+      return ary;
+    }
+#endif
     if (VECTOR_P(xx)) {
       Data_Get_Struct(xx, gsl_vector, v);
       vnew = gsl_vector_alloc(v->size);
@@ -1212,6 +1358,10 @@ VALUE rb_gsl_eval_pdf_cdf2(VALUE xx, VALUE aa,
   gsl_vector *v = NULL, *vnew = NULL;
   gsl_matrix *m = NULL, *mnew = NULL;
   size_t i, j, n;
+#ifdef HAVE_NARRAY_H
+  struct NARRAY *na;
+  double *ptr1, *ptr2;
+#endif
   Need_Float(aa);
   a = NUM2DBL(aa);
   if (CLASS_OF(xx) == rb_cRange) xx = rb_gsl_range2ary(xx);
@@ -1232,6 +1382,18 @@ VALUE rb_gsl_eval_pdf_cdf2(VALUE xx, VALUE aa,
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(xx)) {
+      xx = na_change_type(xx, NA_DFLOAT);
+      GetNArray(xx, na);
+      ptr1 = (double *) na->ptr;
+      n = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(xx));
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < n; i++) ptr2[i] = (*f)(ptr1[i], a);
+      return ary;
+    }
+#endif
     if (VECTOR_P(xx)) {
       Data_Get_Struct(xx, gsl_vector, v);
       vnew = gsl_vector_alloc(v->size);
@@ -1265,6 +1427,10 @@ VALUE rb_gsl_eval_pdf_cdf3(VALUE xx, VALUE aa, VALUE bb,
   gsl_vector *v = NULL, *vnew = NULL;
   gsl_matrix *m = NULL, *mnew = NULL;
   size_t i, j, n;
+#ifdef HAVE_NARRAY_H
+  struct NARRAY *na;
+  double *ptr1, *ptr2;
+#endif
   Need_Float(aa);  Need_Float(bb);
   a = NUM2DBL(aa); b = NUM2DBL(bb);
   if (CLASS_OF(xx) == rb_cRange) xx = rb_gsl_range2ary(xx);
@@ -1285,6 +1451,18 @@ VALUE rb_gsl_eval_pdf_cdf3(VALUE xx, VALUE aa, VALUE bb,
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(xx)) {
+      xx = na_change_type(xx, NA_DFLOAT);
+      GetNArray(xx, na);
+      ptr1 = (double *) na->ptr;
+      n = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(xx));
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < n; i++) ptr2[i] = (*f)(ptr1[i], a, b);
+      return ary;
+    }
+#endif
     if (VECTOR_P(xx)) {
       Data_Get_Struct(xx, gsl_vector, v);
       vnew = gsl_vector_alloc(v->size);
@@ -1320,6 +1498,10 @@ VALUE rb_gsl_eval_pdf_cdf2_uint(VALUE xx, VALUE aa,
   gsl_matrix *m = NULL, *mnew = NULL;
   gsl_matrix_int *mi = NULL;
   size_t i, j, n;
+#ifdef HAVE_NARRAY_H
+  struct NARRAY *na;
+  char *ptr1, *ptr2;
+#endif
   Need_Float(aa);
   a = NUM2DBL(aa);
   if (CLASS_OF(xx) == rb_cRange) xx = rb_gsl_range2ary(xx);
@@ -1339,6 +1521,28 @@ VALUE rb_gsl_eval_pdf_cdf2_uint(VALUE xx, VALUE aa,
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(xx)) {
+      GetNArray(xx, na);
+      n = na->total;
+      ary = na_make_object(na->type, na->rank, na->shape, CLASS_OF(xx));
+      switch (na->type) {
+      case NA_LINT:
+	(int *) ptr1 = (int *) na->ptr;
+	(int *) ptr2 = NA_PTR_TYPE(ary, int*);
+	for (i = 0; i < n; i++) 
+	  ((int*)ptr2)[i] = (*f)((unsigned int) ((int*)ptr1)[i], a);
+	break;
+      default:
+	(double *) ptr1 = (double *) na->ptr;
+	(double *) ptr2 = NA_PTR_TYPE(ary, double*);
+	for (i = 0; i < n; i++) 
+	  ((double*)ptr2)[i] = (*f)((unsigned int) ((double*)ptr1)[i], a);
+	break;
+      }
+      return ary;
+    }
+#endif
     if (VECTOR_P(xx)) {
       Data_Get_Struct(xx, gsl_vector, v);
       vnew = gsl_vector_alloc(v->size);
@@ -1573,7 +1777,7 @@ void Init_gsl_ran(VALUE module)
 
   /*****/
 
-  cgsl_ran_discrete = rb_define_class_under(mgsl_ran, "Discrete", rb_cObject);
+  cgsl_ran_discrete = rb_define_class_under(mgsl_ran, "Discrete", cGSL_Object);
   rb_define_singleton_method(cgsl_ran_discrete, "new", rb_gsl_ran_discrete_new, 1);
   rb_define_singleton_method(cgsl_ran_discrete, "preproc", rb_gsl_ran_discrete_new, 1);
   rb_define_method(cgsl_rng, "discrete", rb_gsl_ran_discrete, 1);
