@@ -8,6 +8,12 @@
   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY.
 */
+
+/* 
+   Document-class: <i>GSL::Rng</i>
+   Random number generator
+*/
+
 #include "rb_gsl_config.h"
 #include "rb_gsl_rng.h"
 #ifdef HAVE_RNGEXTRA_RNGEXTRA_H
@@ -48,6 +54,10 @@ enum rb_gsl_rng_generator {
 };
 
 static const gsl_rng_type* get_gsl_rng_type(VALUE t);
+/*
+  Document-method: <i>GSL::Rng.alloc</i>
+    Constructor.
+*/
 static VALUE rb_gsl_rng_alloc(int argc, VALUE *argv, VALUE klass)
 {
   gsl_rng *r = NULL;
@@ -375,18 +385,56 @@ static VALUE rb_gsl_rng_set(VALUE obj, VALUE s)
   return obj;
 }
 
-static VALUE rb_gsl_rng_get(VALUE obj)
+/*
+  Document-method: <i>GSL::Rng#get</i>
+    Returns a random integer from the generator. 
+    The minimum and maximum values depend on the algorithm used, 
+    but all integers in the range [min,max] are equally likely. 
+    The values of min and max can determined using the auxiliary 
+    methodss GSL::Rng#max and GSL::Rng#min.
+*/
+static VALUE rb_gsl_rng_get(int argc, VALUE *argv, VALUE obj)
 {
   gsl_rng *r = NULL;
+  gsl_vector_int *v;
+  size_t n, i;
   Data_Get_Struct(obj, gsl_rng, r);
-  return UINT2NUM(gsl_rng_get(r));
+  switch (argc) {
+  case 0:
+    return UINT2NUM(gsl_rng_get(r));
+    break;
+  case 1:
+    n = NUM2INT(argv[0]);
+    v = gsl_vector_int_alloc(n);
+    for (i = 0; i < n; i++) gsl_vector_int_set(v, i, (int) gsl_rng_get(r));
+    return Data_Wrap_Struct(cgsl_vector_int, 0, gsl_vector_int_free, v);
+    break;
+  default:
+    rb_raise(rb_eArgError, "wrong number of arguments (%d for 0 or 1)", argc);
+    break;
+  }
 }
 
-static VALUE rb_gsl_rng_uniform(VALUE obj)
+static VALUE rb_gsl_rng_uniform(int argc, VALUE *argv, VALUE obj)
 {
   gsl_rng *r = NULL;
+  gsl_vector *v;
+  size_t n, i;
   Data_Get_Struct(obj, gsl_rng, r);
-  return rb_float_new(gsl_rng_uniform(r));
+  switch (argc) {
+  case 0:
+    return rb_float_new(gsl_rng_uniform(r));
+    break;
+  case 1:
+    n = NUM2INT(argv[0]);
+    v = gsl_vector_alloc(n);
+    for (i = 0; i < n; i++) gsl_vector_set(v, i, gsl_rng_uniform(r));
+    return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+    break;
+  default:
+    rb_raise(rb_eArgError, "wrong number of arguments (%d for 0 or 1)", argc);
+    break;
+  }
 }
 
 static VALUE rb_gsl_rng_uniform_pos(VALUE obj)
@@ -506,7 +554,7 @@ static VALUE rb_gsl_rng_memcpy(VALUE obj, VALUE dst, VALUE org)
 
 void Init_gsl_rng(VALUE module)
 {
-  cgsl_rng = rb_define_class_under(module, "Rng", rb_cObject);
+  cgsl_rng = rb_define_class_under(module, "Rng", cGSL_Object);
 
   rb_gsl_rng_define_const_type(module);
   
@@ -520,9 +568,9 @@ void Init_gsl_rng(VALUE module)
   rb_define_method(cgsl_rng, "set", rb_gsl_rng_set, 1);
   rb_define_alias(cgsl_rng, "set_seed", "set");
   rb_define_alias(cgsl_rng, "seed=", "set");
-  rb_define_method(cgsl_rng, "get", rb_gsl_rng_get, 0);
+  rb_define_method(cgsl_rng, "get", rb_gsl_rng_get, -1);
   rb_define_alias(cgsl_rng, "gen", "get");
-  rb_define_method(cgsl_rng, "uniform", rb_gsl_rng_uniform, 0);
+  rb_define_method(cgsl_rng, "uniform", rb_gsl_rng_uniform, -1);
   rb_define_method(cgsl_rng, "uniform_pos", rb_gsl_rng_uniform_pos, 0);
   rb_define_method(cgsl_rng, "uniform_int", rb_gsl_rng_uniform_int, 1);
 
