@@ -12,6 +12,9 @@
 
 #include "rb_gsl_array.h"
 #include "rb_gsl_sf.h"
+#ifdef HAVE_NARRAY_H
+#include "narray.h"
+#endif
 
 VALUE cgsl_sf_result, cgsl_sf_result_e10;
 
@@ -104,12 +107,6 @@ static VALUE rb_gsl_sf_result_e10_to_s(VALUE obj)
 
 VALUE rb_gsl_sf_eval1(double (*func)(double), VALUE argv)
 {
-  gsl_vector *v = NULL, *vnew = NULL;
-  gsl_matrix *m = NULL, *mnew = NULL;
-  VALUE ary;
-  size_t i, j;
-  size_t n;
-  double val;
   if (CLASS_OF(argv) == rb_cRange) argv = rb_gsl_range2ary(argv);
   switch (TYPE(argv)) {
   case T_FLOAT:
@@ -118,35 +115,22 @@ VALUE rb_gsl_sf_eval1(double (*func)(double), VALUE argv)
     return rb_float_new((*func)(NUM2DBL(argv)));
     break;
   case T_ARRAY:
-    n = RARRAY(argv)->len;
-    ary = rb_ary_new2(n);
-    for (i = 0; i < n; i++) {
-      val = (*func)(NUM2DBL(rb_ary_entry(argv, i)));
-      rb_ary_store(ary, i, rb_float_new(val));
-    }
-    return ary;
+    return rb_gsl_ary_eval1(argv, func);
     break;
   default:
-    if (rb_obj_is_kind_of(argv, cgsl_matrix)) {
-      Data_Get_Struct(argv, gsl_matrix, m);
-      mnew = gsl_matrix_alloc(m->size1, m->size2);
-      for (i = 0; i < m->size1; i++) {
-	for (j = 0; j < m->size2; j++) {
-	  val = (*func)(gsl_matrix_get(m, i, j));
-	  gsl_matrix_set(mnew, i, j, val);
-	}
-      }
-      return Data_Wrap_Struct(cgsl_matrix, 0, gsl_matrix_free, mnew);
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(argv)) {
+      return rb_gsl_nary_eval1(argv, func);
+    }
+#endif
+    if (MATRIX_P(argv)) {
+      return matrix_eval_create(argv, func);
+    } else if (VECTOR_P(argv)) {
+      return vector_eval_create(argv, func);
+    } else if (COMPLEX_P(argv) || VECTOR_COMPLEX_P(argv) || MATRIX_COMPLEX_P(argv)) {
+      return rb_gsl_sf_eval_complex(func, argv);
     } else {
-      CHECK_VECTOR(argv);
-      Data_Get_Struct(argv, gsl_vector, v);
-      n = v->size;
-      vnew = gsl_vector_alloc(n);
-      for (i = 0; i < n; i++) {
-	val = (*func)(gsl_vector_get(v, i));
-	gsl_vector_set(vnew, i, val);
-      }
-      return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, vnew);
+      rb_raise(rb_eTypeError, "wrong argument type %s", rb_class2name(CLASS_OF(argv)));
     }
     break;
   }
@@ -157,9 +141,12 @@ VALUE rb_gsl_sf_eval_int_double(double (*func)(int, double), VALUE jj, VALUE arg
   gsl_vector *v = NULL, *vnew = NULL;
   gsl_matrix *m = NULL, *mnew = NULL;
   VALUE ary, xx;
-  size_t i, j, k;
-  size_t n;
+  size_t i, j, k, n;
   double val;
+#ifdef HAVE_NARRAY_H
+  double *ptr1, *ptr2;
+  struct NARRAY *na;
+#endif
   CHECK_FIXNUM(jj);
   j = FIX2INT(jj);
   if (CLASS_OF(argv) == rb_cRange) argv = rb_gsl_range2ary(argv);
@@ -181,6 +168,18 @@ VALUE rb_gsl_sf_eval_int_double(double (*func)(int, double), VALUE jj, VALUE arg
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(argv)) {
+      argv = na_change_type(argv, NA_DFLOAT);
+      ptr1 = NA_PTR_TYPE(argv, double*);
+      GetNArray(argv, na);
+      n = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(argv));
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < n; i++) ptr2[i] = (*func)(j, ptr1[i]);
+      return ary;
+    }
+#endif
     if (rb_obj_is_kind_of(argv, cgsl_matrix)) {
       Data_Get_Struct(argv, gsl_matrix, m);
       mnew = gsl_matrix_alloc(m->size1, m->size2);
@@ -206,15 +205,17 @@ VALUE rb_gsl_sf_eval_int_double(double (*func)(int, double), VALUE jj, VALUE arg
   }
 }
 
-
 VALUE rb_gsl_sf_eval_double_int(double (*func)(double, int), VALUE argv, VALUE jj)
 {
   gsl_vector *v = NULL, *vnew = NULL;
   gsl_matrix *m = NULL, *mnew = NULL;
   VALUE ary, xx;
-  size_t i, j, k;
-  size_t n;
+  size_t i, j, k, n;
   double val;
+#ifdef HAVE_NARRAY_H
+  double *ptr1, *ptr2;
+  struct NARRAY *na;
+#endif
   CHECK_FIXNUM(jj);
   j = FIX2INT(jj);
   if (CLASS_OF(argv) == rb_cRange) argv = rb_gsl_range2ary(argv);
@@ -236,6 +237,18 @@ VALUE rb_gsl_sf_eval_double_int(double (*func)(double, int), VALUE argv, VALUE j
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(argv)) {
+      argv = na_change_type(argv, NA_DFLOAT);
+      ptr1 = NA_PTR_TYPE(argv, double*);
+      GetNArray(argv, na);
+      n = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(argv));
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < n; i++) ptr2[i] = (*func)(ptr1[i], j);
+      return ary;
+    }
+#endif
     if (rb_obj_is_kind_of(argv, cgsl_matrix)) {
       Data_Get_Struct(argv, gsl_matrix, m);
       mnew = gsl_matrix_alloc(m->size1, m->size2);
@@ -259,6 +272,7 @@ VALUE rb_gsl_sf_eval_double_int(double (*func)(double, int), VALUE argv, VALUE j
     }
     break;
   }
+  return Qnil;
 }
 
 VALUE rb_gsl_sf_eval_int_int_double(double (*func)(int, int, double), VALUE jj,
@@ -267,9 +281,12 @@ VALUE rb_gsl_sf_eval_int_int_double(double (*func)(int, int, double), VALUE jj,
   gsl_vector *v = NULL, *vnew = NULL;
   gsl_matrix *m = NULL, *mnew = NULL;
   VALUE ary, xx;
-  size_t i, j, k, j2;
-  size_t n;
+  size_t i, j, k, j2, n;
   double val;
+#ifdef HAVE_NARRAY_H
+  double *ptr1, *ptr2;
+  struct NARRAY *na;
+#endif
   CHECK_FIXNUM(jj); CHECK_FIXNUM(jj2);
   j = FIX2INT(jj);
   j2 = FIX2INT(jj2);
@@ -292,6 +309,18 @@ VALUE rb_gsl_sf_eval_int_int_double(double (*func)(int, int, double), VALUE jj,
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(argv)) {
+      argv = na_change_type(argv, NA_DFLOAT);
+      ptr1 = NA_PTR_TYPE(argv, double*);
+      GetNArray(argv, na);
+      n = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(argv));
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < n; i++) ptr2[i] = (*func)(j, j2, ptr1[i]);
+      return ary;
+    }
+#endif
     if (rb_obj_is_kind_of(argv, cgsl_matrix)) {
       Data_Get_Struct(argv, gsl_matrix, m);
       mnew = gsl_matrix_alloc(m->size1, m->size2);
@@ -323,10 +352,12 @@ VALUE rb_gsl_sf_eval_int_double_double(double (*func)(int, double, double), VALU
   gsl_vector *v = NULL, *vnew = NULL;
   gsl_matrix *m = NULL, *mnew = NULL;
   VALUE ary, xx;
-  size_t i, j, k;
-  double f;
-  size_t n;
-  double val;
+  size_t i, j, k, n;
+  double f, val;
+#ifdef HAVE_NARRAY_H
+  double *ptr1, *ptr2;
+  struct NARRAY *na;
+#endif
   CHECK_FIXNUM(jj);
   Need_Float(ff);
   j = FIX2INT(jj);
@@ -350,6 +381,18 @@ VALUE rb_gsl_sf_eval_int_double_double(double (*func)(int, double, double), VALU
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(argv)) {
+      argv = na_change_type(argv, NA_DFLOAT);
+      ptr1 = NA_PTR_TYPE(argv, double*);
+      GetNArray(argv, na);
+      n = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(argv));
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < n; i++) ptr2[i] = (*func)(j, f, ptr1[i]);
+      return ary;
+    }
+#endif
     if (rb_obj_is_kind_of(argv, cgsl_matrix)) {
       Data_Get_Struct(argv, gsl_matrix, m);
       mnew = gsl_matrix_alloc(m->size1, m->size2);
@@ -380,9 +423,12 @@ VALUE rb_gsl_sf_eval_double_double(double (*func)(double, double), VALUE ff, VAL
   gsl_vector *v = NULL, *vnew = NULL;
   gsl_matrix *m = NULL, *mnew = NULL;
   VALUE ary, xx;
-  size_t i, k;
-  size_t n;
+  size_t i, k, n;
   double val, f;
+#ifdef HAVE_NARRAY_H
+  double *ptr1, *ptr2;
+  struct NARRAY *na;
+#endif
   Need_Float(ff);
   f = NUM2DBL(ff);
   if (CLASS_OF(argv) == rb_cRange) argv = rb_gsl_range2ary(argv);
@@ -404,24 +450,33 @@ VALUE rb_gsl_sf_eval_double_double(double (*func)(double, double), VALUE ff, VAL
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(argv)) {
+      argv = na_change_type(argv, NA_DFLOAT);
+      ptr1 = NA_PTR_TYPE(argv, double*);
+      GetNArray(argv, na);
+      n = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(argv));
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < n; i++) ptr2[i] = (*func)(f, ptr1[i]);
+      return ary;
+    }
+#endif
     if (rb_obj_is_kind_of(argv, cgsl_matrix)) {
       Data_Get_Struct(argv, gsl_matrix, m);
       mnew = gsl_matrix_alloc(m->size1, m->size2);
       for (i = 0; i < m->size1; i++) {
 	for (k = 0; k < m->size2; k++) {
-	  val = (*func)(f, gsl_matrix_get(m, i, k));
-	  gsl_matrix_set(mnew, i, k, val);
+	  gsl_matrix_set(mnew, i, k, (*func)(f, gsl_matrix_get(m, i, k)));
 	}
       }
       return Data_Wrap_Struct(cgsl_matrix, 0, gsl_matrix_free, mnew);
     } else {
       CHECK_VECTOR(argv);
       Data_Get_Struct(argv, gsl_vector, v);
-      n = v->size;
-      vnew = gsl_vector_alloc(n);
-      for (i = 0; i < n; i++) {
-	val = (*func)(f, gsl_vector_get(v, i));
-	gsl_vector_set(vnew, i, val);
+      vnew = gsl_vector_alloc(v->size);
+      for (i = 0; i < v->size; i++) {
+	gsl_vector_set(vnew, i, (*func)(f, gsl_vector_get(v, i)));
       }
       return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, vnew);
     }
@@ -435,9 +490,12 @@ VALUE rb_gsl_sf_eval_double3(double (*func)(double, double, double),
   gsl_vector *v = NULL, *vnew = NULL;
   gsl_matrix *m = NULL, *mnew = NULL;
   VALUE ary, xx;
-  size_t i, k;
-  size_t n;
+  size_t i, k, n;
   double val, f, f2;
+#ifdef HAVE_NARRAY_H
+  double *ptr1, *ptr2;
+  struct NARRAY *na;
+#endif
   Need_Float(ff); Need_Float(ff2);
   f = NUM2DBL(ff);
   f2 = NUM2DBL(ff2);
@@ -460,6 +518,18 @@ VALUE rb_gsl_sf_eval_double3(double (*func)(double, double, double),
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(argv)) {
+      argv = na_change_type(argv, NA_DFLOAT);
+      ptr1 = NA_PTR_TYPE(argv, double*);
+      GetNArray(argv, na);
+      n = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(argv));
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < n; i++) ptr2[i] = (*func)(f, f2, ptr1[i]);
+      return ary;
+    }
+#endif
     if (rb_obj_is_kind_of(argv, cgsl_matrix)) {
       Data_Get_Struct(argv, gsl_matrix, m);
       mnew = gsl_matrix_alloc(m->size1, m->size2);
@@ -491,9 +561,12 @@ VALUE rb_gsl_sf_eval_double4(double (*func)(double, double, double, double),
   gsl_vector *v = NULL, *vnew = NULL;
   gsl_matrix *m = NULL, *mnew = NULL;
   VALUE ary, xx;
-  size_t i, k;
-  size_t n;
+  size_t i, k, n;
   double val, f, f2, f3;
+#ifdef HAVE_NARRAY_H
+  double *ptr1, *ptr2;
+  struct NARRAY *na;
+#endif
   Need_Float(ff); Need_Float(ff2); Need_Float(ff3);
   f = NUM2DBL(ff);
   f2 = NUM2DBL(ff2);
@@ -517,6 +590,18 @@ VALUE rb_gsl_sf_eval_double4(double (*func)(double, double, double, double),
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(argv)) {
+      argv = na_change_type(argv, NA_DFLOAT);
+      ptr1 = NA_PTR_TYPE(argv, double*);
+      GetNArray(argv, na);
+      n = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(argv));
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < n; i++) ptr2[i] = (*func)(f, f2, f3, ptr1[i]);
+      return ary;
+    }
+#endif
     if (rb_obj_is_kind_of(argv, cgsl_matrix)) {
       Data_Get_Struct(argv, gsl_matrix, m);
       mnew = gsl_matrix_alloc(m->size1, m->size2);
@@ -547,9 +632,12 @@ VALUE rb_gsl_sf_eval1_int(double (*func)(int), VALUE argv)
   gsl_vector *v = NULL, *vnew = NULL;
   gsl_matrix *m = NULL, *mnew = NULL;
   VALUE ary;
-  size_t i, k;
-  size_t n;
+  size_t i, k, n;
   double val;
+#ifdef HAVE_NARRAY_H
+  double *ptr1, *ptr2;
+  struct NARRAY *na;
+#endif
   if (CLASS_OF(argv) == rb_cRange) argv = rb_gsl_range2ary(argv);
   switch (TYPE(argv)) {
   case T_FIXNUM:
@@ -567,6 +655,18 @@ VALUE rb_gsl_sf_eval1_int(double (*func)(int), VALUE argv)
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(argv)) {
+      argv = na_change_type(argv, NA_DFLOAT);
+      ptr1 = NA_PTR_TYPE(argv, double*);
+      GetNArray(argv, na);
+      n = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(argv));
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < n; i++) ptr2[i] = (*func)((int) ptr1[i]);
+      return ary;
+    }
+#endif
     if (rb_obj_is_kind_of(argv, cgsl_matrix)) {
       Data_Get_Struct(argv, gsl_matrix, m);
       mnew = gsl_matrix_alloc(m->size1, m->size2);
@@ -597,9 +697,12 @@ VALUE rb_gsl_sf_eval1_uint(double (*func)(unsigned int), VALUE argv)
   gsl_vector *v = NULL, *vnew = NULL;
   gsl_matrix *m = NULL, *mnew = NULL;
   VALUE ary;
-  size_t i, k;
-  size_t n;
+  size_t i, k, n;
   double val;
+#ifdef HAVE_NARRAY_H
+  double *ptr1, *ptr2;
+  struct NARRAY *na;
+#endif
   if (CLASS_OF(argv) == rb_cRange) argv = rb_gsl_range2ary(argv);
   switch (TYPE(argv)) {
   case T_FIXNUM:
@@ -617,6 +720,18 @@ VALUE rb_gsl_sf_eval1_uint(double (*func)(unsigned int), VALUE argv)
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(argv)) {
+      argv = na_change_type(argv, NA_DFLOAT);
+      ptr1 = NA_PTR_TYPE(argv, double*);
+      GetNArray(argv, na);
+      n = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(argv));
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < n; i++) ptr2[i] = (*func)((unsigned int) ptr1[i]);
+      return ary;
+    }
+#endif
     if (rb_obj_is_kind_of(argv, cgsl_matrix)) {
       Data_Get_Struct(argv, gsl_matrix, m);
       mnew = gsl_matrix_alloc(m->size1, m->size2);
@@ -647,12 +762,14 @@ VALUE rb_gsl_sf_eval_double_m(double (*func)(double, gsl_mode_t), VALUE argv, VA
   gsl_vector *v = NULL, *vnew = NULL;
   gsl_matrix *mm = NULL, *mnew = NULL;
   VALUE ary, xx;
-  size_t i, k;
-  size_t n;
+  size_t i, k, n;
   double val;
   gsl_mode_t mode;
   char c;
-
+#ifdef HAVE_NARRAY_H
+  double *ptr1, *ptr2;
+  struct NARRAY *na;
+#endif
   switch (TYPE(m)) {
   case T_STRING:
     c = tolower(NUM2CHR(m));
@@ -687,6 +804,17 @@ VALUE rb_gsl_sf_eval_double_m(double (*func)(double, gsl_mode_t), VALUE argv, VA
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(argv)) {
+      ptr1 = NA_PTR_TYPE(argv, double*);
+      GetNArray(argv, na);
+      n = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(argv));
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < n; i++) ptr2[i] = (*func)(ptr1[i], m);
+      return ary;
+    }
+#endif
     if (rb_obj_is_kind_of(argv, cgsl_matrix)) {
       Data_Get_Struct(argv, gsl_matrix, mm);
       mnew = gsl_matrix_alloc(mm->size1, mm->size2);
@@ -718,12 +846,16 @@ VALUE rb_gsl_sf_eval_double2_m(double (*func)(double, double, gsl_mode_t),
   gsl_vector *v = NULL, *vnew = NULL;
   gsl_matrix *mm = NULL, *mnew = NULL;
   VALUE ary, xx;
-  size_t i, k;
-  size_t n;
-  double val;
+  size_t i, k, n;
+  double val, xx2;
   gsl_mode_t mode;
   char c;
+#ifdef HAVE_NARRAY_H
+  double *ptr1, *ptr2;
+  struct NARRAY *na;
+#endif
   Need_Float(x2);
+  xx2 = NUM2DBL(x2);
   c = tolower(NUM2CHR(m));
   if (c == 'd') mode = GSL_PREC_DOUBLE;
   else if (c == 's') mode = GSL_PREC_SINGLE;
@@ -734,7 +866,7 @@ VALUE rb_gsl_sf_eval_double2_m(double (*func)(double, double, gsl_mode_t),
   case T_FLOAT:
   case T_FIXNUM:
   case T_BIGNUM:
-    return rb_float_new((*func)(NUM2DBL(argv), NUM2DBL(x2), m));
+    return rb_float_new((*func)(NUM2DBL(argv), xx2, m));
     break;
   case T_ARRAY:
     n = RARRAY(argv)->len;
@@ -742,18 +874,29 @@ VALUE rb_gsl_sf_eval_double2_m(double (*func)(double, double, gsl_mode_t),
     for (i = 0; i < n; i++) {
       xx = rb_ary_entry(argv, i);
       Need_Float(xx);
-      val = (*func)(NUM2DBL(xx), NUM2DBL(x2), m);
+      val = (*func)(NUM2DBL(xx), xx2, m);
       rb_ary_store(ary, i, rb_float_new(val));
     }
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(argv)) {
+      ptr1 = NA_PTR_TYPE(argv, double*);
+      GetNArray(argv, na);
+      n = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(argv));
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < n; i++) ptr2[i] = (*func)(ptr1[i], xx2, m);
+      return ary;
+    }
+#endif
     if (rb_obj_is_kind_of(argv, cgsl_matrix)) {
       Data_Get_Struct(argv, gsl_matrix, mm);
       mnew = gsl_matrix_alloc(mm->size1, mm->size2);
       for (i = 0; i < mm->size1; i++) {
 	for (k = 0; k < mm->size2; k++) {
-	  val = (*func)(gsl_matrix_get(mm, i, k), NUM2DBL(x2), m);
+	  val = (*func)(gsl_matrix_get(mm, i, k), xx2, m);
 	  gsl_matrix_set(mnew, i, k, val);
 	}
       }
@@ -764,7 +907,7 @@ VALUE rb_gsl_sf_eval_double2_m(double (*func)(double, double, gsl_mode_t),
       n = v->size;
       vnew = gsl_vector_alloc(n);
       for (i = 0; i < n; i++) {
-	val = (*func)(gsl_vector_get(v, i), NUM2DBL(x2), m);
+	val = (*func)(gsl_vector_get(v, i), xx2, m);
 	gsl_vector_set(vnew, i, val);
       }
       return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, vnew);
@@ -773,19 +916,23 @@ VALUE rb_gsl_sf_eval_double2_m(double (*func)(double, double, gsl_mode_t),
   }
 }
 
-
 VALUE rb_gsl_sf_eval_double3_m(double (*func)(double, double, double, gsl_mode_t), 
 			       VALUE argv, VALUE x2, VALUE x3, VALUE m)
 {
   gsl_vector *v = NULL, *vnew = NULL;
   gsl_matrix *mm = NULL, *mnew = NULL;
   VALUE ary, xx;
-  size_t i, k;
-  size_t n;
-  double val;
+  size_t i, k, n;
+  double val, xx2, xx3;
   gsl_mode_t mode;
   char c;
+#ifdef HAVE_NARRAY_H
+  double *ptr1, *ptr2;
+  struct NARRAY *na;
+#endif
   Need_Float(x2); Need_Float(x3);
+  xx2 = NUM2DBL(x2);
+  xx3 = NUM2DBL(x3);
   c = tolower(NUM2CHR(m));
   if (c == 'd') mode = GSL_PREC_DOUBLE;
   else if (c == 's') mode = GSL_PREC_SINGLE;
@@ -796,7 +943,7 @@ VALUE rb_gsl_sf_eval_double3_m(double (*func)(double, double, double, gsl_mode_t
   case T_FLOAT:
   case T_FIXNUM:
   case T_BIGNUM:
-    return rb_float_new((*func)(NUM2DBL(argv), NUM2DBL(x2), NUM2DBL(x3),m));
+    return rb_float_new((*func)(NUM2DBL(argv), xx2, xx3, m));
     break;
   case T_ARRAY:
     n = RARRAY(argv)->len;
@@ -804,18 +951,29 @@ VALUE rb_gsl_sf_eval_double3_m(double (*func)(double, double, double, gsl_mode_t
     for (i = 0; i < n; i++) {
       xx = rb_ary_entry(argv, i);
       Need_Float(xx);
-      val = (*func)(NUM2DBL(xx), NUM2DBL(x2), NUM2DBL(x3),m);
+      val = (*func)(NUM2DBL(xx), xx2, xx3, m);
       rb_ary_store(ary, i, rb_float_new(val));
     }
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(argv)) {
+      ptr1 = NA_PTR_TYPE(argv, double*);
+      GetNArray(argv, na);
+      n = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(argv));
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < n; i++) ptr2[i] = (*func)(ptr1[i], xx2, xx3, m);
+      return ary;
+    }
+#endif
     if (rb_obj_is_kind_of(argv, cgsl_matrix)) {
       Data_Get_Struct(argv, gsl_matrix, mm);
       mnew = gsl_matrix_alloc(mm->size1, mm->size2);
       for (i = 0; i < mm->size1; i++) {
 	for (k = 0; k < mm->size2; k++) {
-	  val = (*func)(gsl_matrix_get(mm, i, k), NUM2DBL(x2), NUM2DBL(x3),m);
+	  val = (*func)(gsl_matrix_get(mm, i, k), xx2, xx3, m);
 	  gsl_matrix_set(mnew, i, k, val);
 	}
       }
@@ -826,7 +984,7 @@ VALUE rb_gsl_sf_eval_double3_m(double (*func)(double, double, double, gsl_mode_t
       n = v->size;
       vnew = gsl_vector_alloc(n);
       for (i = 0; i < n; i++) {
-	val = (*func)(gsl_vector_get(v, i), NUM2DBL(x2),NUM2DBL(x3), m);
+	val = (*func)(gsl_vector_get(v, i), xx2, xx3, m);
 	gsl_vector_set(vnew, i, val);
       }
       return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, vnew);
@@ -835,7 +993,6 @@ VALUE rb_gsl_sf_eval_double3_m(double (*func)(double, double, double, gsl_mode_t
   }
 }
 
-
 VALUE rb_gsl_sf_eval_double4_m(double (*func)(double, double, double, double,
 					      gsl_mode_t), 
 			       VALUE argv, VALUE x2, VALUE x3, VALUE x4, VALUE m)
@@ -843,11 +1000,14 @@ VALUE rb_gsl_sf_eval_double4_m(double (*func)(double, double, double, double,
   gsl_vector *v = NULL, *vnew = NULL;
   gsl_matrix *mm = NULL, *mnew = NULL;
   VALUE ary, xx;
-  size_t i, k;
-  size_t n;
+  size_t i, k, n;
   double val, xx2, xx3, xx4;
   gsl_mode_t mode;
   char c;
+#ifdef HAVE_NARRAY_H
+  double *ptr1, *ptr2;
+  struct NARRAY *na;
+#endif
   Need_Float(x2); Need_Float(x3); Need_Float(x4);
   xx2 = NUM2DBL(x2);  xx3 = NUM2DBL(x3); xx4 = NUM2DBL(x4);
   c = tolower(NUM2CHR(m));
@@ -875,6 +1035,17 @@ VALUE rb_gsl_sf_eval_double4_m(double (*func)(double, double, double, double,
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(argv)) {
+      ptr1 = NA_PTR_TYPE(argv, double*);
+      GetNArray(argv, na);
+      n = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(argv));
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < n; i++) ptr2[i] = (*func)(ptr1[i], xx2, xx3, xx4, m);
+      return ary;
+    }
+#endif
     if (rb_obj_is_kind_of(argv, cgsl_matrix)) {
       Data_Get_Struct(argv, gsl_matrix, mm);
       mnew = gsl_matrix_alloc(mm->size1, mm->size2);
@@ -1146,13 +1317,15 @@ VALUE rb_gsl_sf_eval_e_double4_m(int (*func)(double, double, double, double, gsl
 
 VALUE eval_sf(double (*func)(double, gsl_mode_t), VALUE argv)
 {
-  size_t n;
   VALUE ary, xx;
-  size_t i, k;
+  size_t i, k, n;
   double val;
   gsl_vector *v = NULL, *vnew = NULL;
   gsl_matrix *mm = NULL, *mnew = NULL;
-
+#ifdef HAVE_NARRAY_H
+  double *ptr1, *ptr2;
+  struct NARRAY *na;
+#endif
   switch (TYPE(argv)) {
   case T_FLOAT:
   case T_FIXNUM:
@@ -1171,6 +1344,17 @@ VALUE eval_sf(double (*func)(double, gsl_mode_t), VALUE argv)
     return ary;
     break;
   default:
+#ifdef HAVE_NARRAY_H
+    if (NA_IsNArray(argv)) {
+      ptr1 = NA_PTR_TYPE(argv, double*);
+      GetNArray(argv, na);
+      n = na->total;
+      ary = na_make_object(NA_DFLOAT, na->rank, na->shape, CLASS_OF(argv));
+      ptr2 = NA_PTR_TYPE(ary, double*);
+      for (i = 0; i < n; i++) ptr2[i] = (*func)(ptr1[i], GSL_PREC_DOUBLE);
+      return ary;
+    }
+#endif
     if (rb_obj_is_kind_of(argv, cgsl_matrix)) {
       Data_Get_Struct(argv, gsl_matrix, mm);
       mnew = gsl_matrix_alloc(mm->size1, mm->size2);
@@ -1196,13 +1380,55 @@ VALUE eval_sf(double (*func)(double, gsl_mode_t), VALUE argv)
   }
 }
 
+VALUE rb_gsl_sf_eval_complex(double (*f)(double), VALUE obj)
+{
+  gsl_complex *z, *znew, c;
+  gsl_vector_complex *v, *vnew;
+  gsl_matrix_complex *m, *mnew;
+  size_t i, j;
+  if (COMPLEX_P(obj)) {
+    Data_Get_Struct(obj, gsl_complex, z);
+    znew = xmalloc(sizeof(gsl_complex));
+    GSL_SET_REAL(znew, (*f)(GSL_REAL(*z)));
+    GSL_SET_IMAG(znew, (*f)(GSL_IMAG(*z)));
+    return Data_Wrap_Struct(cgsl_complex, 0, free, znew);
+  } else if (VECTOR_COMPLEX_P(obj)) {
+    Data_Get_Struct(obj, gsl_vector_complex, v);
+    vnew = gsl_vector_complex_alloc(v->size);
+    for (i = 0; i < v->size; i++) {
+      z = GSL_COMPLEX_AT(v, i);
+      GSL_SET_REAL(&c, (*f)(GSL_REAL(*z)));
+      GSL_SET_IMAG(&c, (*f)(GSL_IMAG(*z)));
+      gsl_vector_complex_set(vnew, i, c);
+    }
+    return Data_Wrap_Struct(cgsl_vector_complex, 0, gsl_vector_complex_free, vnew);
+  } else if (MATRIX_COMPLEX_P(obj)) {
+    Data_Get_Struct(obj, gsl_matrix_complex, m);
+    mnew = gsl_matrix_complex_alloc(m->size1, m->size2);
+    for (i = 0; i < m->size1; i++) {
+      for (j = 0; j < m->size2; j++) {
+	c = gsl_matrix_complex_get(m, i, j);
+	GSL_SET_REAL(&c, (*f)(GSL_REAL(c)));
+	GSL_SET_IMAG(&c, (*f)(GSL_IMAG(c)));
+	gsl_matrix_complex_set(mnew, i, j, c);
+      }
+    }
+    return Data_Wrap_Struct(cgsl_matrix_complex, 0, gsl_matrix_complex_free, mnew);
+  } else {
+    rb_raise(rb_eTypeError, 
+	     "wrong argument type %s "
+	     " (GSL::Complex or GSL::Vector::Complex expected)", 
+	     rb_class2name(CLASS_OF(obj)));
+  }
+}
+
 void Init_gsl_sf(VALUE module)
 {
   VALUE mgsl_sf;
   mgsl_sf = rb_define_module_under(module, "Sf");
 
   cgsl_sf_result = rb_define_class_under(mgsl_sf, "Result", 
-					 rb_cObject);
+					 cGSL_Object);
   rb_define_singleton_method(cgsl_sf_result, "new", rb_gsl_sf_result_new,
 			     0);
   rb_define_method(cgsl_sf_result, "print", rb_gsl_sf_result_print, 0);
@@ -1213,7 +1439,7 @@ void Init_gsl_sf(VALUE module)
   rb_define_method(cgsl_sf_result, "to_s", rb_gsl_sf_result_to_s, 0);
 
   cgsl_sf_result_e10 = rb_define_class_under(mgsl_sf, "Result_e10", 
-					 rb_cObject);
+					     cGSL_Object);
   rb_define_singleton_method(cgsl_sf_result_e10, "new", 
 			     rb_gsl_sf_result_e10_new, 0);
   rb_define_method(cgsl_sf_result_e10, "val", rb_gsl_sf_result_e10_val, 0);
