@@ -44,7 +44,8 @@ static VALUE rb_gsl_sf_bessel_Jn_e(VALUE obj, VALUE n, VALUE x)
   return rb_gsl_sf_eval_e_int_double(gsl_sf_bessel_Jn_e, n, x);
 }
 
-static VALUE rb_gsl_sf_bessel_Jn_array(VALUE obj, VALUE n0, VALUE n1, VALUE x)
+static VALUE rb_gsl_sf_bessel_Xn_array(VALUE obj, VALUE n0, VALUE n1, VALUE x,
+				       int (*f)(int, int, double, double[]))
 {
   int nmin, nmax, n;
   gsl_vector *v = NULL;
@@ -54,10 +55,15 @@ static VALUE rb_gsl_sf_bessel_Jn_array(VALUE obj, VALUE n0, VALUE n1, VALUE x)
   nmax = FIX2INT(n1);
   n = nmax - nmin + 1;
   v = gsl_vector_alloc(n);
-  gsl_sf_bessel_Jn_array(nmin, nmax, NUM2DBL(x), v->data);
+  (*f)(nmin, nmax, NUM2DBL(x), v->data);
   return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
 }
 
+static VALUE rb_gsl_sf_bessel_Jn_array(VALUE obj, VALUE n0, VALUE n1, VALUE x)
+{
+  return rb_gsl_sf_bessel_Xn_array(obj, n0, n1, x, gsl_sf_bessel_Jn_array);
+}
+
 /* Irregular Cylindrical Bessel Functions */
 static VALUE rb_gsl_sf_bessel_Y0(VALUE obj, VALUE x)
 {
@@ -91,16 +97,7 @@ static VALUE rb_gsl_sf_bessel_Yn_e(VALUE obj, VALUE n, VALUE x)
 
 static VALUE rb_gsl_sf_bessel_Yn_array(VALUE obj, VALUE n0, VALUE n1, VALUE x)
 {
-  int nmin, nmax, n;
-  gsl_vector *v = NULL;
-  CHECK_FIXNUM(n0); CHECK_FIXNUM(n1);
-  Need_Float(x);
-  nmin = FIX2INT(n0);
-  nmax = FIX2INT(n1);
-  n = nmax - nmin + 1;
-  v = gsl_vector_alloc(n);
-  gsl_sf_bessel_Yn_array(nmin, nmax, NUM2DBL(x), v->data);
-  return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+  return rb_gsl_sf_bessel_Xn_array(obj, n0, n1, x, gsl_sf_bessel_Yn_array);
 }
 
 /* Regular Modified Cylindrical Bessel Functions */
@@ -136,16 +133,7 @@ static VALUE rb_gsl_sf_bessel_In_e(VALUE obj, VALUE n, VALUE x)
 
 static VALUE rb_gsl_sf_bessel_In_array(VALUE obj, VALUE n0, VALUE n1, VALUE x)
 {
-  int nmin, nmax, n;
-  gsl_vector *v = NULL;
-  CHECK_FIXNUM(n0); CHECK_FIXNUM(n1);
-  Need_Float(x);
-  nmin = FIX2INT(n0);
-  nmax = FIX2INT(n1);
-  n = nmax - nmin + 1;
-  v = gsl_vector_alloc(n);
-  gsl_sf_bessel_In_array(nmin, nmax, NUM2DBL(x), v->data);
-  return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+  return rb_gsl_sf_bessel_Xn_array(obj, n0, n1, x, gsl_sf_bessel_In_array);
 }
 
 static VALUE rb_gsl_sf_bessel_I0_scaled(VALUE obj, VALUE x)
@@ -181,16 +169,7 @@ static VALUE rb_gsl_sf_bessel_In_scaled_e(VALUE obj, VALUE n, VALUE x)
 static VALUE rb_gsl_sf_bessel_In_scaled_array(VALUE obj, VALUE n0, VALUE n1,
 					      VALUE x)
 {
-  int nmin, nmax, n;
-  gsl_vector *v = NULL;
-  CHECK_FIXNUM(n0); CHECK_FIXNUM(n1);
-  Need_Float(x);
-  nmin = FIX2INT(n0);
-  nmax = FIX2INT(n1);
-  n = nmax - nmin + 1;
-  v = gsl_vector_alloc(n);
-  gsl_sf_bessel_In_scaled_array(nmin, nmax, NUM2DBL(x), v->data);
-  return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+  return rb_gsl_sf_bessel_Xn_array(obj, n0, n1, x, gsl_sf_bessel_In_scaled_array);
 }
 
 /* Irregular Modified Cylindrical Bessel Functions */
@@ -226,16 +205,7 @@ static VALUE rb_gsl_sf_bessel_Kn_e(VALUE obj, VALUE n, VALUE x)
 
 static VALUE rb_gsl_sf_bessel_Kn_array(VALUE obj, VALUE n0, VALUE n1, VALUE x)
 {
-  int nmin, nmax, n;
-  gsl_vector *v = NULL;
-  CHECK_FIXNUM(n0); CHECK_FIXNUM(n1);
-  Need_Float(x);
-  nmin = FIX2INT(n0);
-  nmax = FIX2INT(n1);
-  n = nmax - nmin + 1;
-  v = gsl_vector_alloc(n);
-  gsl_sf_bessel_Kn_array(nmin, nmax, NUM2DBL(x), v->data);
-  return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+  return rb_gsl_sf_bessel_Xn_array(obj, n0, n1, x, gsl_sf_bessel_Kn_array);
 }
 
 static VALUE rb_gsl_sf_bessel_K0_scaled(VALUE obj, VALUE x)
@@ -271,16 +241,7 @@ static VALUE rb_gsl_sf_bessel_Kn_scaled_e(VALUE obj, VALUE n, VALUE x)
 static VALUE rb_gsl_sf_bessel_Kn_scaled_array(VALUE obj, VALUE n0, VALUE n1,
 					      VALUE x)
 {
-  int nmin, nmax, n;
-  gsl_vector *v = NULL;
-  CHECK_FIXNUM(n0); CHECK_FIXNUM(n1);
-  Need_Float(x);
-  nmin = FIX2INT(n0);
-  nmax = FIX2INT(n1);
-  n = nmax - nmin + 1;
-  v = gsl_vector_alloc(n);
-  gsl_sf_bessel_Kn_scaled_array(nmin, nmax, NUM2DBL(x), v->data);
-  return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+  return rb_gsl_sf_bessel_Xn_array(obj, n0, n1, x, gsl_sf_bessel_Kn_scaled_array);
 }
 
 /* Spherical Bessel Functions */
@@ -324,30 +285,28 @@ static VALUE rb_gsl_sf_bessel_jl_e(VALUE obj, VALUE n, VALUE x)
   return rb_gsl_sf_eval_e_int_double(gsl_sf_bessel_jl_e, n, x);
 }
 
-static VALUE rb_gsl_sf_bessel_jl_array(VALUE obj, VALUE n1, VALUE x)
+static VALUE rb_gsl_sf_bessel_xl_array(VALUE obj, VALUE n1, VALUE x,
+				       int (*f)(int, double, double[]))
 {
-  int nmax, n;
+  int nmax, n, status;
   gsl_vector *v = NULL;
   CHECK_FIXNUM(n1);
   Need_Float(x);
   nmax = FIX2INT(n1);
   n = nmax  + 1;
   v = gsl_vector_alloc(n);
-  gsl_sf_bessel_jl_array(nmax, NUM2DBL(x), v->data);
+  status = (*f)(nmax, NUM2DBL(x), v->data);
   return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
 }
 
+static VALUE rb_gsl_sf_bessel_jl_array(VALUE obj, VALUE n1, VALUE x)
+{
+  return rb_gsl_sf_bessel_xl_array(obj, n1, x, gsl_sf_bessel_jl_array);
+}
+
 static VALUE rb_gsl_sf_bessel_jl_steed_array(VALUE obj, VALUE n1, VALUE x)
 {
-  int nmax, n;
-  gsl_vector *v = NULL;
-  CHECK_FIXNUM(n1);
-  Need_Float(x);
-  nmax = FIX2INT(n1);
-  n = nmax  + 1;
-  v = gsl_vector_alloc(n);
-  gsl_sf_bessel_jl_steed_array(nmax, NUM2DBL(x), v->data);
-  return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+  return rb_gsl_sf_bessel_xl_array(obj, n1, x, gsl_sf_bessel_jl_steed_array);
 }
 
 /* Irregular Cylindrical Bessel Functions */
@@ -393,15 +352,7 @@ static VALUE rb_gsl_sf_bessel_yl_e(VALUE obj, VALUE n, VALUE x)
 
 static VALUE rb_gsl_sf_bessel_yl_array(VALUE obj, VALUE n1, VALUE x)
 {
-  int nmax, n;
-  gsl_vector *v = NULL;
-  CHECK_FIXNUM(n1);
-  Need_Float(x);
-  nmax = FIX2INT(n1);
-  n = nmax + 1;
-  v = gsl_vector_alloc(n);
-  gsl_sf_bessel_yl_array(nmax, NUM2DBL(x), v->data);
-  return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+  return rb_gsl_sf_bessel_xl_array(obj, n1, x, gsl_sf_bessel_yl_array);
 }
 
 /* Regular Modified Cylindrical Bessel Functions */
@@ -447,15 +398,7 @@ static VALUE rb_gsl_sf_bessel_il_scaled_e(VALUE obj, VALUE n, VALUE x)
 
 static VALUE rb_gsl_sf_bessel_il_scaled_array(VALUE obj, VALUE n1, VALUE x)
 {
-  int nmax, n;
-  gsl_vector *v = NULL;
-  CHECK_FIXNUM(n1);
-  Need_Float(x);
-  nmax = FIX2INT(n1);
-  n = nmax + 1;
-  v = gsl_vector_alloc(n);
-  gsl_sf_bessel_il_scaled_array(nmax, NUM2DBL(x), v->data);
-  return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+  return rb_gsl_sf_bessel_xl_array(obj, n1, x, gsl_sf_bessel_il_scaled_array);
 }
 
 /* Irregular Modified Cylindrical Bessel Functions */
@@ -501,17 +444,9 @@ static VALUE rb_gsl_sf_bessel_kl_scaled_e(VALUE obj, VALUE n, VALUE x)
 }
 
 static VALUE rb_gsl_sf_bessel_kl_scaled_array(VALUE obj, VALUE n1,
-				       VALUE x)
+					      VALUE x)
 {
-  int nmax, n;
-  gsl_vector *v = NULL;
-  CHECK_FIXNUM(n1);
-  Need_Float(x);
-  nmax = FIX2INT(n1);
-  n = nmax + 1;
-  v = gsl_vector_alloc(n);
-  gsl_sf_bessel_kl_scaled_array(nmax, NUM2DBL(x), v->data);
-  return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+  return rb_gsl_sf_bessel_xl_array(obj, n1, x, gsl_sf_bessel_kl_scaled_array);
 }
 
 /* Regular Bessel Function - Fractional Order */
@@ -559,9 +494,11 @@ static VALUE rb_gsl_sf_bessel_sequence_Jnu_e(int argc, VALUE *argv, VALUE obj)
     default:
       rb_raise(rb_eTypeError, "wrong argument type %s (String or Fixnum expected)",
 	       rb_class2name(CLASS_OF(m)));
+      break;
     }
   default:
     rb_raise(rb_eArgError, "wrong number of arguments (%d for 2 or 3)", argc);
+    break;
   }
 
   switch (TYPE(ary)) {
@@ -576,6 +513,7 @@ static VALUE rb_gsl_sf_bessel_sequence_Jnu_e(int argc, VALUE *argv, VALUE obj)
     Data_Get_Struct(ary, gsl_vector, v);
     size = v->size;
     flag = 0;
+    break;
   }
   status = gsl_sf_bessel_sequence_Jnu_e(NUM2DBL(nu), mode, size, v->data);
   if (flag == 1) return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
@@ -614,7 +552,6 @@ static VALUE rb_gsl_sf_bessel_Inu_scaled_e(VALUE obj, VALUE nu, VALUE x)
   return rb_gsl_sf_eval_e_double2(gsl_sf_bessel_Inu_scaled_e, nu, x);
 }
 
-
 /* Irregular Modified Bessel Function - Fractional Order */
 static VALUE rb_gsl_sf_bessel_Knu(VALUE obj, VALUE nu, VALUE x)
 {
