@@ -586,16 +586,16 @@ static VALUE rb_gsl_siman_solver_solve(VALUE obj, VALUE rng,
     Data_Get_Struct(obj, siman_solver, ss);
   }
   if (!rb_obj_is_kind_of(rng, cgsl_rng))
-    rb_raise(rb_eTypeError, "wrong argument type %s (Rng expected)",
+    rb_raise(rb_eTypeError, "wrong argument type %s (GSL::Rng expected)",
 	     rb_class2name(CLASS_OF(rng)));
   if (!rb_obj_is_kind_of(vefunc, cgsl_siman_Efunc))
-    rb_raise(rb_eTypeError, "wrong argument type %s (Siman::Efunc expected)",
+    rb_raise(rb_eTypeError, "wrong argument type %s (GSL::Siman::Efunc expected)",
 	     rb_class2name(CLASS_OF(vefunc)));
   if (!rb_obj_is_kind_of(vstep, cgsl_siman_step))
-    rb_raise(rb_eTypeError, "wrong argument type %s (Siman::Step expected)",
+    rb_raise(rb_eTypeError, "wrong argument type %s (GSL::Siman::Step expected)",
 	     rb_class2name(CLASS_OF(vstep)));
   if (!rb_obj_is_kind_of(vmetric, cgsl_siman_metric))
-    rb_raise(rb_eTypeError, "wrong argument type %s (Siman::Metric expected)",
+    rb_raise(rb_eTypeError, "wrong argument type %s (GSL::Siman::Metric expected)",
 	     rb_class2name(CLASS_OF(vmetric)));
 
   Data_Get_Struct(rng, gsl_rng, r);
@@ -606,13 +606,13 @@ static VALUE rb_gsl_siman_solver_solve(VALUE obj, VALUE rng,
     ss->proc_print = Qnil;
   } else {
     if (!rb_obj_is_kind_of(vprint, cgsl_siman_print))
-      rb_raise(rb_eTypeError, "wrong argument type %s (Siman::Print expected)",
+      rb_raise(rb_eTypeError, "wrong argument type %s (GSL::Siman::Print expected)",
 	       rb_class2name(CLASS_OF(vprint)));
     Data_Get_Struct(vprint, siman_print, print);
     ss->proc_print   = print->proc;
   }
   if (!rb_obj_is_kind_of(vparams, cgsl_siman_params))
-    rb_raise(rb_eTypeError, "wrong argument type %s (Siman::Params expected)",
+    rb_raise(rb_eTypeError, "wrong argument type %s (GSL::Siman::Params expected)",
 	     rb_class2name(CLASS_OF(vparams)));
 
   Data_Get_Struct(vparams, gsl_siman_params_t, params);
@@ -658,12 +658,12 @@ void Init_gsl_siman(VALUE module)
 
   mgsl_siman = rb_define_module_under(module, "Siman");
 
-  cgsl_siman_Efunc = rb_define_class_under(mgsl_siman, "Efunc", rb_cObject);
-  cgsl_siman_step = rb_define_class_under(mgsl_siman, "Step", rb_cObject);
-  cgsl_siman_metric = rb_define_class_under(mgsl_siman, "Metric", rb_cObject);
-  cgsl_siman_print = rb_define_class_under(mgsl_siman, "Print", rb_cObject);
-  cgsl_siman_params = rb_define_class_under(mgsl_siman, "Params", rb_cObject);
-  cgsl_siman_solver = rb_define_class_under(mgsl_siman, "Solver", rb_cObject);
+  cgsl_siman_Efunc = rb_define_class_under(mgsl_siman, "Efunc", cGSL_Object);
+  cgsl_siman_step = rb_define_class_under(mgsl_siman, "Step", cGSL_Object);
+  cgsl_siman_metric = rb_define_class_under(mgsl_siman, "Metric", cGSL_Object);
+  cgsl_siman_print = rb_define_class_under(mgsl_siman, "Print", cGSL_Object);
+  cgsl_siman_params = rb_define_class_under(mgsl_siman, "Params", cGSL_Object);
+  cgsl_siman_solver = rb_define_class_under(mgsl_siman, "Solver", cGSL_Object);
 
   /***** Efunc *****/
   rb_define_singleton_method(cgsl_siman_Efunc, "new", rb_gsl_siman_Efunc_new, -1);
@@ -689,25 +689,25 @@ void Init_gsl_siman(VALUE module)
   rb_define_method(cgsl_siman_params, "params", rb_gsl_siman_params_params, 0);
   rb_define_method(cgsl_siman_params, "n_tries", rb_gsl_siman_params_n_tries, 0);
   rb_define_method(cgsl_siman_params, "set_n_tries", rb_gsl_siman_params_set_n_tries, 1);
-  rb_define_alias(cgsl_siman_params, "n_tries", "set_n_tries");
+  rb_define_alias(cgsl_siman_params, "n_tries=", "set_n_tries");
   rb_define_method(cgsl_siman_params, "iters_fixed_T", rb_gsl_siman_params_iters_fixed_T, 0);
   rb_define_method(cgsl_siman_params, "set_iters_fixed_T", rb_gsl_siman_params_set_iters_fixed_T, 1);
-  rb_define_alias(cgsl_siman_params, "iters_fixed_T", "set_iters_fixed_T");
+  rb_define_alias(cgsl_siman_params, "iters_fixed_T=", "set_iters_fixed_T");
   rb_define_method(cgsl_siman_params, "step_size", rb_gsl_siman_params_step_size, 0);
   rb_define_method(cgsl_siman_params, "set_step_size", rb_gsl_siman_params_set_step_size, 1);
-  rb_define_alias(cgsl_siman_params, "step_size", "set_step_size");
+  rb_define_alias(cgsl_siman_params, "step_size=", "set_step_size");
   rb_define_method(cgsl_siman_params, "k", rb_gsl_siman_params_k, 0);
   rb_define_method(cgsl_siman_params, "set_k", rb_gsl_siman_params_set_k, 1);
-  rb_define_alias(cgsl_siman_params, "k", "set_k");
+  rb_define_alias(cgsl_siman_params, "k=", "set_k");
   rb_define_method(cgsl_siman_params, "t_initial", rb_gsl_siman_params_t_initial, 0);
   rb_define_method(cgsl_siman_params, "set_t_initial", rb_gsl_siman_params_set_t_initial, 1);
-  rb_define_alias(cgsl_siman_params, "t_initial", "set_t_initial");
+  rb_define_alias(cgsl_siman_params, "t_initial=", "set_t_initial");
   rb_define_method(cgsl_siman_params, "mu_t", rb_gsl_siman_params_mu_t, 0);
   rb_define_method(cgsl_siman_params, "set_mu_t", rb_gsl_siman_params_set_mu_t, 1);
-  rb_define_alias(cgsl_siman_params, "mu_t", "set_mu_t");
+  rb_define_alias(cgsl_siman_params, "mu_t=", "set_mu_t");
   rb_define_method(cgsl_siman_params, "t_min", rb_gsl_siman_params_t_min, 0);
   rb_define_method(cgsl_siman_params, "set_t_min", rb_gsl_siman_params_set_t_min, 1);
-  rb_define_alias(cgsl_siman_params, "t_min", "set_t_min");
+  rb_define_alias(cgsl_siman_params, "t_min=", "set_t_min");
 
   /***** solver *****/
   rb_define_singleton_method(cgsl_siman_solver, "new", rb_gsl_siman_solver_new, -1);
