@@ -20,12 +20,9 @@
 #include "rb_gsl_tensor.h"
 #include "rb_gsl_common.h"
 
-EXTERN VALUE cgsl_tensor_index;
-
 #ifdef BASE_DOUBLE
 VALUE cgsl_tensor, cgsl_tensor_int;
 VALUE cgsl_tensor_view, cgsl_tensor_int_view;
-VALUE cgsl_tensor_index;
 #define NUMCONV(x) NUM2DBL(x)
 #define C_TO_VALUE(x) rb_float_new(x)
 #define CHECK_TEN(x) CHECK_TENSOR(x)
@@ -381,7 +378,7 @@ static VALUE FUNCTION(rb_gsl_tensor,max_index)(VALUE obj)
   GSL_TYPE(rbgsl_tensor) *t;
   Data_Get_Struct(obj, GSL_TYPE(rbgsl_tensor), t);
   FUNCTION(gsl_tensor,max_index)(t->tensor, t->indices->data);
-  return Data_Wrap_Struct(cgsl_tensor_index, 0, NULL, t->indices);
+  return Data_Wrap_Struct(cgsl_index, 0, NULL, t->indices);
 }
 
 static VALUE FUNCTION(rb_gsl_tensor,min_index)(VALUE obj)
@@ -389,7 +386,7 @@ static VALUE FUNCTION(rb_gsl_tensor,min_index)(VALUE obj)
   GSL_TYPE(rbgsl_tensor) *t;
   Data_Get_Struct(obj, GSL_TYPE(rbgsl_tensor), t);
   FUNCTION(gsl_tensor,min_index)(t->tensor, t->indices->data);
-  return Data_Wrap_Struct(cgsl_tensor_index, 0, NULL, t->indices);
+  return Data_Wrap_Struct(cgsl_index, 0, NULL, t->indices);
 }
 
 static VALUE FUNCTION(rb_gsl_tensor,minmax_index)(VALUE obj)
@@ -406,8 +403,8 @@ static VALUE FUNCTION(rb_gsl_tensor,minmax_index)(VALUE obj)
   }
   FUNCTION(gsl_tensor,minmax_index)(t->tensor, min->data, max->data);
   return rb_ary_new3(2,
-		     Data_Wrap_Struct(cgsl_tensor_index, 0, gsl_permutation_free, min), 
-		     Data_Wrap_Struct(cgsl_tensor_index, 0, gsl_permutation_free, max));
+		     Data_Wrap_Struct(cgsl_index, 0, gsl_permutation_free, min), 
+		     Data_Wrap_Struct(cgsl_index, 0, gsl_permutation_free, max));
 }
 
 static VALUE FUNCTION(rb_gsl_tensor,isnull)(VALUE obj)
@@ -841,7 +838,7 @@ static VALUE FUNCTION(rb_gsl_tensor,inspect)(VALUE obj)
 {
   VALUE str;
   char buf[64];
-  sprintf(buf, "%s: \n", rb_class2name(CLASS_OF(obj)));
+  sprintf(buf, "%s\n", rb_class2name(CLASS_OF(obj)));
   str = rb_str_new2(buf);
   return rb_str_concat(str, FUNCTION(rb_gsl_tensor,to_s)(obj));
 }
@@ -934,17 +931,30 @@ static VALUE FUNCTION(rb_gsl_tensor,coerce)(VALUE obj, VALUE other)
   }
 }
 
+static VALUE FUNCTION(rb_gsl_tensor,info)(VALUE obj)
+{
+  GSL_TYPE(rbgsl_tensor) *t;
+  char buf[256];
+  Data_Get_Struct(obj, GSL_TYPE(rbgsl_tensor), t);
+  sprintf(buf, "Class:      %s\n", rb_class2name(CLASS_OF(obj)));
+  sprintf(buf, "%sSuperClass: %s\n", buf, rb_class2name(RCLASS(CLASS_OF(obj))->super));
+  sprintf(buf, "%sRank:       %d\n", buf, (int) t->tensor->rank);
+  sprintf(buf, "%sDimension:  %d\n", buf, (int) t->tensor->dimension);
+  sprintf(buf, "%sSize:       %d\n", buf, (int) t->tensor->size);
+  return rb_str_new2(buf);
+}
+
 void FUNCTION(Init_gsl_tensor,init)(VALUE module)
 {
 #ifdef BASE_DOUBLE
-  cgsl_tensor = rb_define_class_under(module, "Tensor", rb_cObject);
-  cgsl_tensor_int = rb_define_class_under(cgsl_tensor, "Int", rb_cObject);
+  cgsl_tensor = rb_define_class_under(module, "Tensor", cGSL_Object);
+  cgsl_tensor_int = rb_define_class_under(cgsl_tensor, "Int", cGSL_Object);
   cgsl_tensor_view = rb_define_class_under(cgsl_tensor, "View", cgsl_tensor);
   cgsl_tensor_int_view = rb_define_class_under(cgsl_tensor_int, "View",
 					       cgsl_tensor_int);
-
-  cgsl_tensor_index = rb_define_class_under(cgsl_tensor, "Index", 
-					    cgsl_permutation);
+  /*
+  cgsl_index = rb_define_class_under(cgsl_tensor, "Index", 
+  cgsl_permutation);*/
 #endif
 
   rb_define_singleton_method(GSL_TYPE(cgsl_tensor), "new",
@@ -1095,6 +1105,9 @@ void FUNCTION(Init_gsl_tensor,init)(VALUE module)
 
   rb_define_method(GSL_TYPE(cgsl_tensor), "coerce",
 			     FUNCTION(rb_gsl_tensor,coerce), 1);
+
+  rb_define_method(GSL_TYPE(cgsl_tensor), "info", 
+		   FUNCTION(rb_gsl_tensor,info), 0);
 }
 
 #undef NUMCONV
