@@ -15,14 +15,15 @@
 EXTERN VALUE cgsl_complex;
 static VALUE rb_gsl_vector_complex_inner_product(int argc, VALUE *argv, VALUE obj);
 static VALUE rb_gsl_vector_complex_product_to_m(int argc, VALUE *argv, VALUE obj);
+void get_range_beg_en_n(VALUE range, int *beg, int *en, size_t *n, int *step);
 
 static VALUE rb_gsl_vector_complex_new(int argc, VALUE *argv, VALUE klass)
 {
   VALUE tmp;
   gsl_vector_complex *v = NULL;
+  gsl_vector *x, *y;
   gsl_complex z, *z2 = NULL;
-  size_t n;
-  size_t i;
+  size_t n, i;
   switch (argc) {
   case 1:
     switch (TYPE(argv[0])) {
@@ -59,6 +60,18 @@ static VALUE rb_gsl_vector_complex_new(int argc, VALUE *argv, VALUE klass)
     }
     break;
   default:
+    if (argc == 2 && (VECTOR_P(argv[0]) && VECTOR_P(argv[1]))) {
+      Data_Get_Struct(argv[0], gsl_vector, x);
+      Data_Get_Struct(argv[1], gsl_vector, y);
+      n = GSL_MIN_INT(x->size, y->size);
+      v = gsl_vector_complex_alloc(n);
+      for (i = 0; i < n; i++) {
+	z.dat[0] = gsl_vector_get(x, i);
+	z.dat[1] = gsl_vector_get(y, i);
+	gsl_vector_complex_set(v, i, z);
+      }
+      break;
+    }
     n = argc;
     v = gsl_vector_complex_alloc(n);
     if (v == NULL) rb_raise(rb_eNoMemError, "gsl_vector_complex_alloc failed");
@@ -78,7 +91,6 @@ static VALUE rb_gsl_vector_complex_new(int argc, VALUE *argv, VALUE klass)
     }
     break;
   }
-
   return Data_Wrap_Struct(klass, 0, gsl_vector_complex_free, v);
 }
 
@@ -117,21 +129,70 @@ static VALUE rb_gsl_vector_complex_owner(VALUE obj)
   return INT2FIX(v->owner);
 }
 
-static VALUE rb_gsl_vector_complex_get(VALUE obj, VALUE ii)
+static VALUE rb_gsl_vector_complex_ptr(VALUE obj, VALUE i)
 {
   gsl_vector_complex *v = NULL;
-  gsl_complex *c = NULL;
-  int i;
-  size_t index;
-  CHECK_FIXNUM(ii);
   Data_Get_Struct(obj, gsl_vector_complex, v);
-  i = FIX2INT(ii);
-  if (i < 0) index = (size_t) (v->size + i);
-  else index = (size_t) i;
-  c = ALLOC(gsl_complex);
-  *c = gsl_vector_complex_get(v, index);
-  return Data_Wrap_Struct(cgsl_complex, 0, rb_gsl_complex_free, c);
+  return Data_Wrap_Struct(cgsl_complex, 0, NULL, gsl_vector_complex_ptr(v, FIX2INT(i)));
+}
 
+static VALUE rb_gsl_vector_complex_get(int argc, VALUE *argv, VALUE obj)
+{
+  gsl_vector_complex *v = NULL, *vnew;
+  gsl_complex *c = NULL, z;
+  int i;
+  int beg, en, step;
+  size_t index, n, j;
+  Data_Get_Struct(obj, gsl_vector_complex, v);
+  switch (argc) {
+  case 1:
+    switch (TYPE(argv[0])) {
+    case T_FIXNUM:
+      CHECK_FIXNUM(argv[0]);
+      i = FIX2INT(argv[0]);
+      if (i < 0) index = (size_t) (v->size + i);
+      else index = (size_t) i;
+      c = ALLOC(gsl_complex);
+      *c = gsl_vector_complex_get(v, index);
+      return Data_Wrap_Struct(cgsl_complex, 0, free, c);
+      break;
+    case T_ARRAY:
+      vnew = gsl_vector_complex_alloc(RARRAY(argv[0])->len);
+      for (j = 0; j < vnew->size; j++) {
+	i = FIX2INT(rb_ary_entry(argv[0], j));
+	if (i < 0) i = v->size + i;
+	gsl_vector_complex_set(vnew, j, gsl_vector_complex_get(v, i));
+      }
+      return Data_Wrap_Struct(cgsl_vector_complex, 0, gsl_vector_complex_free, vnew);
+      break;
+    default:
+      if (CLASS_OF(argv[0]) == rb_cRange) {
+	get_range_beg_en_n(argv[0], &beg, &en, &n, &step);
+	vnew = gsl_vector_complex_alloc(n);
+	for (j = 0; j < n; j++) {
+	  z = gsl_vector_complex_get(v, j+beg);
+	  gsl_vector_complex_set(vnew, j, z);
+	}
+	return Data_Wrap_Struct(cgsl_vector_complex, 0, gsl_vector_complex_free, vnew);
+      }
+      break;
+    }
+    break;
+  case 0:
+    rb_raise(rb_eArgError, "number of arguments must be > 0");
+    break;
+  default:
+    vnew = gsl_vector_complex_alloc(argc);
+    for (j = 0; j < argc; j++) {
+      i = FIX2INT(argv[j]);
+      if (i >= 0) z = gsl_vector_complex_get(v, i);
+      else z = gsl_vector_complex_get(v, v->size+i);
+      gsl_vector_complex_set(vnew, j, z);
+    }
+    return Data_Wrap_Struct(cgsl_vector_complex, 0, gsl_vector_complex_free, vnew);
+    break;
+  }
+  return Qnil;
 }
 
 static VALUE rb_gsl_vector_complex_set(int argc, VALUE *argv, VALUE obj)
@@ -157,6 +218,7 @@ static VALUE rb_gsl_vector_complex_set(int argc, VALUE *argv, VALUE obj)
       default:
 	CHECK_COMPLEX(argv[i]);
 	Data_Get_Struct(argv[i], gsl_complex, z);
+	tmp = *z;
 	break;
       }
       gsl_vector_complex_set(v, i, tmp);
@@ -168,6 +230,7 @@ static VALUE rb_gsl_vector_complex_set(int argc, VALUE *argv, VALUE obj)
   case 2:
     if (rb_obj_is_kind_of(argv[1], cgsl_complex)) {
       Data_Get_Struct(argv[1], gsl_complex, z);
+      tmp = *z;
     } else {
       switch(TYPE(argv[1])) {
       case T_ARRAY:
@@ -182,13 +245,13 @@ static VALUE rb_gsl_vector_complex_set(int argc, VALUE *argv, VALUE obj)
       default:
 	CHECK_COMPLEX(argv[1]);
 	Data_Get_Struct(argv[1], gsl_complex, z);
+	tmp = *z;
 	break;
       }
     }
     break;
   case 3:
     tmp = gsl_complex_rect(NUM2DBL(argv[1]), NUM2DBL(argv[2]));
-    z = &tmp;
     break;
   default:
     rb_raise(rb_eArgError, "wrong number of arguments");
@@ -209,19 +272,18 @@ static VALUE rb_gsl_vector_complex_set_all(int argc, VALUE *argv, VALUE obj)
   case 1:
     if (rb_obj_is_kind_of(argv[0], cgsl_complex)) {
       Data_Get_Struct(argv[0], gsl_complex, z);
+      tmp = *z;
     } else {
       switch(TYPE(argv[0])) {
       case T_ARRAY:
 	tmp.dat[0] = NUM2DBL(rb_ary_entry(argv[0], 0));
 	tmp.dat[1] = NUM2DBL(rb_ary_entry(argv[0], 1));
-	z = &tmp;
 	break;
       case T_FLOAT:
       case T_FIXNUM:
       case T_BIGNUM:
 	tmp.dat[0] = NUM2DBL(argv[0]);
 	tmp.dat[1] = 0.0;
-	z = &tmp;
 	break;
       default:
 	rb_raise(rb_eTypeError, 
@@ -232,14 +294,13 @@ static VALUE rb_gsl_vector_complex_set_all(int argc, VALUE *argv, VALUE obj)
     break;
   case 2:
     tmp = gsl_complex_rect(NUM2DBL(argv[0]), NUM2DBL(argv[1]));
-    z = &tmp;
     break;
   default:
     rb_raise(rb_eArgError, "wrong number of arguments");
     break;
   }
 
-  gsl_vector_complex_set_all(v, *z);
+  gsl_vector_complex_set_all(v, tmp);
 
   return obj;
 }
@@ -303,7 +364,7 @@ static VALUE rb_gsl_vector_complex_fprintf(int argc, VALUE *argv, VALUE obj)
     Check_Type(argv[1], T_STRING);
     status = gsl_vector_complex_fprintf(fp, h, STR2CSTR(argv[1]));
   } else {
-    status = gsl_vector_complex_fprintf(fp, h, "%g");
+    status = gsl_vector_complex_fprintf(fp, h, "%4.3e");
   }
   if (flag == 1) fclose(fp);
   return INT2FIX(status);
@@ -319,7 +380,7 @@ static VALUE rb_gsl_vector_complex_printf(int argc, VALUE *argv, VALUE obj)
     Check_Type(argv[0], T_STRING);
     status = gsl_vector_complex_fprintf(stdout, h, STR2CSTR(argv[0]));
   } else {
-    status = gsl_vector_complex_fprintf(stdout, h, "%g");
+    status = gsl_vector_complex_fprintf(stdout, h, "%4.3e");
   }
   return INT2FIX(status);
 }
@@ -436,7 +497,7 @@ static VALUE rb_gsl_vector_complex_to_a2(VALUE obj)
   for (i = 0; i < c->size; i++) {
     z = GSL_COMPLEX_AT(c, i);
     znew = make_complex(z->dat[0], z->dat[1]);
-    vz = Data_Wrap_Struct(cgsl_complex, 0, rb_gsl_complex_free, znew);
+    vz = Data_Wrap_Struct(cgsl_complex, 0, free, znew);
     rb_ary_store(ary, i, vz);
   }
   return ary;
@@ -649,13 +710,9 @@ static VALUE rb_gsl_vector_complex_to_real(VALUE obj)
 
 enum {
   GSL_VECTOR_COMPLEX_ADD,
-  GSL_VECTOR_COMPLEX_ADD2,
   GSL_VECTOR_COMPLEX_SUB,
-  GSL_VECTOR_COMPLEX_SUB2,
   GSL_VECTOR_COMPLEX_MUL,
-  GSL_VECTOR_COMPLEX_MUL2,
   GSL_VECTOR_COMPLEX_DIV,
-  GSL_VECTOR_COMPLEX_DIV2,
 };
 
 static VALUE rb_gsl_vector_complex_arithmetics(int flag, VALUE obj, VALUE bb);
@@ -677,37 +734,21 @@ static VALUE rb_gsl_vector_complex_arithmetics(int flag, VALUE obj, VALUE bb)
       gsl_vector_complex_add_constant(cvnew, z);
       return Data_Wrap_Struct(VECTOR_COMPLEX_ROW_COL(obj), 0, gsl_vector_complex_free, cvnew);
       break;
-    case GSL_VECTOR_COMPLEX_ADD2:
-      gsl_vector_complex_add_constant(cv, z);
-      return obj;
-      break;
     case GSL_VECTOR_COMPLEX_SUB:
       cvnew = make_vector_complex_clone(cv);
       gsl_vector_complex_add_constant(cvnew, gsl_complex_negative(z));
       return Data_Wrap_Struct(VECTOR_COMPLEX_ROW_COL(obj), 0, gsl_vector_complex_free, cvnew);
       break;
-    case GSL_VECTOR_COMPLEX_SUB2:
-      gsl_vector_complex_add_constant(cv, gsl_complex_negative(z));
-      return obj;
-      break;
     case GSL_VECTOR_COMPLEX_MUL:
       cvnew = make_vector_complex_clone(cv);
       gsl_vector_complex_scale(cvnew, z);
       return Data_Wrap_Struct(VECTOR_COMPLEX_ROW_COL(obj), 0, gsl_vector_complex_free, cvnew);
       break;
-    case GSL_VECTOR_COMPLEX_MUL2:
-      gsl_vector_complex_scale(cv, z);
-      return obj;
-      break;
     case GSL_VECTOR_COMPLEX_DIV:
       cvnew = make_vector_complex_clone(cv);
       gsl_vector_complex_scale(cvnew, gsl_complex_inverse(z));
       return Data_Wrap_Struct(VECTOR_COMPLEX_ROW_COL(obj), 0, gsl_vector_complex_free, cvnew);
       break;
-    case GSL_VECTOR_COMPLEX_DIV2:
-      gsl_vector_complex_scale(cv, gsl_complex_inverse(z));
-      return obj;
-      break;
     default:
       rb_raise(rb_eRuntimeError, "unknown operation");
       break;
@@ -722,34 +763,18 @@ static VALUE rb_gsl_vector_complex_arithmetics(int flag, VALUE obj, VALUE bb)
 	cvnew = make_vector_complex_clone(cv);
 	gsl_vector_complex_add(cvnew, cb);
 	break;
-      case GSL_VECTOR_COMPLEX_ADD2:
-	gsl_vector_complex_add(cv, cb);
-	return obj;
-	break;
       case GSL_VECTOR_COMPLEX_SUB:
 	cvnew = make_vector_complex_clone(cv);
 	gsl_vector_complex_sub(cvnew, cb);
 	break;
-      case GSL_VECTOR_COMPLEX_SUB2:
-	gsl_vector_complex_sub(cv, cb);
-	return obj;
-	break;
       case GSL_VECTOR_COMPLEX_MUL:
 	cvnew = make_vector_complex_clone(cv);
 	gsl_vector_complex_mul(cvnew, cb);
 	break;
-      case GSL_VECTOR_COMPLEX_MUL2:
-	gsl_vector_complex_mul(cv, cb);
-	return obj;
-	break;
       case GSL_VECTOR_COMPLEX_DIV:
 	cvnew = make_vector_complex_clone(cv);
 	gsl_vector_complex_div(cvnew, cb);
 	break;
-      case GSL_VECTOR_COMPLEX_DIV2:
-	gsl_vector_complex_div(cv, cb);
-	return obj;
-	break;
       default:
 	rb_raise(rb_eRuntimeError, "unknown operation");
 	break;
@@ -766,34 +791,18 @@ static VALUE rb_gsl_vector_complex_arithmetics(int flag, VALUE obj, VALUE bb)
 	cvnew = make_vector_complex_clone(cv);
 	gsl_vector_complex_add(cvnew, cb);
 	break;
-      case GSL_VECTOR_COMPLEX_ADD2:
-	gsl_vector_complex_add(cv, cb);
-	return obj;
-	break;
       case GSL_VECTOR_COMPLEX_SUB:
 	cvnew = make_vector_complex_clone(cv);
 	gsl_vector_complex_sub(cvnew, cb);
 	break;
-      case GSL_VECTOR_COMPLEX_SUB2:
-	gsl_vector_complex_sub(cv, cb);
-	return obj;
-	break;
       case GSL_VECTOR_COMPLEX_MUL:
 	cvnew = make_vector_complex_clone(cv);
 	gsl_vector_complex_mul(cvnew, cb);
 	break;
-      case GSL_VECTOR_COMPLEX_MUL2:
-	gsl_vector_complex_mul(cv, cb);
-	return obj;
-	break;
       case GSL_VECTOR_COMPLEX_DIV:
 	cvnew = make_vector_complex_clone(cv);
 	gsl_vector_complex_div(cvnew, cb);
 	break;
-      case GSL_VECTOR_COMPLEX_DIV2:
-	gsl_vector_complex_div(cv, cb);
-	return obj;
-	break;
       default:
 	rb_raise(rb_eRuntimeError, "unknown operation");
 	break;
@@ -809,34 +818,18 @@ static VALUE rb_gsl_vector_complex_arithmetics(int flag, VALUE obj, VALUE bb)
 	cvnew = make_vector_complex_clone(cv);
 	gsl_vector_complex_add_constant(cvnew, *c);
 	break;
-      case GSL_VECTOR_COMPLEX_ADD2:
-	gsl_vector_complex_add_constant(cv, *c);
-	return obj;
-	break;
       case GSL_VECTOR_COMPLEX_SUB:
 	cvnew = make_vector_complex_clone(cv);
 	gsl_vector_complex_add_constant(cvnew, gsl_complex_negative(*c));
 	break;
-      case GSL_VECTOR_COMPLEX_SUB2:
-	gsl_vector_complex_add_constant(cv, gsl_complex_negative(*c));
-	return obj;
-	break;
       case GSL_VECTOR_COMPLEX_MUL:
 	cvnew = make_vector_complex_clone(cv);
 	gsl_vector_complex_scale(cvnew, *c);
 	break;
-      case GSL_VECTOR_COMPLEX_MUL2:
-	gsl_vector_complex_scale(cv, *c);
-	return obj;
-	break;
       case GSL_VECTOR_COMPLEX_DIV:
 	cvnew = make_vector_complex_clone(cv);
 	gsl_vector_complex_scale(cvnew, gsl_complex_inverse(*c));
 	break;
-      case GSL_VECTOR_COMPLEX_DIV2:
-	gsl_vector_complex_scale(cv, gsl_complex_inverse(*c));
-	return obj;
-	break;
       default:
 	rb_raise(rb_eRuntimeError, "unknown operation");
 	break;
@@ -947,7 +940,7 @@ static VALUE rb_gsl_vector_complex_inner_product(int argc, VALUE *argv, VALUE ob
   }
   z = ALLOC(gsl_complex);
   *z = prod;
-  return Data_Wrap_Struct(cgsl_complex, 0, rb_gsl_complex_free, z);
+  return Data_Wrap_Struct(cgsl_complex, 0, free, z);
 }
 
 static VALUE rb_gsl_vector_complex_product_to_m(int argc, VALUE *argv, VALUE obj)
@@ -1031,9 +1024,354 @@ static VALUE rb_gsl_vector_complex_scale_bang(VALUE obj, VALUE s)
   gsl_vector_complex_scale(m, *z);
   return obj;
 }
+
+/*****/
+static VALUE rb_gsl_vector_complex_XXX(VALUE obj, double (*f)(gsl_complex))
+{
+  gsl_vector_complex *m;
+  gsl_vector *v;
+  gsl_complex c;
+  size_t i;
+  Data_Get_Struct(obj, gsl_vector_complex, m);
+  v = gsl_vector_alloc(m->size);
+  for (i = 0; i < m->size; i++) {
+    c = gsl_vector_complex_get(m, i);
+    gsl_vector_set(v, i, (*f)(c));
+  }
+  return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
+}
+
+static VALUE rb_gsl_vector_complex_XXXz(VALUE obj, gsl_complex (*f)(gsl_complex))
+{
+  gsl_vector_complex *m, *v;
+  gsl_complex c;
+  size_t i;
+  Data_Get_Struct(obj, gsl_vector_complex, m);
+  v = gsl_vector_complex_alloc(m->size);
+  for (i = 0; i < m->size; i++) {
+    c = gsl_vector_complex_get(m, i);
+    gsl_vector_complex_set(v, i, (*f)(c));
+  }
+  return Data_Wrap_Struct(cgsl_vector_complex, 0, gsl_vector_complex_free, v);
+}
+
+static VALUE rb_gsl_vector_complex_XXXz2(VALUE obj, VALUE a, 
+					 gsl_complex (*f)(gsl_complex, gsl_complex))
+{
+  gsl_vector_complex *m, *v;
+  gsl_complex c, *z;
+  size_t i;
+  CHECK_COMPLEX(a);
+  Data_Get_Struct(obj, gsl_vector_complex, m);
+  Data_Get_Struct(a, gsl_complex, z);
+  v = gsl_vector_complex_alloc(m->size);
+  for (i = 0; i < m->size; i++) {
+    c = gsl_vector_complex_get(m, i);
+    gsl_vector_complex_set(v, i, (*f)(c, *z));
+  }
+  return Data_Wrap_Struct(cgsl_vector_complex, 0, gsl_vector_complex_free, v);
+}
+
+static VALUE rb_gsl_vector_complex_abs2(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXX(obj, gsl_complex_abs2);
+}
+
+static VALUE rb_gsl_vector_complex_abs(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXX(obj, gsl_complex_abs);
+}
+
+static VALUE rb_gsl_vector_complex_logabs(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXX(obj, gsl_complex_logabs);
+}
+
+static VALUE rb_gsl_vector_complex_arg(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXX(obj, gsl_complex_arg);
+}
+
+static VALUE rb_gsl_vector_complex_sqrt(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_sqrt);
+}
+
+static VALUE rb_gsl_vector_complex_exp(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_exp);
+}
+
+static VALUE rb_gsl_vector_complex_pow(VALUE obj, VALUE a)
+{
+  return rb_gsl_vector_complex_XXXz2(obj, a, gsl_complex_pow);
+}
+
+static VALUE rb_gsl_vector_complex_log(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_log);
+}
+
+static VALUE rb_gsl_vector_complex_log10(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_log10);
+}
+
+static VALUE rb_gsl_vector_complex_log_b(VALUE obj, VALUE a)
+{
+  return rb_gsl_vector_complex_XXXz2(obj, a, gsl_complex_log_b);
+}
+
+static VALUE rb_gsl_vector_complex_sin(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_sin);
+}
+
+static VALUE rb_gsl_vector_complex_cos(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_cos);
+}
+
+static VALUE rb_gsl_vector_complex_tan(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_tan);
+}
+
+static VALUE rb_gsl_vector_complex_sec(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_sec);
+}
+
+static VALUE rb_gsl_vector_complex_csc(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_csc);
+}
+static VALUE rb_gsl_vector_complex_cot(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_cot);
+}
+
+static VALUE rb_gsl_vector_complex_arcsin(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_arcsin);
+}
+
+static VALUE rb_gsl_vector_complex_arccos(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_arccos);
+}
+
+static VALUE rb_gsl_vector_complex_arctan(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_arctan);
+}
+
+static VALUE rb_gsl_vector_complex_arcsec(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_arcsec);
+}
+
+static VALUE rb_gsl_vector_complex_arccsc(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_arccsc);
+}
+
+static VALUE rb_gsl_vector_complex_arccot(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_arccot);
+}
+
+static VALUE rb_gsl_vector_complex_sinh(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_sinh);
+}
+
+static VALUE rb_gsl_vector_complex_cosh(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_cosh);
+}
+
+static VALUE rb_gsl_vector_complex_tanh(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_tanh);
+}
+
+static VALUE rb_gsl_vector_complex_sech(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_sech);
+}
+
+static VALUE rb_gsl_vector_complex_csch(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_csch);
+}
+
+static VALUE rb_gsl_vector_complex_coth(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_coth);
+}
+
+static VALUE rb_gsl_vector_complex_arcsinh(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_arcsinh);
+}
+
+static VALUE rb_gsl_vector_complex_arccosh(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_arccosh);
+}
+
+static VALUE rb_gsl_vector_complex_arctanh(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_arctanh);
+}
+
+static VALUE rb_gsl_vector_complex_arcsech(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_arcsech);
+}
+
+static VALUE rb_gsl_vector_complex_arccsch(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_arccsch);
+}
+
+static VALUE rb_gsl_vector_complex_arccoth(VALUE obj)
+{
+  return rb_gsl_vector_complex_XXXz(obj, gsl_complex_arccoth);
+}
+
+static VALUE rb_gsl_vector_complex_shift(VALUE obj)
+{
+  gsl_vector_complex *v = NULL;
+  gsl_complex *z = NULL;
+  Data_Get_Struct(obj, gsl_vector_complex, v);
+  if (v->stride != 1) rb_raise(rb_eRuntimeError, "vector must have stride 1");
+  if (v->size == 0) return Qnil;
+  z = ALLOC(gsl_complex);
+  *z = gsl_vector_complex_get(v, 0);
+  v->size -= 1;
+  memmove(v->block->data, v->block->data+2, sizeof(double)*v->size*2);
+  return Data_Wrap_Struct(cgsl_complex, 0, free, z);
+}
+
+static VALUE rb_gsl_vector_complex_pop(VALUE obj)
+{
+  gsl_vector_complex *v = NULL;
+  gsl_complex *z = NULL;
+  Data_Get_Struct(obj, gsl_vector_complex, v);
+  if (v->stride != 1) rb_raise(rb_eRuntimeError, "vector must have stride 1");
+  if (v->size == 0) return Qnil;
+  z = ALLOC(gsl_complex);
+  *z = gsl_vector_complex_get(v, v->size-1);
+  v->size -= 1;
+  return Data_Wrap_Struct(cgsl_complex, 0, free, z);
+}
+
+static VALUE rb_gsl_vector_complex_unshift(VALUE obj, VALUE x)
+{
+  gsl_vector_complex *v = NULL;
+  gsl_block_complex *b = NULL, *bnew = NULL;
+  gsl_complex *z, c;
+  switch (TYPE(x)) {
+  case T_FIXNUM: case T_FLOAT:
+    c.dat[0] = NUM2DBL(x); c.dat[1] = 0.0;
+    break;
+  case T_ARRAY:
+    c.dat[0] = NUM2DBL(rb_ary_entry(x, 0));
+    c.dat[1] = NUM2DBL(rb_ary_entry(x, 1));
+    break;
+  default:
+    CHECK_COMPLEX(x);
+    Data_Get_Struct(x, gsl_complex, z);
+    c = *z;
+    break;
+  }
+  Data_Get_Struct(obj, gsl_vector_complex, v);
+  if (v->stride != 1) rb_raise(rb_eRuntimeError, "vector must have stride 1");
+  b = v->block;
+  if (b->size < (v->size+1)*2) {
+    bnew = gsl_block_complex_alloc(v->size + 1);
+    memcpy(bnew->data+2, b->data, sizeof(double)*b->size*2);
+    gsl_block_complex_free(b);
+  } else {
+    bnew = b;
+    memmove(bnew->data+2, v->data, sizeof(double)*v->size*2);
+  }
+  v->data = bnew->data;
+  v->block = bnew;
+  v->size += 1;
+  gsl_vector_complex_set(v, 0, c);
+  return obj;
+}
+
+static VALUE rb_gsl_vector_complex_push(VALUE obj, VALUE x)
+{
+  gsl_vector_complex *v = NULL;
+  gsl_block_complex *b = NULL, *bnew = NULL;
+  gsl_complex *z, c;
+  switch (TYPE(x)) {
+  case T_FIXNUM: case T_FLOAT:
+    c.dat[0] = NUM2DBL(x); c.dat[1] = 0.0;
+    break;
+  case T_ARRAY:
+    c.dat[0] = NUM2DBL(rb_ary_entry(x, 0));
+    c.dat[1] = NUM2DBL(rb_ary_entry(x, 1));
+    break;
+  default:
+    CHECK_COMPLEX(x);
+    Data_Get_Struct(x, gsl_complex, z);
+    c = *z;
+    break;
+  }
+  Data_Get_Struct(obj, gsl_vector_complex, v);
+  if (v->stride != 1) rb_raise(rb_eRuntimeError, "vector must have stride 1");
+  b = v->block;
+  if (b->size < (v->size+1)*2) {
+    bnew = gsl_block_complex_alloc(v->size + 1);
+    memcpy(bnew->data, b->data, sizeof(double)*b->size*2);
+    gsl_block_complex_free(b);
+  } else {
+    bnew = b;
+    memmove(bnew->data+2, b->data, sizeof(double)*v->size*2);
+  }
+  v->block = bnew;
+  v->size += 1;
+  v->data = bnew->data;
+  gsl_vector_complex_set(v, v->size-1, c);
+  return obj;
+}
+
+static VALUE rb_gsl_vector_complex_concat(VALUE obj, VALUE other)
+{
+  gsl_vector_complex *v = NULL, *v2 = NULL;
+  gsl_block_complex *bnew = NULL;
+  CHECK_VECTOR_COMPLEX(other);
+  Data_Get_Struct(obj, gsl_vector_complex, v);
+  if (v->stride != 1) rb_raise(rb_eRuntimeError, "vector must have stride 1");
+  Data_Get_Struct(other, gsl_vector_complex, v2);
+  bnew = gsl_block_complex_alloc(v->size + v2->size);
+  memcpy(bnew->data, v->block->data, sizeof(double)*v->block->size*2);
+  memcpy(bnew->data + v->block->size*2, v2->block->data, 
+	 sizeof(double)*v2->block->size*2);
+  gsl_block_complex_free(v->block);
+  v->size += v2->size;
+  v->block = bnew;
+  v->data = bnew->data;
+  return obj;
+}
+
+static VALUE rb_gsl_vector_complex_block(VALUE obj)
+{
+  gsl_vector_complex *v = NULL;
+  Data_Get_Struct(obj, gsl_vector_complex, v);
+  return Data_Wrap_Struct(cgsl_block_complex, 0, NULL, v->block);
+}
+
 void Init_gsl_vector_complex(VALUE module)
 {
   rb_define_singleton_method(cgsl_vector_complex, "new", rb_gsl_vector_complex_new, -1);
+  rb_define_singleton_method(cgsl_vector_complex, "[]", rb_gsl_vector_complex_new, -1);
   rb_define_singleton_method(cgsl_vector_complex, "alloc", rb_gsl_vector_complex_new, -1);
   rb_define_singleton_method(cgsl_vector_complex, "calloc", rb_gsl_vector_complex_calloc, 1);
 
@@ -1044,9 +1382,9 @@ void Init_gsl_vector_complex(VALUE module)
   rb_define_alias(cgsl_vector_complex, "len", "size");
   rb_define_method(cgsl_vector_complex, "stride", rb_gsl_vector_complex_stride, 0);
   rb_define_method(cgsl_vector_complex, "owner", rb_gsl_vector_complex_owner, 0);
-  rb_define_method(cgsl_vector_complex, "get", rb_gsl_vector_complex_get, 1);
-
+  rb_define_method(cgsl_vector_complex, "get", rb_gsl_vector_complex_get, -1);
   rb_define_alias(cgsl_vector_complex, "[]", "get");
+  rb_define_method(cgsl_vector_complex, "ptr", rb_gsl_vector_complex_ptr, 1);
 
   rb_define_method(cgsl_vector_complex, "set", rb_gsl_vector_complex_set, -1);
   rb_define_alias(cgsl_vector_complex, "[]=", "set");
@@ -1123,10 +1461,62 @@ void Init_gsl_vector_complex(VALUE module)
   rb_define_singleton_method(cgsl_vector, "inner_product", rb_gsl_vector_complex_inner_product, -1);
   rb_define_singleton_method(cgsl_vector_complex, "dot", rb_gsl_vector_complex_inner_product, -1);
   rb_define_method(cgsl_vector_complex, "inner_product", rb_gsl_vector_complex_inner_product, -1);
-  rb_define_alias(cgsl_vector_complex, "dot", "inner_product");
+  /*  rb_define_alias(cgsl_vector_complex, "dot", "inner_product");*/
 
   /*****/
   rb_define_method(cgsl_vector_complex, "-@", rb_gsl_vector_complex_uminus, 0);
   rb_define_method(cgsl_vector_complex, "+@", rb_gsl_vector_complex_uplus, 0);
+
+  rb_define_method(cgsl_vector_complex, "abs2", rb_gsl_vector_complex_abs2, 0);
+  rb_define_alias(cgsl_vector_complex, "square", "abs2");
+  rb_define_method(cgsl_vector_complex, "abs", rb_gsl_vector_complex_abs, 0);
+  rb_define_alias(cgsl_vector_complex, "amp", "abs");
+  rb_define_alias(cgsl_vector_complex, "mag", "abs");
+  rb_define_method(cgsl_vector_complex, "arg", rb_gsl_vector_complex_arg, 0);
+  rb_define_alias(cgsl_vector_complex, "phase", "arg");
+  rb_define_method(cgsl_vector_complex, "logabs", rb_gsl_vector_complex_logabs, 0);
+
+  rb_define_method(cgsl_vector_complex, "sqrt", rb_gsl_vector_complex_sqrt, 0);
+  rb_define_method(cgsl_vector_complex, "exp", rb_gsl_vector_complex_exp, 0);
+  rb_define_method(cgsl_vector_complex, "pow", rb_gsl_vector_complex_pow, 1);
+  rb_define_method(cgsl_vector_complex, "log", rb_gsl_vector_complex_log, 0);
+  rb_define_method(cgsl_vector_complex, "log10", rb_gsl_vector_complex_log10, 0);
+  rb_define_method(cgsl_vector_complex, "log_b", rb_gsl_vector_complex_log_b, 1);
+
+  rb_define_method(cgsl_vector_complex, "sin", rb_gsl_vector_complex_sin, 0);
+  rb_define_method(cgsl_vector_complex, "cos", rb_gsl_vector_complex_cos, 0);
+  rb_define_method(cgsl_vector_complex, "tan", rb_gsl_vector_complex_tan, 0);
+  rb_define_method(cgsl_vector_complex, "sec", rb_gsl_vector_complex_sec, 0);
+  rb_define_method(cgsl_vector_complex, "csc", rb_gsl_vector_complex_csc, 0);
+  rb_define_method(cgsl_vector_complex, "cot", rb_gsl_vector_complex_cot, 0);
+
+  rb_define_method(cgsl_vector_complex, "arcsin", rb_gsl_vector_complex_arcsin, 0);
+  rb_define_method(cgsl_vector_complex, "arccos", rb_gsl_vector_complex_arccos, 0);
+  rb_define_method(cgsl_vector_complex, "arctan", rb_gsl_vector_complex_arctan, 0);
+  rb_define_method(cgsl_vector_complex, "arcsec", rb_gsl_vector_complex_arcsec, 0);
+  rb_define_method(cgsl_vector_complex, "arccsc", rb_gsl_vector_complex_arccsc, 0);
+  rb_define_method(cgsl_vector_complex, "arccot", rb_gsl_vector_complex_arccot, 0);
+
+  rb_define_method(cgsl_vector_complex, "sinh", rb_gsl_vector_complex_sinh, 0);
+  rb_define_method(cgsl_vector_complex, "cosh", rb_gsl_vector_complex_cosh, 0);
+  rb_define_method(cgsl_vector_complex, "tanh", rb_gsl_vector_complex_tanh, 0);
+  rb_define_method(cgsl_vector_complex, "sech", rb_gsl_vector_complex_sech, 0);
+  rb_define_method(cgsl_vector_complex, "csch", rb_gsl_vector_complex_csch, 0);
+  rb_define_method(cgsl_vector_complex, "coth", rb_gsl_vector_complex_coth, 0);
+
+  rb_define_method(cgsl_vector_complex, "arcsinh", rb_gsl_vector_complex_arcsinh, 0);
+  rb_define_method(cgsl_vector_complex, "arccosh", rb_gsl_vector_complex_arccosh, 0);
+  rb_define_method(cgsl_vector_complex, "arctanh", rb_gsl_vector_complex_arctanh, 0);
+  rb_define_method(cgsl_vector_complex, "arcsech", rb_gsl_vector_complex_arcsech, 0);
+  rb_define_method(cgsl_vector_complex, "arccsch", rb_gsl_vector_complex_arccsch, 0);
+  rb_define_method(cgsl_vector_complex, "arccoth", rb_gsl_vector_complex_arccoth, 0);
+
+  /*****/
+  rb_define_method(cgsl_vector_complex, "shift", rb_gsl_vector_complex_shift, 0);
+  rb_define_method(cgsl_vector_complex, "pop", rb_gsl_vector_complex_pop, 0);
+  rb_define_method(cgsl_vector_complex, "unshift", rb_gsl_vector_complex_unshift, 1);
+  rb_define_method(cgsl_vector_complex, "push", rb_gsl_vector_complex_push, 1);
+  rb_define_method(cgsl_vector_complex, "concat", rb_gsl_vector_complex_concat, 1);
+  rb_define_method(cgsl_vector_complex, "block", rb_gsl_vector_complex_block, 0);
 }
 
