@@ -9,6 +9,7 @@
   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY.
 */
+
 #include "rb_gsl_config.h"
 #include "rb_gsl_array.h"
 #include "rb_gsl_histogram.h"
@@ -35,9 +36,10 @@ static VALUE rb_gsl_vector_Xspace(double min, double max, int i,
   return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
 }
 
-static gsl_vector* gsl_vector_linspace(const double min, const double max, const size_t n)
+static gsl_vector* gsl_vector_linspace(const double min, 
+				       const double max, const size_t n)
 {
-  gsl_vector *v;
+  gsl_vector *v = NULL;
   double dx;
   size_t i;
   v = gsl_vector_alloc(n);
@@ -52,7 +54,8 @@ static gsl_vector* gsl_vector_linspace(const double min, const double max, const
   return v;
 }
 
-static gsl_vector* gsl_vector_logspace(const double min, const double max, const size_t n)
+static gsl_vector* gsl_vector_logspace(const double min, 
+				       const double max, const size_t n)
 {
   gsl_vector *v = NULL;
   double dx;
@@ -137,13 +140,9 @@ VALUE rb_gsl_vector_do_something(VALUE obj, void (*func)(gsl_vector*));
 
 enum {
   GSL_VECTOR_ADD,
-  GSL_VECTOR_ADD2,
   GSL_VECTOR_SUB,
-  GSL_VECTOR_SUB2,
   GSL_VECTOR_MUL,
-  GSL_VECTOR_MUL2,
   GSL_VECTOR_DIV,
-  GSL_VECTOR_DIV2,
 };
 
 static VALUE rb_gsl_vector_arithmetics(int flag, VALUE obj, VALUE bb) 
@@ -160,34 +159,18 @@ static VALUE rb_gsl_vector_arithmetics(int flag, VALUE obj, VALUE bb)
       vnew = make_vector_clone(v);
       gsl_vector_add_constant(vnew, NUM2DBL(bb));
       break;
-    case GSL_VECTOR_ADD2:
-      gsl_vector_add_constant(v, NUM2DBL(bb));
-      return obj;
-      break;
     case GSL_VECTOR_SUB:
       vnew = make_vector_clone(v);
       gsl_vector_add_constant(vnew, -NUM2DBL(bb));
       break;
-    case GSL_VECTOR_SUB2:
-      gsl_vector_add_constant(v, -NUM2DBL(bb));
-      return obj;
-      break;
     case GSL_VECTOR_MUL:
       vnew = make_vector_clone(v);
       gsl_vector_scale(vnew, NUM2DBL(bb));
       break;
-    case GSL_VECTOR_MUL2:
-      gsl_vector_scale(v, NUM2DBL(bb));
-      return obj;
-      break;
     case GSL_VECTOR_DIV:
       vnew = make_vector_clone(v);
       gsl_vector_scale(vnew, 1.0/NUM2DBL(bb));
       break;
-    case GSL_VECTOR_DIV2:
-      gsl_vector_scale(v, 1.0/NUM2DBL(bb));
-      return obj;
-      break;
     }
     if (!VECTOR_VIEW_P(obj)) 
       return Data_Wrap_Struct(CLASS_OF(obj), 0, gsl_vector_free, vnew);
@@ -203,58 +186,37 @@ static VALUE rb_gsl_vector_arithmetics(int flag, VALUE obj, VALUE bb)
 	vnew = make_vector_clone(v);
 	gsl_vector_add(vnew, b);
 	break;
-      case GSL_VECTOR_ADD2:
-	gsl_vector_add(v, b);
-	return obj;
-	break;
       case GSL_VECTOR_SUB:
 	vnew = make_vector_clone(v);
 	gsl_vector_sub(vnew, b);
 	break;
-      case GSL_VECTOR_SUB2:
-	gsl_vector_sub(v, b);
-	return obj;
-	break;
       case GSL_VECTOR_MUL:
 	vnew = make_vector_clone(v);
 	gsl_vector_mul(vnew, b);
 	break;
-      case GSL_VECTOR_MUL2:
-	gsl_vector_mul(v, b);
-	return obj;
-	break;
       case GSL_VECTOR_DIV:
 	vnew = make_vector_clone(v);
 	gsl_vector_div(vnew, b);
 	break;
-      case GSL_VECTOR_DIV2:
-	gsl_vector_div(v, b);
-	return obj;
-	break;
       }
       if (!VECTOR_VIEW_P(obj)) 
 	return Data_Wrap_Struct(CLASS_OF(obj), 0, gsl_vector_free, vnew);
       else 
 	return Data_Wrap_Struct(VECTOR_ROW_COL(obj), 0, gsl_vector_free, vnew);
-      /*      return Data_Wrap_Struct(VECTOR_ROW_COL(obj), 0, gsl_vector_free, vnew);*/
     } else if (VECTOR_COMPLEX_P(bb)) {
       Data_Get_Struct(bb, gsl_vector_complex, cb);
       cvnew = vector_to_complex(v);
       switch (flag) {
       case GSL_VECTOR_ADD:
-      case GSL_VECTOR_ADD2:
 	gsl_vector_complex_add(cvnew, cb);
 	break;
       case GSL_VECTOR_SUB:
-      case GSL_VECTOR_SUB2:
 	gsl_vector_complex_sub(cvnew, cb);
 	break;
       case GSL_VECTOR_MUL:
-      case GSL_VECTOR_MUL2:
 	gsl_vector_complex_mul(cvnew, cb);
 	break;
       case GSL_VECTOR_DIV:
-      case GSL_VECTOR_DIV2:
 	gsl_vector_complex_div(cvnew, cb);
 	break;
       }
@@ -267,19 +229,15 @@ static VALUE rb_gsl_vector_arithmetics(int flag, VALUE obj, VALUE bb)
       cvnew = vector_to_complex(v);
       switch (flag) {
       case GSL_VECTOR_ADD:
-      case GSL_VECTOR_ADD2:
 	gsl_vector_complex_add_constant(cvnew, *c);
 	break;
       case GSL_VECTOR_SUB:
-      case GSL_VECTOR_SUB2:
 	gsl_vector_complex_add_constant(cvnew, gsl_complex_negative(*c));
 	break;
       case GSL_VECTOR_MUL:
-      case GSL_VECTOR_MUL2:
 	gsl_vector_complex_scale(cvnew, *c);
 	break;
       case GSL_VECTOR_DIV:
-      case GSL_VECTOR_DIV2:
 	gsl_vector_complex_scale(cvnew, gsl_complex_inverse(*c));
 	break;
       }
@@ -401,7 +359,7 @@ static VALUE rb_gsl_vector_to_complex(VALUE obj)
   for (i = 0; i < v->size; i++) {
     x = gsl_vector_get(v, i);
     z.dat[0] = x;
-    z.dat[1] = 0.0;
+    z.dat[1] = 0;
     gsl_vector_complex_set(cv, i, z);
   }
   if (VECTOR_COL_P(obj))
@@ -410,6 +368,29 @@ static VALUE rb_gsl_vector_to_complex(VALUE obj)
     return Data_Wrap_Struct(cgsl_vector_complex, 0, gsl_vector_complex_free, cv);
 }
 
+static VALUE rb_gsl_vector_to_complex2(VALUE obj)
+{
+  gsl_vector *v = NULL;
+  gsl_vector_complex *cv = NULL;
+  gsl_complex z;
+  size_t i;
+  double re, im;
+  Data_Get_Struct(obj, gsl_vector, v);
+  cv = gsl_vector_complex_alloc(ceil((double)v->size/2));
+  for (i = 0; i < v->size; i += 2) {
+    re = gsl_vector_get(v, i);
+    if (i+1 == v->size) im = 0.0;
+    else im = gsl_vector_get(v, i+1);
+    z.dat[0] = re;
+    z.dat[1] = im;
+    gsl_vector_complex_set(cv, i/2, z);
+  }
+  if (VECTOR_COL_P(obj))
+    return Data_Wrap_Struct(cgsl_vector_complex_col, 0, gsl_vector_complex_free, cv);
+  else
+    return Data_Wrap_Struct(cgsl_vector_complex, 0, gsl_vector_complex_free, cv);
+}
+
 static VALUE rb_gsl_vector_coerce(VALUE obj, VALUE other)
 {
   gsl_vector *v = NULL, *vnew = NULL;
@@ -522,13 +503,16 @@ static void draw_hist(VALUE obj, FILE *fp);
 static void draw_vector(VALUE obj, FILE *fp);
 static void draw_vector2(VALUE xx, VALUE yy, FILE *fp);
 static void draw_vector_array(VALUE ary, FILE *fp);
+#ifdef HAVE_NARRAY_H
+static void draw_narray(VALUE obj, FILE *fp);
+#endif
 static VALUE rb_gsl_vector_graph2(int argc, VALUE *argv, VALUE obj)
 {
 #ifdef HAVE_GNU_GRAPH
   size_t i, iend, j, n = 0;
   gsl_vector *x = NULL, *y = NULL;
   gsl_histogram *h = NULL;
-  VALUE vx;
+  VALUE vx = (VALUE) NULL;
   char command[1024];
   int flag = 0;
   FILE *fp = NULL;
@@ -552,6 +536,10 @@ static VALUE rb_gsl_vector_graph2(int argc, VALUE *argv, VALUE obj)
       draw_hist(argv[0], fp);
     } else if (TYPE(argv[0]) == T_ARRAY) {
       draw_vector_array(argv[0], fp);
+#ifdef HAVE_NARRAY_H
+    } else if (NA_IsNArray(argv[0])) {
+      draw_narray(argv[0], fp);
+#endif
     } else {
       if (fp) fclose(fp);
       rb_raise(rb_eTypeError, "wrong argument type %s", 
@@ -579,6 +567,12 @@ static VALUE rb_gsl_vector_graph2(int argc, VALUE *argv, VALUE obj)
     } else if (TYPE(argv[0]) == T_ARRAY) {
       draw_vector_array(argv[0], fp);
       fprintf(fp, "\n");
+#ifdef HAVE_NARRAY_H
+    } else if (NA_IsNArray(argv[0])) {
+      vx = argv[0];
+      n = NA_TOTAL(argv[0]);
+      flag = 0;
+#endif
     } else if (NIL_P(argv[0])) {
       if (argc < 2) rb_raise(rb_eArgError, "too few arguments");
       if (VECTOR_P(argv[1])) {
@@ -587,6 +581,10 @@ static VALUE rb_gsl_vector_graph2(int argc, VALUE *argv, VALUE obj)
       } else if (HISTOGRAM_P(argv[1])) {
 	Data_Get_Struct(argv[1], gsl_histogram, h);
 	n = h->n;
+#ifdef HAVE_NARRAY_H
+      } else if (NA_IsNArray(argv[1])) {
+	n = NA_TOTAL(argv[1]);
+#endif
       } else {
 	rb_raise(rb_eTypeError, "wrong argument type %s", 
 		 rb_class2name(CLASS_OF(argv[0])));
@@ -604,6 +602,9 @@ static VALUE rb_gsl_vector_graph2(int argc, VALUE *argv, VALUE obj)
       if (VECTOR_P(argv[i])) draw_vector2(vx, argv[i], fp);
       else if (HISTOGRAM_P(argv[i])) draw_hist(argv[i], fp);
       else if (TYPE(argv[i]) == T_ARRAY) draw_vector_array(argv[i], fp);
+#ifdef HAVE_NARRAY_H
+      else if (NA_IsNArray(argv[i])) draw_vector2(vx, argv[i], fp);
+#endif
       else 
 	rb_raise(rb_eTypeError, "wrong argument type %s", 
 		 rb_class2name(CLASS_OF(argv[i])));
@@ -626,28 +627,75 @@ static void draw_vector(VALUE obj, FILE *fp)
   size_t j;
   Data_Get_Vector(obj, x);
   for (j = 0; j < x->size; j++)
-    fprintf(fp, "%d %e\n", (int) j, gsl_vector_get(x, j));
+    fprintf(fp, "%d %g\n", (int) j, gsl_vector_get(x, j));
   fflush(fp);
 }
 
 static void draw_vector2(VALUE xx, VALUE yy, FILE *fp)
 {
-  gsl_vector *x = NULL, *y = NULL;
-  size_t j;
-  Data_Get_Vector(xx, x);
-  Data_Get_Vector(yy, y);
-  for (j = 0; j < x->size; j++)
-    fprintf(fp, "%e %e\n", gsl_vector_get(x, j), gsl_vector_get(y, j));
+#ifdef HAVE_NARRAY_H
+  struct NARRAY *nax, *nay;
+#endif
+  double *ptr1 = NULL, *ptr2 = NULL;
+  gsl_vector *vx, *vy;
+  size_t j, n, stridex = 1, stridey = 1;
+  if (VECTOR_P(xx)) {
+    Data_Get_Struct(xx, gsl_vector, vx);
+    ptr1 = vx->data;
+    n = vx->size;
+    stridex = vx->stride;
+#ifdef HAVE_NARRAY_H
+  } else if (NA_IsNArray(xx)) {
+    GetNArray(xx, nax);
+    ptr1 = (double *) nax->ptr;
+    n = nax->total;
+    stridex = 1;
+#endif
+  } else {
+    rb_raise(rb_eTypeError, "wrong argument type %s (Vector expected)",
+	     rb_class2name(CLASS_OF(xx)));
+  }
+  if (VECTOR_P(yy)) {
+    Data_Get_Struct(yy, gsl_vector, vy);
+    ptr2 = vy->data;
+    n = vy->size;
+    stridey = vy->stride;
+#ifdef HAVE_NARRAY_H
+  } else if (NA_IsNArray(yy)) {
+    GetNArray(yy, nay);
+    ptr2 = (double *) nay->ptr;
+    stridey = 1;
+#endif
+  } else {
+    rb_raise(rb_eTypeError, "wrong argument type %s (Vector expected)",
+	     rb_class2name(CLASS_OF(yy)));
+  }
+  for (j = 0; j < n; j++)
+    fprintf(fp, "%g %g\n", ptr1[j*stridex], ptr2[j*stridey]);
   fflush(fp);
 }
 
+#ifdef HAVE_NARRAY_H
+static void draw_narray(VALUE obj, FILE *fp)
+{
+  struct NARRAY *na;
+  double *ptr;
+  size_t j;
+  GetNArray(obj, na);
+  ptr = (double *) na->ptr;
+  for (j = 0; j < na->total; j++)
+    fprintf(fp, "%d %g\n", (int) j, ptr[j]);
+  fflush(fp);
+}
+#endif
+
 static void draw_hist(VALUE obj, FILE *fp)
 {
   gsl_histogram *h = NULL;
   size_t j;
   Data_Get_Struct(obj, gsl_histogram, h);
   for (j = 0; j < h->n; j++) {
-    fprintf(fp, "%e %e\n%e %e\n", 
+    fprintf(fp, "%g %g\n%g %g\n", 
 	    h->range[j], h->bin[j], h->range[j+1], h->bin[j]);
   }
   fflush(fp);
@@ -655,24 +703,31 @@ static void draw_hist(VALUE obj, FILE *fp)
 
 static void draw_vector_array(VALUE ary, FILE *fp)
 {
-  gsl_vector *x = NULL, *y = NULL;
+  double *ptrx = NULL, *ptry = NULL, *ptrz = NULL;
   VALUE vx;
-  size_t j, n;
+  size_t j, n, stridex, stridey, stridez;
   int flag = 0;
   switch (RARRAY(ary)->len) {
   case 1:
     flag = 1;
-    Data_Get_Vector(rb_ary_entry(ary, 0), y);
-    n = y->size;
+    ptry = get_vector_ptr(rb_ary_entry(ary, 0), &stridey, &n);
     break;
   case 2:
-    Data_Get_Vector(rb_ary_entry(ary, 1), y);
-    n = y->size;
+    ptry = get_vector_ptr(rb_ary_entry(ary, 1), &stridey, &n);
     vx = rb_ary_entry(ary, 0);
     if (NIL_P(vx)) {flag = 1;}
     else {
-      Data_Get_Vector(vx, x);
-      n = x->size;
+      ptrx = get_vector_ptr(vx, &stridex, &n);
+    }
+    break;
+  case 3:
+    ptrz = get_vector_ptr(rb_ary_entry(ary, 2), &stridez, &n);
+    ptry = get_vector_ptr(rb_ary_entry(ary, 1), &stridey, &n);
+    vx = rb_ary_entry(ary, 0);
+    if (NIL_P(vx)) {flag = 2;}
+    else {
+      ptrx = get_vector_ptr(vx, &stridex, &n);
+      flag = 3;
     }
     break;
   default:
@@ -680,12 +735,25 @@ static void draw_vector_array(VALUE ary, FILE *fp)
 	     RARRAY(ary)->len);
     break;
   }
-  if (flag == 0) {
+  switch (flag) {
+  case 0:
     for (j = 0; j < n; j++) 
-      fprintf(fp, "%e %e\n", gsl_vector_get(x, j), gsl_vector_get(y, j));
-  } else {
+      fprintf(fp, "%g %g\n", ptrx[j*stridex], ptry[j*stridey]);
+    break;
+  case 1:
     for (j = 0; j < n; j++) 
-      fprintf(fp, "%d %e\n", (int) j, gsl_vector_get(y, j));
+      fprintf(fp, "%d %g\n", (int) j, ptry[j*stridey]);
+    break;
+  case 2:
+    for (j = 0; j < n; j++) 
+      fprintf(fp, "%d %g %g\n", (int) j, ptry[j*stridey], ptrz[j*stridez]);
+    break;
+  case 3:
+    for (j = 0; j < n; j++) 
+      fprintf(fp, "%g %g %g\n", ptrx[j*stridex], ptry[j*stridey], ptrz[j*stridez]);
+    break;
+  default:
+    break;
   }
   fflush(fp);
 }
@@ -754,14 +822,14 @@ static VALUE rb_gsl_vector_plot2(int argc, VALUE *argv, VALUE obj)
   fprintf(fp, "%s\n", command);
   for (i = 0; i < n; i++) {
     if (y == NULL) 
-      fprintf(fp, "%d %e\n", (int) i, gsl_vector_get(x, i));
+      fprintf(fp, "%d %g\n", (int) i, gsl_vector_get(x, i));
     else if (yerr == NULL)
-      fprintf(fp, "%e %e\n", gsl_vector_get(x, i), gsl_vector_get(y, i));
+      fprintf(fp, "%g %g\n", gsl_vector_get(x, i), gsl_vector_get(y, i));
     else if (xerr) 
-      fprintf(fp, "%e %e %e %e\n", gsl_vector_get(x, i), gsl_vector_get(y, i),
+      fprintf(fp, "%g %g %g %g\n", gsl_vector_get(x, i), gsl_vector_get(y, i),
 	      gsl_vector_get(xerr, i), gsl_vector_get(yerr, i));
     else
-     fprintf(fp, "%e %e %e\n", gsl_vector_get(x, i), gsl_vector_get(y, i),
+     fprintf(fp, "%g %g %g\n", gsl_vector_get(x, i), gsl_vector_get(y, i),
 	     gsl_vector_get(yerr, i));
   }
   fprintf(fp, "e\n");
@@ -826,6 +894,110 @@ static VALUE rb_gsl_vector_normalize_bang(int argc, VALUE *argv, VALUE obj)
   return obj;
 }
 
+#ifdef HAVE_NARRAY_H
+static VALUE rb_gsl_vector_filescan_na(VALUE klass, VALUE file)
+{
+  FILE *fp = NULL;
+  int nn;
+  char buf[1024], filename[1024], *p;
+  size_t n, lines, i, j;
+  double **ptr;
+  double val;
+  int shape[1];
+  VALUE ary, na;
+  Check_Type(file, T_STRING);
+  strcpy(filename, STR2CSTR(file));
+  sprintf(buf, "wc %s", filename);
+  fp = popen(buf, "r");
+  fgets(buf, 1024, fp);
+  fclose(fp);
+  sscanf(buf, "%d", &nn);
+  lines = (size_t) nn;      /* vector length */
+  shape[0] = lines;
+  fp = fopen(filename, "r");
+  fgets(buf, 1024, fp);
+  n = count_columns(buf);   /* number of vectors created */
+  ptr = (double**) xmalloc(sizeof(double**)*n);
+  ary = rb_ary_new2(n);
+  p = buf;
+  for (j = 0; j < n; j++) {
+    na = na_make_object(NA_DFLOAT, 1, shape, cNArray);
+    rb_ary_store(ary, j, na);
+    ptr[j] = NA_PTR_TYPE(na, double*);
+    p = str_scan_double(p, &val);
+    if (p) ptr[j][0] = val;
+    else break;
+  }
+  for (i = 1; i < lines; i++) {
+    fgets(buf, 1024, fp);
+    p = buf;
+    for (j = 0; j < n; j++) {
+      p = str_scan_double(p, &val);
+      if (p) ptr[j][i] = val;
+      else break;      
+    }
+  }
+  fclose(fp);
+  free(ptr);
+  return ary;
+}
+#endif
+
+static VALUE rb_gsl_vector_decimate(VALUE obj, VALUE nn)
+{
+  gsl_vector *v = NULL, *vnew = NULL;
+  gsl_vector_view vv;
+  size_t i, n, n2, n3;
+  CHECK_FIXNUM(nn);
+  Data_Get_Vector(obj, v);
+  n = (size_t) FIX2INT(nn);
+  if (n > v->size) 
+    rb_raise(rb_eArgError, 
+	     "decimation factor must be smaller than the vector length.");
+  if (n == 0) rb_raise(rb_eArgError, "decimation factor must be greater than 1");
+  n2 = (size_t) ceil((double)v->size/n);
+  vnew = gsl_vector_alloc(n2);
+  n3 = n - (n2*n - v->size);
+  for (i = 0; i < n2; i++) {
+    if (i == n2-1) vv = gsl_vector_subvector(v, i*n, n3);
+    else vv = gsl_vector_subvector(v, i*n, n);
+    gsl_vector_set(vnew, i, gsl_stats_mean(vv.vector.data, vv.vector.stride,
+					   vv.vector.size));
+  }
+  return Data_Wrap_Struct(VECTOR_ROW_COL(obj), 0, gsl_vector_free, vnew);
+}
+
+static VALUE rb_gsl_vector_xxx(VALUE obj, double (*f)(double))
+{
+  gsl_vector *v;
+  gsl_vector_int *vnew;
+  size_t i;
+  Data_Get_Struct(obj, gsl_vector, v);
+  vnew = gsl_vector_int_alloc(v->size);
+  for (i = 0; i < v->size; i++) {
+    gsl_vector_int_set(vnew, i, (int) (*f)(gsl_vector_get(v, i)));
+  }
+  return Data_Wrap_Struct(cgsl_vector_int, 0, gsl_vector_int_free, vnew);
+}
+
+static VALUE rb_gsl_vector_floor(VALUE obj)
+{
+  return rb_gsl_vector_xxx(obj, floor);
+}
+
+static VALUE rb_gsl_vector_ceil(VALUE obj)
+{
+  return rb_gsl_vector_xxx(obj, ceil);
+}
+
+#ifdef HAVE_ROUND
+double round(double);
+static VALUE rb_gsl_vector_round(VALUE obj)
+{
+  return rb_gsl_vector_xxx(obj, round);
+}
+#endif
+
 void Init_gsl_vector(VALUE module)
 {
   rb_define_singleton_method(cgsl_vector, "linspace", rb_gsl_vector_linspace, -1);
@@ -845,6 +1017,7 @@ void Init_gsl_vector(VALUE module)
   rb_define_alias(cgsl_vector, "/", "div");
 
   rb_define_method(cgsl_vector, "to_complex", rb_gsl_vector_to_complex, 0);
+  rb_define_method(cgsl_vector, "to_complex2", rb_gsl_vector_to_complex2, 0);
 
   /*****/
 
@@ -874,5 +1047,18 @@ void Init_gsl_vector(VALUE module)
   rb_define_method(cgsl_vector, "normalize", rb_gsl_vector_normalize, -1);
   rb_define_method(cgsl_vector, "normalize!", rb_gsl_vector_normalize_bang, -1);
 
+#ifdef HAVE_NARRAY_H
+  rb_define_singleton_method(cgsl_vector, "filescan_na", rb_gsl_vector_filescan_na, 1);
+  rb_define_singleton_method(cNArray, "filescan", rb_gsl_vector_filescan_na, 1);
+#endif
+
+  rb_define_method(cgsl_vector, "decimate", rb_gsl_vector_decimate, 1);
+
+  rb_define_method(cgsl_vector, "floor", rb_gsl_vector_floor, 0);
+  rb_define_method(cgsl_vector, "ceil", rb_gsl_vector_ceil, 0);
+#ifdef HAVE_ROUND
+  rb_define_method(cgsl_vector, "round", rb_gsl_vector_round, 0);
+#endif
+
   Init_gsl_vector_init(module);
 }
