@@ -2,7 +2,7 @@
 #define ___RB_GSL_CONFIG_H___
 
 #ifndef GSL_VERSION
-#define GSL_VERSION "1.5"
+#define GSL_VERSION "1.6"
 #endif
 #ifndef GSL_0_9_4_LATER
 #define GSL_0_9_4_LATER
@@ -28,8 +28,11 @@
 #ifndef GSL_1_4_9_LATER
 #define GSL_1_4_9_LATER
 #endif
+#ifndef GSL_1_6_LATER
+#define GSL_1_6_LATER
+#endif
 #ifndef RUBY_GSL_VERSION
-#define RUBY_GSL_VERSION "1.5.4"
+#define RUBY_GSL_VERSION "1.6.2-preview3"
 #endif
 #ifndef RUBY_1_8_LATER
 #define RUBY_1_8_LATER
