@@ -8,5 +8,5 @@ c = Matrix.new(2, 2)
 
 p Blas.dgemm(NoTrans, NoTrans, 1, a, b, 1, c)
 p Blas.dgemm(a, b)
-p a**b
+p a*b
 
