@@ -11,9 +11,8 @@ for i in 1..10 do
   data[n-i] = [1.0, 0.0]
 end
 
-datasave = data.clone
-data.radix2_forward()
-data /= Math::sqrt(n)
+ffted = data.radix2_forward
+ffted /= Math::sqrt(n)
 
-graph(nil, datasave.re, data.re, "-T X -C -g 3 -L 'Radix-2' -x 0 #{data.size/2}")
+graph(nil, data.re, ffted.re, "-T X -C -g 3 -L 'Radix-2' -x 0 #{data.size/2}")
 
