@@ -1,15 +1,14 @@
 #!/usr/bin/env ruby
 require("gsl")
 
+N = 10000
 rng = Rng.alloc
+data = rng.exponential(2, N)
 h = Histogram.alloc(100, [0, 15])
-for i in 0...10000 do
-#  r = rng.exponential(2)
-  r = Ran::exponential(rng, 2)
-  h.fill(r)
-end
+h.fill(data)
 
-result = h.fit_exponential
+#result = h.fit_exponential
+result = h.fit("exponential")
 a = result[0]
 b = result[1]
 
