@@ -24,5 +24,5 @@ a.scale!(1.0/a[BINS/2])
 
 hi.normalize!                  # this is equivalent to hi.scale(1.0/hi[BINS-1])
 
-GSL::graph(h, hi,a,  "-T X -C")
+GSL::graph(h, hi,a,  "-T X -C -g 3")
 
