@@ -11,9 +11,11 @@ f = Function.new { |x|
   atan(sqrtx22)/(sqrtx22*(x*x + 1))
 }
 
+val = f.qng(0, 1)[0]
+
 puts("Ahmed's integral")
 puts("Expect: 5pi^2/96 = #{5.0*M_PI*M_PI/96}")
-puts("QNG result: #{f.qng(0, 1)[0]}")
+puts("QNG result:        #{val}")
 
 
  
