@@ -4,6 +4,8 @@ require("gsl")
 n = 10
 x = Vector[n]
 y = Vector[n]
+#x = NArray.float(n)
+#y = NArray.float(n)
 File.open("data0.dat", "w") do |f|
   for i in 0...n do
     a = i.to_f
@@ -14,10 +16,10 @@ File.open("data0.dat", "w") do |f|
 end
 
 interp = Interp.new("akima", n)
-#p interp.class
-#p interp.name
-
 interp.init(x, y)
+#interp = Interp.alloc(x, y)
+p interp.class
+p interp.name
 
 File.open("data1.dat", "w") do |f|
   xi = x[0]
@@ -28,7 +30,7 @@ File.open("data1.dat", "w") do |f|
   end
 end
 
-system("graph -T X -g 3 -C -L 'GSL::Interp, Akima' -m -1 -S 4 data0.dat -S 1 data1.dat")
+system("graph -T X -g 3 -C -m -1 -S 4 data0.dat -S 1 data1.dat")
 File.delete("data0.dat")
 File.delete("data1.dat")
 
