@@ -10,7 +10,7 @@ lu, perm = m.LU_decomp
 m.print
 lu.print
 
-b = Vector.new(1, 2, 3, 4)
+b = Vector[1, 2, 3, 4]
 x = LU.solve(lu, perm, b)
 x = LU.solve(lu, perm, b)
 p x
