@@ -20,25 +20,25 @@ zm.set_row(3, [0.51, 0], [0.13, 0], [0.19, 0], [0.85, 0])
 
 p zm.inv
 p zm.det
-#exit
 
 b = Vector::Complex.new(4)
 b.set(1.0, 2, 3, 4)
 
 #p zm.LU_solve(b)
-#exit
 
 #lu2, perm2, signum = zm.LU_decomp
 lu2, perm2, signum = LU::decomp(zm)
 p lu2
 p lu2.det(signum)
-#exit
-p lu2.solve(perm2, b)
+
+p x = lu2.solve(perm2, b)
+
+p zm*x
 
 p lu2.LU_invert(perm2)
 p lu2.LU_det(signum)
 
-p zm.LU_svx(b)
+#p zm.LU_svx(b)
 
 p Linalg::Complex::LU_invert(zm)
 p Linalg::Complex::LU::invert(zm)
