@@ -22,3 +22,8 @@ begin
   printf("%5d [%.7f, %.7f] %.7f %+.7f %.7f\n",
          iter, a, b, m, m - m_expected, b - a);
 end while status == GSL::CONTINUE and iter < max_iter
+
+x = Vector.linspace(0, 6, 50)
+mx = gmf.x_minimum
+min = fn1.eval(mx)
+graph([x, fn1.eval(x)], [Vector[mx], Vector[min]], "-C -g 3 -S 4 -m -1")
