@@ -21,7 +21,7 @@ df = Proc.new { |x|
 expected = sqrt(5.0)
 x = 5.0
 
-function_fdf = Root::Function_fdf.new(f, df)
+function_fdf = Function_fdf.new(f, df)
 
 solver = Root::FdfSolver.new(Root::FdfSolver::STEFFENSON)
 puts "using #{solver.name} method"
