@@ -2,4 +2,4 @@
 require("gsl")
 x = Vector.linspace(0.01, 14, 50)
 y = Sf::lngamma(x)
-y.graph(x, "-T X -C -g 3 -l y -L 'lngamma(x)'")
+graph(x, y, "-T X -C -g 3 -l y -L 'lngamma(x)'")
